@@ -42,13 +42,14 @@ type InitRequest struct {
 
 // EnvInitRequest contains the parameters for calling copilot env init.
 type EnvInitRequest struct {
-	AppName       string
-	EnvName       string
-	Profile       string
-	Prod          bool
-	CustomizedEnv bool
-	VPCImport     EnvInitRequestVPCImport
-	VPCConfig     EnvInitRequestVPCConfig
+	AppName           string
+	EnvName           string
+	Profile           string
+	Prod              bool
+	CustomizedEnv     bool
+	VPCImport         EnvInitRequestVPCImport
+	VPCConfig         EnvInitRequestVPCConfig
+	CertificateImport string
 }
 
 // EnvInitRequestVPCImport contains the parameters for configuring VPC import when
@@ -73,6 +74,12 @@ type EnvInitRequestVPCConfig struct {
 	PrivateSubnetCIDRs string
 }
 
+// EnvDeployRequest contains the parameters for calling copilot env deploy.
+type EnvDeployRequest struct {
+	AppName string
+	Name    string
+}
+
 // EnvShowRequest contains the parameters for calling copilot env show.
 type EnvShowRequest struct {
 	AppName string
@@ -220,6 +227,7 @@ type PipelineInitInput struct {
 	URL          string
 	GitBranch    string
 	Environments []string
+	Type         string
 }
 
 // PipelineDeployInput contains the parameters for calling copilot pipeline deploy.
@@ -579,6 +587,9 @@ func (cli *CLI) EnvInit(opts *EnvInitRequest) (string, error) {
 		"--app", opts.AppName,
 		"--profile", opts.Profile,
 	}
+	if opts.CertificateImport != "" {
+		commands = append(commands, "--import-cert-arns", opts.CertificateImport)
+	}
 	if opts.Prod {
 		commands = append(commands, "--prod")
 	}
@@ -598,6 +609,19 @@ func (cli *CLI) EnvInit(opts *EnvInitRequest) (string, error) {
 	return cli.exec(exec.Command(cli.path, commands...))
 }
 
+/*EnvDeploy runs:
+copilot env deploy
+	--name $n
+	--app $a
+*/
+func (cli *CLI) EnvDeploy(opts *EnvDeployRequest) (string, error) {
+	commands := []string{"env", "deploy",
+		"--name", opts.Name,
+		"--app", opts.AppName,
+	}
+	return cli.exec(exec.Command(cli.path, commands...))
+}
+
 /*EnvShow runs:
 copilot env show
 	--app $a
@@ -686,6 +710,7 @@ func (cli *CLI) PipelineInit(opts PipelineInitInput) (string, error) {
 		"--url", opts.URL,
 		"--git-branch", opts.GitBranch,
 		"--environments", strings.Join(opts.Environments, ","),
+		"--pipeline-type", opts.Type,
 	}
 
 	return cli.exec(exec.Command(cli.path, args...))
