@@ -72,10 +72,17 @@ var _ = Describe("Multiple Env App", func() {
 		})
 
 		It("env init should succeed for test and prod envs", func() {
+			fmt.Println(testEnvInitErr)
+			fmt.Println(prodEnvInitErr)
 			Expect(testEnvInitErr).NotTo(HaveOccurred())
 			Expect(prodEnvInitErr).NotTo(HaveOccurred())
 		})
 
+		It("should create environment manifests", func() {
+			Expect("./copilot/environments/test/manifest.yml").Should(BeAnExistingFile())
+			Expect("./copilot/environments/prod/manifest.yml").Should(BeAnExistingFile())
+		})
+
 		It("env ls should list both envs", func() {
 			envListOutput, err := cli.EnvList(appName)
 			Expect(err).NotTo(HaveOccurred())
@@ -100,6 +107,47 @@ var _ = Describe("Multiple Env App", func() {
 				Expect(envs["test"].Account).NotTo(Equal(envs["prod"].Account))
 			}
 		})
+
+		It("should show only bootstrap resources in env show", func() {
+			testEnvShowOutput, testEnvShowError := cli.EnvShow(&client.EnvShowRequest{
+				AppName: appName,
+				EnvName: "test",
+			})
+			prodEnvShowOutput, prodEnvShowError := cli.EnvShow(&client.EnvShowRequest{
+				AppName: appName,
+				EnvName: "prod",
+			})
+			Expect(testEnvShowError).NotTo(HaveOccurred())
+			Expect(prodEnvShowError).NotTo(HaveOccurred())
+
+			Expect(testEnvShowOutput.Environment.Name).To(Equal("test"))
+			Expect(testEnvShowOutput.Environment.App).To(Equal(appName))
+			Expect(prodEnvShowOutput.Environment.Name).To(Equal("prod"))
+			Expect(prodEnvShowOutput.Environment.App).To(Equal(appName))
+
+			// Contains only bootstrap resources - two IAM roles.
+			Expect(len(testEnvShowOutput.Resources)).To(Equal(2))
+			Expect(len(prodEnvShowOutput.Resources)).To(Equal(2))
+		})
+	})
+
+	Context("when deploying the environments", func() {
+		var testEnvDeployErr, prodEnvDeployErr error
+		BeforeAll(func() {
+			_, testEnvDeployErr = cli.EnvDeploy(&client.EnvDeployRequest{
+				AppName: appName,
+				Name:    "test",
+			})
+			_, prodEnvDeployErr = cli.EnvDeploy(&client.EnvDeployRequest{
+				AppName: appName,
+				Name:    "prod",
+			})
+		})
+
+		It("should succeed", func() {
+			Expect(testEnvDeployErr).NotTo(HaveOccurred())
+			Expect(prodEnvDeployErr).NotTo(HaveOccurred())
+		})
 	})
 
 	Context("when adding a svc", func() {
