@@ -46,7 +46,7 @@ var _ = Describe("Multiple Service App", func() {
 		})
 	})
 
-	Context("when creating a new environment", func() {
+	Context("when adding a new environment", func() {
 		var (
 			testEnvInitErr error
 		)
@@ -64,6 +64,20 @@ var _ = Describe("Multiple Service App", func() {
 		})
 	})
 
+	Context("when deploying the environment", func() {
+		var envDeployErr error
+		BeforeAll(func() {
+			_, envDeployErr = cli.EnvDeploy(&client.EnvDeployRequest{
+				AppName: appName,
+				Name:    "test",
+			})
+		})
+
+		It("should succeed", func() {
+			Expect(envDeployErr).NotTo(HaveOccurred())
+		})
+	})
+
 	Context("when adding a svc", func() {
 		var (
 			frontEndInitErr error
@@ -95,7 +109,7 @@ var _ = Describe("Multiple Service App", func() {
 			_, jobInitErr = cli.JobInit(&client.JobInitInput{
 				Name:       "query",
 				Dockerfile: "./query/Dockerfile",
-				Schedule:   "@every 4m", // This should run once, immediately after creation, then every 4m thereafter.
+				Schedule:   "@every 4m",
 			})
 		})
 
