@@ -96,7 +96,7 @@ var _ = Describe("sidecars flow", func() {
 		})
 	})
 
-	Context("when creating a new environment", func() {
+	Context("when adding a new environment", func() {
 		var (
 			testEnvInitErr error
 		)
@@ -114,6 +114,20 @@ var _ = Describe("sidecars flow", func() {
 		})
 	})
 
+	Context("when deploying the environment", func() {
+		var envDeployErr error
+		BeforeAll(func() {
+			_, envDeployErr = cli.EnvDeploy(&client.EnvDeployRequest{
+				AppName: appName,
+				Name:    "test",
+			})
+		})
+
+		It("should succeed", func() {
+			Expect(envDeployErr).NotTo(HaveOccurred())
+		})
+	})
+
 	Context("when creating a service", func() {
 		var (
 			svcInitErr error
