@@ -37,7 +37,7 @@ var _ = Describe("Worker Service E2E Test", func() {
 		})
 	})
 
-	Context("create an environment", func() {
+	Context("add an environment", func() {
 		It("env init should succeed", func() {
 			_, err := cli.EnvInit(&client.EnvInitRequest{
 				AppName: appName,
@@ -48,6 +48,16 @@ var _ = Describe("Worker Service E2E Test", func() {
 		})
 	})
 
+	Context("when deploying the environment", func() {
+		It("env deploy should succeed", func() {
+			_, err := cli.EnvDeploy(&client.EnvDeployRequest{
+				AppName: appName,
+				Name:    envName,
+			})
+			Expect(err).NotTo(HaveOccurred())
+		})
+	})
+
 	Context("create a load balanced web service", func() {
 		It("svc init should succeed", func() {
 			_, err := cli.SvcInit(&client.SvcInitRequest{
