@@ -11,6 +11,8 @@ import (
 	"io"
 	"strings"
 
+	"github.com/aws/aws-sdk-go/aws/endpoints"
+
 	"github.com/aws/aws-sdk-go/aws/awserr"
 
 	"github.com/aws/aws-sdk-go/aws"
@@ -151,8 +153,38 @@ func ParseURL(url string) (bucket string, key string, err error) {
 	if len(parsedURL) != 2 {
 		return "", "", fmt.Errorf("cannot parse S3 URL %s into bucket name and key", url)
 	}
-	bucket, key = strings.Split(parsedURL[0], ".")[0], parsedURL[1]
-	return
+
+	// go through the host backwards and find the first piece that
+	// starts with 's3' - the rest of the host (to the left of 's3')
+	// is the bucket name. this is to support both URL host formats:
+	// <bucket>.s3-<region>.amazonaws.com and <bucket>.s3.<region>.amazonaws.com
+	split := strings.Split(parsedURL[0], ".")
+	bucketEndIdx := len(split) - 1
+	for ; bucketEndIdx > 0; bucketEndIdx-- {
+		if strings.HasPrefix(split[bucketEndIdx], "s3") {
+			break
+		}
+	}
+
+	return strings.Join(split[:bucketEndIdx], "."), parsedURL[1], nil
+}
+
+// URL returns a virtual-hosted–style S3 url for the object stored at key in a bucket created in the specified region.
+func URL(region, bucket, key string) string {
+	tld := "com"
+	for cn := range endpoints.AwsCnPartition().Regions() {
+		if cn == region {
+			tld = "cn"
+			break
+		}
+	}
+
+	return fmt.Sprintf("https://%s.s3.%s.amazonaws.%s/%s", bucket, region, tld, key)
+}
+
+// Location returns an S3 object location URI in the format "s3://bucket/key".
+func Location(bucket, key string) string {
+	return fmt.Sprintf("s3://%s/%s", bucket, key)
 }
 
 // FormatARN formats an S3 object ARN.
