@@ -392,6 +392,16 @@ func TestS3_ParseURL(t *testing.T) {
 			wantedBucketName: "stackset-myapp-infrastru-pipelinebuiltartifactbuc-1nk5t9zkymh8r",
 			wantedKey:        "scripts/dns-cert-validator/dd2278811c3",
 		},
+		"success with dots": {
+			inURL:            "https://bucket.with.dots.in.name.s3.us-west-2.amazonaws.com/scripts/dns-cert-validator/dd2278811c3",
+			wantedBucketName: "bucket.with.dots.in.name",
+			wantedKey:        "scripts/dns-cert-validator/dd2278811c3",
+		},
+		"success with dots legacy URL": {
+			inURL:            "https://bucket.with.dots.in.name.s3-us-west-2.amazonaws.com/scripts/dns-cert-validator/dd2278811c3",
+			wantedBucketName: "bucket.with.dots.in.name",
+			wantedKey:        "scripts/dns-cert-validator/dd2278811c3",
+		},
 	}
 
 	for name, tc := range testCases {
@@ -402,13 +412,45 @@ func TestS3_ParseURL(t *testing.T) {
 				require.EqualError(t, gotErr, tc.wantError.Error())
 			} else {
 				require.Equal(t, gotErr, nil)
-				require.Equal(t, gotBucketName, tc.wantedBucketName)
-				require.Equal(t, gotKey, tc.wantedKey)
+				require.Equal(t, tc.wantedBucketName, gotBucketName)
+				require.Equal(t, tc.wantedKey, gotKey)
 			}
 		})
 	}
 }
 
+func TestURL(t *testing.T) {
+	testCases := map[string]struct {
+		region string
+		bucket string
+		key    string
+
+		wanted string
+	}{
+		// See https://docs.aws.amazon.com/AmazonS3/latest/userguide/access-bucket-intro.html#virtual-host-style-url-ex
+		"Formats a virtual-hosted-style URL": {
+			region: "us-west-2",
+			bucket: "mybucket",
+			key:    "puppy.jpg",
+
+			wanted: "https://mybucket.s3.us-west-2.amazonaws.com/puppy.jpg",
+		},
+		"Formats the URL for a region in the aws-cn partition": {
+			region: "cn-north-1",
+			bucket: "mybucket",
+			key:    "puppy.jpg",
+
+			wanted: "https://mybucket.s3.cn-north-1.amazonaws.cn/puppy.jpg",
+		},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			require.Equal(t, tc.wanted, URL(tc.region, tc.bucket, tc.key))
+		})
+	}
+}
+
 func TestS3_FormatARN(t *testing.T) {
 	testCases := map[string]struct {
 		inPartition string
