@@ -8,8 +8,11 @@ import (
 	"fmt"
 	"net"
 	"os"
+	"path/filepath"
 	"strings"
 
+	"github.com/aws/copilot-cli/internal/pkg/manifest"
+	"github.com/aws/copilot-cli/internal/pkg/workspace"
 	"github.com/dustin/go-humanize/english"
 
 	"github.com/aws/aws-sdk-go/service/ssm"
@@ -28,7 +31,6 @@ import (
 	"github.com/aws/copilot-cli/internal/pkg/deploy"
 	deploycfn "github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation"
 	"github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation/stack"
-	"github.com/aws/copilot-cli/internal/pkg/template"
 	"github.com/aws/copilot-cli/internal/pkg/term/color"
 	"github.com/aws/copilot-cli/internal/pkg/term/log"
 	termprogress "github.com/aws/copilot-cli/internal/pkg/term/progress"
@@ -39,8 +41,6 @@ import (
 )
 
 const (
-	envInitAppNameHelpPrompt = "An environment will be created in the selected application."
-
 	envInitNamePrompt              = "What is your environment's name?"
 	envInitNameHelpPrompt          = "A unique identifier for an environment (e.g. dev, test, prod)."
 	envInitDefaultEnvConfirmPrompt = `Would you like to use the default configuration for a new environment?
@@ -77,7 +77,6 @@ https://aws.github.io/copilot-cli/docs/credentials/#environment-credentials`
 )
 
 var (
-	envInitAppNamePrompt                  = fmt.Sprintf("In which %s would you like to create the environment?", color.Emphasize("application"))
 	envInitDefaultConfigSelectOption      = "Yes, use default."
 	envInitAdjustEnvResourcesSelectOption = "Yes, but I'd like configure the default resources (CIDR ranges, AZs)."
 	envInitImportEnvResourcesSelectOption = "No, I'd like to import existing resources (VPC, subnets)."
@@ -146,7 +145,7 @@ type initEnvVars struct {
 	importVPC          importVPCVars // Existing VPC resources to use instead of creating new ones.
 	adjustVPC          adjustVPCVars // Configure parameters for VPC resources generated while initializing an environment.
 	telemetry          telemetryVars // Configure observability and monitoring settings.
-	importCerts        []string      // Addtional existing ACM certificates to use.
+	importCerts        []string      // Additional existing ACM certificates to use.
 	internalALBSubnets []string      // Subnets to be used for internal ALB placement.
 	allowVPCIngress    bool          // True means the env stack will create ingress to the internal ALB from ports 80/443.
 
@@ -158,25 +157,28 @@ type initEnvOpts struct {
 	initEnvVars
 
 	// Interfaces to interact with dependencies.
-	sessProvider sessionProvider
-	store        store
-	envDeployer  deployer
-	appDeployer  deployer
-	identity     identityService
-	envIdentity  identityService
-	ec2Client    ec2Client
-	iam          roleManager
-	cfn          stackExistChecker
-	prog         progress
-	prompt       prompter
-	selVPC       ec2Selector
-	selCreds     credsSelector
-	selApp       appSelector
-	appCFN       appResourcesGetter
-	newS3        func(string) (uploader, error)
-	uploader     customResourcesUploader
+	sessProvider   sessionProvider
+	store          store
+	envDeployer    deployer
+	appDeployer    deployer
+	identity       identityService
+	envIdentity    identityService
+	ec2Client      ec2Client
+	iam            roleManager
+	cfn            stackExistChecker
+	prog           progress
+	prompt         prompter
+	selVPC         ec2Selector
+	selCreds       credsSelector
+	selApp         appSelector
+	appCFN         appResourcesGetter
+	manifestWriter environmentManifestWriter
 
 	sess *session.Session // Session pointing to environment's AWS account and region.
+
+	// Cached variables.
+	wsAppName        string
+	mftDisplayedPath string
 }
 
 func newInitEnvOpts(vars initEnvVars) (*initEnvOpts, error) {
@@ -193,11 +195,16 @@ func newInitEnvOpts(vars initEnvVars) (*initEnvOpts, error) {
 	}
 
 	prompter := prompt.New()
+
+	ws, err := workspace.New()
+	if err != nil {
+		return nil, fmt.Errorf("create workspace: %w", err)
+	}
 	return &initEnvOpts{
 		initEnvVars:  vars,
 		sessProvider: sessProvider,
 		store:        store,
-		appDeployer:  deploycfn.New(defaultSession),
+		appDeployer:  deploycfn.New(defaultSession, deploycfn.WithProgressTracker(os.Stderr)),
 		identity:     identity.New(defaultSession),
 		prog:         termprogress.NewSpinner(log.DiagnosticWriter),
 		prompt:       prompter,
@@ -206,21 +213,21 @@ func newInitEnvOpts(vars initEnvVars) (*initEnvOpts, error) {
 			Profile: cfg,
 			Prompt:  prompter,
 		},
-		selApp:   selector.NewAppEnvSelector(prompt.New(), store),
-		uploader: template.New(),
-		appCFN:   deploycfn.New(defaultSession),
-		newS3: func(region string) (uploader, error) {
-			sess, err := sessProvider.DefaultWithRegion(region)
-			if err != nil {
-				return nil, err
-			}
-			return s3.New(sess), nil
-		},
+		selApp:         selector.NewAppEnvSelector(prompt.New(), store),
+		appCFN:         deploycfn.New(defaultSession, deploycfn.WithProgressTracker(os.Stderr)),
+		manifestWriter: ws,
+
+		wsAppName: tryReadingAppName(),
 	}, nil
 }
 
 // Validate returns an error if the values passed by flags are invalid.
 func (o *initEnvOpts) Validate() error {
+	if err := validateWorkspaceApp(o.wsAppName, o.appName, o.store); err != nil {
+		return err
+	}
+	o.appName = o.wsAppName
+
 	if o.name != "" {
 		if err := validateEnvironmentName(o.name); err != nil {
 			return err
@@ -238,9 +245,6 @@ func (o *initEnvOpts) Validate() error {
 
 // Ask asks for fields that are required but not passed in.
 func (o *initEnvOpts) Ask() error {
-	if err := o.askAppName(); err != nil {
-		return err
-	}
 	if err := o.askEnvName(); err != nil {
 		return err
 	}
@@ -261,24 +265,31 @@ func (o *initEnvOpts) Execute() error {
 		// Ensure the app actually exists before we do a deployment.
 		return err
 	}
-
 	envCaller, err := o.envIdentity.Get()
 	if err != nil {
 		return fmt.Errorf("get identity: %w", err)
 	}
 
+	// 1. Write environment manifest.
+	path, err := o.writeManifest()
+	if err != nil {
+		return err
+	}
+	o.mftDisplayedPath = path
+
+	// 2. Perform DNS delegation from app to env.
 	if app.Domain != "" {
 		if err := o.delegateDNSFromApp(app, envCaller.Account); err != nil {
 			return fmt.Errorf("granting DNS permissions: %w", err)
 		}
 	}
 
-	// 1. Attempt to create the service linked role if it doesn't exist.
+	// 3. Attempt to create the service linked role if it doesn't exist.
 	// If the call fails because the role already exists, nothing to do.
 	// If the call fails because the user doesn't have permissions, then the role must be created outside of Copilot.
 	_ = o.iam.CreateECSServiceLinkedRole()
 
-	// 2. Add the stack set instance to the app stackset.
+	// 4. Add the stack set instance to the app stackset.
 	if err := o.addToStackset(&deploycfn.AddEnvToAppOpts{
 		App:          app,
 		EnvName:      o.name,
@@ -288,66 +299,32 @@ func (o *initEnvOpts) Execute() error {
 		return err
 	}
 
-	// 3. Upload environment custom resource scripts to the S3 bucket, because of the 4096 characters limit (see
-	// https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lambda-function-code.html#cfn-lambda-function-code-zipfile)
-	envRegion := aws.StringValue(o.sess.Config.Region)
-	resources, err := o.appCFN.GetAppResourcesByRegion(app, envRegion)
-	if err != nil {
-		return fmt.Errorf("get app resources: %w", err)
-	}
-	s3Client, err := o.newS3(envRegion)
-	if err != nil {
-		return err
-	}
-	urls, err := o.uploader.UploadEnvironmentCustomResources(func(key string, objects ...s3.NamedBinary) (string, error) {
-		return s3Client.ZipAndUpload(resources.S3Bucket, key, objects...)
-	})
-	if err != nil {
-		return fmt.Errorf("upload custom resources to bucket %s: %w", resources.S3Bucket, err)
-	}
-
-	// 4. Start creating the CloudFormation stack for the environment.
-	if resources.S3Bucket == "" {
-		log.Errorf("Cannot find the S3 artifact bucket in %s region created by app %s. The S3 bucket is necessary for many future operations. For example, when you need addons to your services.", envRegion, app.Name)
-		return fmt.Errorf("cannot find the S3 artifact bucket in %s region", envRegion)
-	}
-	partition, err := partitions.Region(envRegion).Partition()
-	if err != nil {
-		return err
-	}
-	if err := o.deployEnv(app, s3.FormatARN(partition.ID(), resources.S3Bucket), resources.KMSKeyARN, urls); err != nil {
+	// 5. Start creating the CloudFormation stack for the environment.
+	if err := o.deployEnv(app); err != nil {
 		return err
 	}
 
-	// 5. Get the environment
+	// 6. Store the environment in SSM with information about the deployed bootstrap roles.
 	env, err := o.envDeployer.GetEnvironment(o.appName, o.name)
 	if err != nil {
 		return fmt.Errorf("get environment struct for %s: %w", o.name, err)
 	}
 	env.Prod = o.isProduction
-	customizedEnv := config.CustomizeEnv{
-		ImportVPC:                   o.importVPCConfig(),
-		VPCConfig:                   o.adjustVPCConfig(),
-		ImportCertARNs:              o.importCerts,
-		InternalALBSubnets:          o.internalALBSubnets,
-		EnableInternalALBVPCIngress: o.allowVPCIngress,
-	}
-	if !customizedEnv.IsEmpty() {
-		env.CustomConfig = &customizedEnv
-	}
-	env.Telemetry = o.telemetry.toConfig()
-
-	// 6. Store the environment in SSM.
 	if err := o.store.CreateEnvironment(env); err != nil {
 		return fmt.Errorf("store environment: %w", err)
 	}
-	log.Successf("Created environment %s in region %s under application %s.\n",
+	log.Successf("Provisioned bootstrap resources for environment %s in region %s under application %s.\n",
 		color.HighlightUserInput(env.Name), color.Emphasize(env.Region), color.HighlightUserInput(env.App))
 	return nil
 }
 
 // RecommendActions returns follow-up actions the user can take after successfully executing the command.
 func (o *initEnvOpts) RecommendActions() error {
+	logRecommendedActions([]string{
+		fmt.Sprintf("Update your manifest %s to change the defaults.", color.HighlightResource(o.mftDisplayedPath)),
+		fmt.Sprintf("Run %s to deploy your environment.",
+			color.HighlightCode(fmt.Sprintf("copilot env deploy --name %s", o.name))),
+	})
 	return nil
 }
 
@@ -357,7 +334,7 @@ func (o *initEnvOpts) initRuntimeClients() {
 		o.envIdentity = identity.New(o.sess)
 	}
 	if o.envDeployer == nil {
-		o.envDeployer = deploycfn.New(o.sess)
+		o.envDeployer = deploycfn.New(o.sess, deploycfn.WithProgressTracker(os.Stderr))
 	}
 	if o.cfn == nil {
 		o.cfn = cloudformation.New(o.sess)
@@ -399,19 +376,6 @@ For default config without subnet placement specification, Copilot will place th
 	return nil
 }
 
-func (o *initEnvOpts) askAppName() error {
-	if o.appName != "" {
-		return nil
-	}
-
-	app, err := o.selApp.Application(envInitAppNamePrompt, envInitAppNameHelpPrompt)
-	if err != nil {
-		return fmt.Errorf("ask for application: %w", err)
-	}
-	o.appName = app
-	return nil
-}
-
 func (o *initEnvOpts) askEnvName() error {
 	if o.name != "" {
 		return nil
@@ -665,11 +629,18 @@ func (o *initEnvOpts) askAZs() ([]string, error) {
 func (o *initEnvOpts) validateDuplicateEnv() error {
 	_, err := o.store.GetEnvironment(o.appName, o.name)
 	if err == nil {
-		log.Errorf(`It seems like you are trying to init an environment that already exists.
-To recreate the environment, please run:
+		dir := filepath.Join("copilot", "environments", o.name)
+		log.Infof(`It seems like you are trying to init an environment that already exists.
+To generate a manifest for the environment:
+1. %s
+2. %s
+
+Alternatively, to recreate the environment:
 1. %s
 2. And then %s
 `,
+			color.HighlightCode(fmt.Sprintf("mkdir -p %s", dir)),
+			color.HighlightCode(fmt.Sprintf("copilot env show -n %s --manifest > %s", o.name, filepath.Join(dir, "manifest.yml"))),
 			color.HighlightCode(fmt.Sprintf("copilot env delete --name %s", o.name)),
 			color.HighlightCode(fmt.Sprintf("copilot env init --name %s", o.name)))
 		return fmt.Errorf("environment %s already exists", color.HighlightUserInput(o.name))
@@ -705,8 +676,22 @@ func (o *initEnvOpts) adjustVPCConfig() *config.AdjustVPC {
 	}
 }
 
-func (o *initEnvOpts) deployEnv(app *config.Application,
-	artifactBucketARN, artifactBucketKeyARN string, customResourcesURLs map[string]string) error {
+func (o *initEnvOpts) deployEnv(app *config.Application) error {
+	envRegion := aws.StringValue(o.sess.Config.Region)
+	resources, err := o.appCFN.GetAppResourcesByRegion(app, envRegion)
+	if err != nil {
+		return fmt.Errorf("get app resources: %w", err)
+	}
+	if resources.S3Bucket == "" {
+		log.Errorf("Cannot find the S3 artifact bucket in %s region created by app %s. The S3 bucket is necessary for many future operations. For example, when you need addons to your services.", envRegion, app.Name)
+		return fmt.Errorf("cannot find the S3 artifact bucket in %s region", envRegion)
+	}
+	partition, err := partitions.Region(envRegion).Partition()
+	if err != nil {
+		return err
+	}
+	artifactBucketARN := s3.FormatARN(partition.ID(), resources.S3Bucket)
+
 	caller, err := o.identity.Get()
 	if err != nil {
 		return fmt.Errorf("get identity: %w", err)
@@ -719,22 +704,14 @@ func (o *initEnvOpts) deployEnv(app *config.Application,
 			AccountPrincipalARN: caller.RootUserARN,
 		},
 		AdditionalTags:       app.Tags,
-		CustomResourcesURLs:  customResourcesURLs,
 		ArtifactBucketARN:    artifactBucketARN,
-		ArtifactBucketKeyARN: artifactBucketKeyARN,
-		AdjustVPCConfig:      o.adjustVPCConfig(),
-		ImportCertARNs:       o.importCerts,
-		InternalALBSubnets:   o.internalALBSubnets,
-		ImportVPCConfig:      o.importVPCConfig(),
-		AllowVPCIngress:      o.allowVPCIngress,
-		Telemetry:            o.telemetry.toConfig(),
-		Version:              deploy.LatestEnvTemplateVersion,
+		ArtifactBucketKeyARN: resources.KMSKeyARN,
 	}
 
 	if err := o.cleanUpDanglingRoles(o.appName, o.name); err != nil {
 		return err
 	}
-	if err := o.envDeployer.CreateAndRenderEnvironment(os.Stderr, deployEnvInput); err != nil {
+	if err := o.envDeployer.CreateAndRenderEnvironment(deployEnvInput); err != nil {
 		var existsErr *cloudformation.ErrStackAlreadyExists
 		if errors.As(err, &existsErr) {
 			// Do nothing if the stack already exists.
@@ -752,7 +729,7 @@ func (o *initEnvOpts) addToStackset(opts *deploycfn.AddEnvToAppOpts) error {
 	o.prog.Start(fmt.Sprintf(fmtAddEnvToAppStart, color.Emphasize(opts.EnvAccountID), color.Emphasize(opts.EnvRegion), color.HighlightUserInput(o.appName)))
 	if err := o.appDeployer.AddEnvToApp(opts); err != nil {
 		o.prog.Stop(log.Serrorf(fmtAddEnvToAppFailed, color.Emphasize(opts.EnvAccountID), color.Emphasize(opts.EnvRegion), color.HighlightUserInput(o.appName)))
-		return fmt.Errorf("deploy env %s to application %s: %w", opts.EnvName, opts.App.Name, err)
+		return fmt.Errorf("add env %s to application %s: %w", opts.EnvName, opts.App.Name, err)
 	}
 	o.prog.Stop(log.Ssuccessf(fmtAddEnvToAppComplete, color.Emphasize(opts.EnvAccountID), color.Emphasize(opts.EnvRegion), color.HighlightUserInput(o.appName)))
 
@@ -847,6 +824,42 @@ func (o *initEnvOpts) tryDeletingEnvRoles(app, env string) {
 	}
 }
 
+func (o *initEnvOpts) writeManifest() (string, error) {
+	customizedEnv := &config.CustomizeEnv{
+		ImportVPC:                   o.importVPCConfig(),
+		VPCConfig:                   o.adjustVPCConfig(),
+		ImportCertARNs:              o.importCerts,
+		InternalALBSubnets:          o.internalALBSubnets,
+		EnableInternalALBVPCIngress: o.allowVPCIngress,
+	}
+	if customizedEnv.IsEmpty() {
+		customizedEnv = nil
+	}
+	props := manifest.EnvironmentProps{
+		Name:         o.name,
+		CustomConfig: customizedEnv,
+		Telemetry:    o.telemetry.toConfig(),
+	}
+
+	var manifestExists bool
+	manifestPath, err := o.manifestWriter.WriteEnvironmentManifest(manifest.NewEnvironment(&props), props.Name)
+	if err != nil {
+		e, ok := err.(*workspace.ErrFileExists)
+		if !ok {
+			return "", fmt.Errorf("write environment manifest: %w", err)
+		}
+		manifestExists = true
+		manifestPath = e.FileName
+	}
+	manifestPath = displayPath(manifestPath)
+	manifestMsgFmt := "Wrote the manifest for environment %s at %s\n"
+	if manifestExists {
+		manifestMsgFmt = "Manifest file for environment %s already exists at %s, skipping writing it.\n"
+	}
+	log.Successf(manifestMsgFmt, color.HighlightUserInput(props.Name), color.HighlightResource(manifestPath))
+	return manifestPath, nil
+}
+
 // buildEnvInitCmd builds the command for adding an environment.
 func buildEnvInitCmd() *cobra.Command {
 	vars := initEnvVars{}
