@@ -8,8 +8,6 @@ import (
 	"strings"
 
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
-
-	"github.com/aws/copilot-cli/internal/pkg/template"
 )
 
 // Long flag names.
@@ -38,6 +36,7 @@ const (
 	stackOutputDirFlag    = "output-dir"
 	uploadAssetsFlag      = "upload-assets"
 	limitFlag             = "limit"
+	lastFlag              = "last"
 	followFlag            = "follow"
 	sinceFlag             = "since"
 	startTimeFlag         = "start-time"
@@ -47,15 +46,18 @@ const (
 	prodEnvFlag           = "prod"
 	deployFlag            = "deploy"
 	resourcesFlag         = "resources"
+
 	githubURLFlag         = "github-url"
 	repoURLFlag           = "url"
 	githubAccessTokenFlag = "github-access-token"
 	gitBranchFlag         = "git-branch"
 	envsFlag              = "environments"
-	domainNameFlag        = "domain"
-	localFlag             = "local"
-	deleteSecretFlag      = "delete-secret"
-	svcPortFlag           = "port"
+	pipelineTypeFlag      = "pipeline-type"
+
+	domainNameFlag   = "domain"
+	localFlag        = "local"
+	deleteSecretFlag = "delete-secret"
+	svcPortFlag      = "port"
 
 	noSubscriptionFlag  = "no-subscribe"
 	subscribeTopicsFlag = "subscribe-topics"
@@ -82,6 +84,7 @@ const (
 	subnetsFlag                  = "subnets"
 	securityGroupsFlag           = "security-groups"
 	envVarsFlag                  = "env-vars"
+	envFileFlag                  = "env-file"
 	secretsFlag                  = "secrets"
 	commandFlag                  = "command"
 	entrypointFlag               = "entrypoint"
@@ -140,6 +143,7 @@ const (
 	githubAccessTokenFlagShort = "t"
 	gitBranchFlagShort         = "b"
 	envsFlagShort              = "e"
+	pipelineTypeShort          = "p"
 
 	scheduleFlagShort = "s"
 )
@@ -147,7 +151,7 @@ const (
 // Descriptions for flags.
 var (
 	svcTypeFlagDescription = fmt.Sprintf(`Type of service to create. Must be one of:
-%s.`, strings.Join(template.QuoteSliceFunc(manifest.ServiceTypes()), ", "))
+%s.`, strings.Join(quoteStringSlice(manifest.ServiceTypes()), ", "))
 	imageFlagDescription = fmt.Sprintf(`The location of an existing Docker image.
 Cannot be specified with --%s or --%s.`, dockerFileFlag, dockerFileContextFlag)
 	dockerFileFlagDescription = fmt.Sprintf(`Path to the Dockerfile.
@@ -155,11 +159,11 @@ Cannot be specified with --%s.`, imageFlag)
 	dockerFileContextFlagDescription = fmt.Sprintf(`Path to the Docker build context.
 Cannot be specified with --%s.`, imageFlag)
 	storageTypeFlagDescription = fmt.Sprintf(`Type of storage to add. Must be one of:
-%s.`, strings.Join(template.QuoteSliceFunc(storageTypes), ", "))
+%s.`, strings.Join(quoteStringSlice(storageTypes), ", "))
 	jobTypeFlagDescription = fmt.Sprintf(`Type of job to create. Must be one of:
-%s.`, strings.Join(template.QuoteSliceFunc(manifest.JobTypes()), ", "))
+%s.`, strings.Join(quoteStringSlice(manifest.JobTypes()), ", "))
 	wkldTypeFlagDescription = fmt.Sprintf(`Type of job or svc to create. Must be one of:
-%s.`, strings.Join(template.QuoteSliceFunc(manifest.WorkloadTypes()), ", "))
+%s.`, strings.Join(quoteStringSlice(manifest.WorkloadTypes()), ", "))
 
 	clusterFlagDescription = fmt.Sprintf(`Optional. The short name or full ARN of the cluster to run the task in. 
 Cannot be specified with --%s, --%s or --%s.`, appFlag, envFlag, taskDefaultFlag)
@@ -193,23 +197,26 @@ Supported providers are: %s.`, strings.Join(manifest.PipelineProviders, ", "))
 )
 
 const (
-	appFlagDescription        = "Name of the application."
-	envFlagDescription        = "Name of the environment."
-	svcFlagDescription        = "Name of the service."
-	jobFlagDescription        = "Name of the job."
-	workloadFlagDescription   = "Name of the service or job."
-	nameFlagDescription       = "Name of the service, job, or task group."
-	pipelineFlagDescription   = "Name of the pipeline."
-	profileFlagDescription    = "Name of the profile."
-	yesFlagDescription        = "Skips confirmation prompt."
-	execYesFlagDescription    = "Optional. Whether to update the Session Manager Plugin."
-	jsonFlagDescription       = "Optional. Outputs in JSON format."
-	forceFlagDescription      = "Optional. Force a new service deployment using the existing image."
-	noRollbackFlagDescription = `Optional. Disable automatic stack 
+	appFlagDescription            = "Name of the application."
+	envFlagDescription            = "Name of the environment."
+	svcFlagDescription            = "Name of the service."
+	jobFlagDescription            = "Name of the job."
+	workloadFlagDescription       = "Name of the service or job."
+	nameFlagDescription           = "Name of the service, job, or task group."
+	pipelineFlagDescription       = "Name of the pipeline."
+	profileFlagDescription        = "Name of the profile."
+	yesFlagDescription            = "Skips confirmation prompt."
+	execYesFlagDescription        = "Optional. Whether to update the Session Manager Plugin."
+	jsonFlagDescription           = "Optional. Output in JSON format."
+	forceFlagDescription          = "Optional. Force a new service deployment using the existing image."
+	forceEnvDeployFlagDescription = "Optional. Force update the environment stack template."
+	noRollbackFlagDescription     = `Optional. Disable automatic stack 
 rollback in case of deployment failure.
 We do not recommend using this flag for a
 production environment.`
 	manifestFlagDescription = "Optional. Output the manifest file used for the deployment."
+	svcManifestFlagDescription = `Optional. Name of the environment in which the service was deployed;
+output the manifest file used for that deployment.`
 
 	imageTagFlagDescription     = `Optional. The container image tag.`
 	resourceTagsFlagDescription = `Optional. Labels with a key and value separated by commas.
@@ -221,6 +228,8 @@ Uploaded asset locations are filled in the template configuration.`
 
 	limitFlagDescription = `Optional. The maximum number of log events returned. Default is 10
 unless any time filtering flags are set.`
+	lastFlagDescription = `Optional. The number of executions of the scheduled job for which
+logs should be shown.`
 	followFlagDescription = "Optional. Specifies if the logs should be streamed."
 	sinceFlagDescription  = `Optional. Only return logs newer than a relative duration like 5s, 2m, or 3h.
 Defaults to all logs. Only one of start-time / since may be used.`
@@ -237,6 +246,7 @@ Defaults to all logs. Only one of end-time / follow may be used.`
 	githubAccessTokenFlagDescription = "GitHub personal access token for your repository."
 	gitBranchFlagDescription         = "Branch used to trigger your pipeline."
 	pipelineEnvsFlagDescription      = "Environments to add to the pipeline."
+	pipelineTypeFlagDescription      = `The type of pipeline. Must be either "Workloads" or "Environments".`
 	domainNameFlagDescription        = "Optional. Your existing custom domain name."
 	envResourcesFlagDescription      = "Optional. Show the resources in your environment."
 	svcResourcesFlagDescription      = "Optional. Show the resources in your service."
@@ -272,6 +282,9 @@ Must be either "MySQL" or "PostgreSQL".`
 	taskRoleFlagDescription      = "Optional. The ARN of the role for the task to use."
 	executionRoleFlagDescription = "Optional. The ARN of the role that grants the container agent permission to make AWS API calls."
 	envVarsFlagDescription       = "Optional. Environment variables specified by key=value separated by commas."
+	envFileFlagDescription       = `Optional. A path to an environment variable (.env) file. 
+Each line should be of the form of VARIABLE=VALUE. 
+Values specified with --env-vars take precedence over --env-file.`
 	secretsFlagDescription       = "Optional. Secrets to inject into the container. Specified by key=value separated by commas."
 	runCommandFlagDescription    = `Optional. The command that is passed to "docker run" to override the default command.`
 	entrypointFlagDescription    = `Optional. The entrypoint that is passed to "docker run" to override the default entrypoint.`
