@@ -28,7 +28,7 @@ type pipelineInitMocks struct {
 	workspace      *mocks.MockwsPipelineIniter
 	secretsmanager *mocks.MocksecretsManager
 	parser         *templatemocks.MockParser
-	runner         *mocks.Mockrunner
+	runner         *mocks.MockexecRunner
 	sessProvider   *mocks.MocksessionProvider
 	cfnClient      *mocks.MockappResourcesGetter
 	store          *mocks.Mockstore
@@ -57,6 +57,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 		inRepoURL           string
 		inGitHubAccessToken string
 		inGitBranch         string
+		inType              string
 
 		setupMocks func(m pipelineInitMocks)
 		buffer     bytes.Buffer
@@ -144,6 +145,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 			buffer:         *bytes.NewBufferString("devBranch"),
 			setupMocks: func(m pipelineInitMocks) {
 				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 				m.runner.EXPECT().Run(gomock.Any(), gomock.Any(), gomock.Any()).Return(nil)
 				m.store.EXPECT().GetEnvironment("my-app", "test").Return(
 					&config.Environment{
@@ -162,6 +164,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 			inName:         wantedName,
 			setupMocks: func(m pipelineInitMocks) {
 				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 				m.runner.EXPECT().Run(gomock.Any(), gomock.Any(), gomock.Any()).Return(nil)
 				m.store.EXPECT().GetEnvironment("my-app", "test").Return(
 					&config.Environment{
@@ -196,6 +199,31 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 
 			expectedError: fmt.Errorf("get pipeline name: mock error"),
 		},
+		"invalid pipeline type": {
+			inWsAppName: mockAppName,
+			inRepoURL:   githubAnotherURL,
+			inGitBranch: "main",
+			inName:      "mock-pipeline",
+			inType:      "RandomType",
+			setupMocks: func(m pipelineInitMocks) {
+				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+			},
+			expectedError: errors.New(`invalid pipeline type "RandomType"; must be one of "Workloads" or "Environments"`),
+		},
+		"returns an error if fail to get pipeline type": {
+			inWsAppName: mockAppName,
+			inRepoURL:   githubAnotherURL,
+			inGitBranch: "main",
+			setupMocks: func(m pipelineInitMocks) {
+				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+				m.prompt.EXPECT().Get(gomock.Eq("What would you like to name this pipeline?"), gomock.Any(), gomock.Any(), gomock.Any()).
+					Return(wantedName, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Eq("What type of continuous delivery pipeline is this?"), gomock.Any(), gomock.Any()).
+					Return("", errors.New("mock error"))
+			},
+
+			expectedError: fmt.Errorf("prompt for pipeline type: mock error"),
+		},
 		"prompt for pipeline name": {
 			inWsAppName:    mockAppName,
 			inRepoURL:      githubAnotherURL,
@@ -209,6 +237,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 				m.workspace.EXPECT().ListPipelines().Return([]workspace.PipelineManifest{}, nil)
 				m.prompt.EXPECT().Get(gomock.Eq("What would you like to name this pipeline?"), gomock.Any(), gomock.Any(), gomock.Any()).
 					Return(wantedName, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 			},
 		},
 		"passed-in URL to unsupported repo provider": {
@@ -218,6 +247,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 			inEnvironments: []string{"test"},
 			setupMocks: func(m pipelineInitMocks) {
 				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 			},
 
 			expectedError: errors.New("repository unsupported.org/repositories/repoName must be from a supported provider: GitHub, CodeCommit or Bitbucket"),
@@ -230,6 +260,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 			inGitBranch:    "main",
 			setupMocks: func(m pipelineInitMocks) {
 				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 				m.store.EXPECT().GetEnvironment("my-app", "test").Return(nil, mockError)
 				m.pipelineLister.EXPECT().ListDeployedPipelines(mockAppName).Return([]deploy.Pipeline{}, nil)
 				m.workspace.EXPECT().ListPipelines().Return([]workspace.PipelineManifest{}, nil)
@@ -245,6 +276,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 			inGitBranch:    "main",
 			setupMocks: func(m pipelineInitMocks) {
 				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 				m.store.EXPECT().GetEnvironment("my-app", "test").Return(
 					&config.Environment{
 						Name: "test",
@@ -270,6 +302,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 						Region: aws.String("us-west-2"),
 					},
 				}, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 				m.store.EXPECT().GetEnvironment("my-app", "test").Return(
 					&config.Environment{
 						Name: "test",
@@ -299,6 +332,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 					Region: "us-west-2",
 				}, nil)
 				m.prompt.EXPECT().Get(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any()).Return(wantedName, nil)
+				m.prompt.EXPECT().SelectOption("What type of continuous delivery pipeline is this?", gomock.Any(), gomock.Any()).Return(pipelineTypeEnvironments, nil)
 				m.prompt.EXPECT().SelectOne(pipelineSelectURLPrompt, gomock.Any(), gomock.Any(), gomock.Any()).Return(githubAnotherURL, nil).Times(1)
 				m.pipelineLister.EXPECT().ListDeployedPipelines(mockAppName).Return([]deploy.Pipeline{}, nil)
 				m.workspace.EXPECT().ListPipelines().Return([]workspace.PipelineManifest{}, nil)
@@ -312,6 +346,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 			inGitBranch: "main",
 			setupMocks: func(m pipelineInitMocks) {
 				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 				m.pipelineLister.EXPECT().ListDeployedPipelines(mockAppName).Return([]deploy.Pipeline{}, nil)
 				m.workspace.EXPECT().ListPipelines().Return(nil, nil)
 				m.sel.EXPECT().Environments(pipelineSelectEnvPrompt, gomock.Any(), "my-app", gomock.Any()).Return(nil, errors.New("some error"))
@@ -341,6 +376,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 			inEnvironments: []string{},
 			setupMocks: func(m pipelineInitMocks) {
 				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 				m.pipelineLister.EXPECT().ListDeployedPipelines(mockAppName).Return([]deploy.Pipeline{}, nil)
 				m.workspace.EXPECT().ListPipelines().Return(nil, nil)
 				m.sel.EXPECT().Environments(pipelineSelectEnvPrompt, gomock.Any(), "my-app", gomock.Any()).Return([]string{"test", "prod"}, nil)
@@ -360,6 +396,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 			buffer:      *bytes.NewBufferString("archer\tgit@github.com:goodGoose/bhaOS (fetch)\n"),
 			setupMocks: func(m pipelineInitMocks) {
 				m.store.EXPECT().GetApplication(mockAppName).Return(mockApp, nil)
+				m.prompt.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any()).AnyTimes()
 				m.runner.EXPECT().Run(gomock.Any(), gomock.Any(), gomock.Any()).Return(nil)
 				m.pipelineLister.EXPECT().ListDeployedPipelines(mockAppName).Return([]deploy.Pipeline{}, nil)
 				m.workspace.EXPECT().ListPipelines().Return(nil, nil)
@@ -384,7 +421,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 
 			mocks := pipelineInitMocks{
 				prompt:         mocks.NewMockprompter(ctrl),
-				runner:         mocks.NewMockrunner(ctrl),
+				runner:         mocks.NewMockexecRunner(ctrl),
 				sessProvider:   mocks.NewMocksessionProvider(ctrl),
 				sel:            mocks.NewMockpipelineEnvSelector(ctrl),
 				store:          mocks.NewMockstore(ctrl),
@@ -403,6 +440,7 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 					repoURL:           tc.inRepoURL,
 					githubAccessToken: tc.inGitHubAccessToken,
 					repoBranch:        tc.inGitBranch,
+					pipelineType:      tc.inType,
 				},
 				wsAppName:      tc.inWsAppName,
 				prompt:         mocks.prompt,
@@ -433,10 +471,11 @@ func TestInitPipelineOpts_Ask(t *testing.T) {
 
 func TestInitPipelineOpts_Execute(t *testing.T) {
 	const (
-		wantedName          = "mypipe"
-		wantedManifestFile  = "/pipelines/mypipe/manifest.yml"
-		wantedBuildspecFile = "/pipelines/mypipe/buildspec.yml"
-		wantedRelativePath  = "/copilot/pipelines/mypipe/manifest.yml"
+		wantedName             = "mypipe"
+		wantedManifestFile     = "/pipelines/mypipe/manifest.yml"
+		wantedManifestRelPath  = "/copilot/pipelines/mypipe/manifest.yml"
+		wantedBuildspecFile    = "/pipelines/mypipe/buildspec.yml"
+		wantedBuildspecRelPath = "/copilot/pipelines/mypipe/buildspec.yml"
 	)
 
 	buildspecExistsErr := &workspace.ErrFileExists{FileName: wantedBuildspecFile}
@@ -449,6 +488,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 		inRepoURL      string
 		inBranch       string
 		inAppName      string
+		inType         string
 
 		setupMocks func(m pipelineInitMocks)
 		buffer     bytes.Buffer
@@ -457,6 +497,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 	}{
 		"creates secret and writes manifest and buildspec for GHV1 provider": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -469,8 +510,8 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 				m.secretsmanager.EXPECT().CreateSecret("github-token-badgoose-goose", "hunter2").Return("some-arn", nil)
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
 				m.workspace.EXPECT().WritePipelineBuildspec(gomock.Any(), wantedName).Return(wantedBuildspecFile, nil)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
-				m.parser.EXPECT().Parse(buildspecTemplatePath, gomock.Any()).Return(&template.Content{
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
+				m.parser.EXPECT().Parse(workloadsPipelineBuildspecTemplatePath, gomock.Any()).Return(&template.Content{
 					Buffer: bytes.NewBufferString("hello"),
 				}, nil)
 				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
@@ -487,8 +528,9 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 			},
 			expectedError: nil,
 		},
-		"writes manifest and buildspec for GH(v2) provider": {
+		"writes workloads pipeline manifest and buildspec for GH(v2) provider": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -499,8 +541,8 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 			setupMocks: func(m pipelineInitMocks) {
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
 				m.workspace.EXPECT().WritePipelineBuildspec(gomock.Any(), wantedName).Return(wantedBuildspecFile, nil)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
-				m.parser.EXPECT().Parse(buildspecTemplatePath, gomock.Any()).Return(&template.Content{
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
+				m.parser.EXPECT().Parse(workloadsPipelineBuildspecTemplatePath, gomock.Any()).Return(&template.Content{
 					Buffer: bytes.NewBufferString("hello"),
 				}, nil)
 				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
@@ -517,8 +559,9 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 			},
 			expectedError: nil,
 		},
-		"writes manifest and buildspec for CC provider": {
+		"writes workloads pipeline manifest and buildspec for CC provider": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -529,8 +572,8 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 			setupMocks: func(m pipelineInitMocks) {
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
 				m.workspace.EXPECT().WritePipelineBuildspec(gomock.Any(), wantedName).Return(wantedBuildspecFile, nil)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
-				m.parser.EXPECT().Parse(buildspecTemplatePath, gomock.Any()).Return(&template.Content{
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
+				m.parser.EXPECT().Parse(workloadsPipelineBuildspecTemplatePath, gomock.Any()).Return(&template.Content{
 					Buffer: bytes.NewBufferString("hello"),
 				}, nil)
 				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
@@ -552,8 +595,9 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 			},
 			expectedError: nil,
 		},
-		"writes manifest and buildspec for BB provider": {
+		"writes workloads pipeline manifest and buildspec for BB provider": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -564,8 +608,39 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 			setupMocks: func(m pipelineInitMocks) {
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
 				m.workspace.EXPECT().WritePipelineBuildspec(gomock.Any(), wantedName).Return(wantedBuildspecFile, nil)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
-				m.parser.EXPECT().Parse(buildspecTemplatePath, gomock.Any()).Return(&template.Content{
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
+				m.parser.EXPECT().Parse(workloadsPipelineBuildspecTemplatePath, gomock.Any()).Return(&template.Content{
+					Buffer: bytes.NewBufferString("hello"),
+				}, nil)
+				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
+					Name: "badgoose",
+				}, nil)
+				m.cfnClient.EXPECT().GetRegionalAppResources(&config.Application{
+					Name: "badgoose",
+				}).Return([]*stack.AppRegionalResources{
+					{
+						Region:   "us-west-2",
+						S3Bucket: "gooseBucket",
+					},
+				}, nil)
+			},
+			expectedError: nil,
+		},
+		"writes environments pipeline manifest for GH(v2) provider": {
+			inName: wantedName,
+			inType: pipelineTypeEnvironments,
+			inEnvConfigs: []*config.Environment{
+				{
+					Name: "test",
+				},
+			},
+			inRepoURL: "git@github.com:badgoose/goose.git",
+			inAppName: "badgoose",
+			setupMocks: func(m pipelineInitMocks) {
+				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
+				m.workspace.EXPECT().WritePipelineBuildspec(gomock.Any(), wantedName).Return(wantedBuildspecFile, nil)
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
+				m.parser.EXPECT().Parse(environmentsPipelineBuildspecTemplatePath, gomock.Any()).Return(&template.Content{
 					Buffer: bytes.NewBufferString("hello"),
 				}, nil)
 				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
@@ -584,6 +659,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 		},
 		"does not return an error if secret already exists": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -597,8 +673,8 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 				m.secretsmanager.EXPECT().CreateSecret("github-token-badgoose-goose", "hunter2").Return("", existsErr)
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
 				m.workspace.EXPECT().WritePipelineBuildspec(gomock.Any(), wantedName).Return(wantedBuildspecFile, nil)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
-				m.parser.EXPECT().Parse(buildspecTemplatePath, gomock.Any()).Return(&template.Content{
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
+				m.parser.EXPECT().Parse(workloadsPipelineBuildspecTemplatePath, gomock.Any()).Return(&template.Content{
 					Buffer: bytes.NewBufferString("hello"),
 				}, nil)
 				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
@@ -617,6 +693,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 		},
 		"returns an error if can't write manifest": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -633,6 +710,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 		},
 		"returns an error if application cannot be retrieved": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -644,13 +722,14 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 			setupMocks: func(m pipelineInitMocks) {
 				m.secretsmanager.EXPECT().CreateSecret("github-token-badgoose-goose", "hunter2").Return("some-arn", nil)
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
 				m.store.EXPECT().GetApplication("badgoose").Return(nil, errors.New("some error"))
 			},
 			expectedError: errors.New("get application badgoose: some error"),
 		},
 		"returns an error if can't get regional application resources": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -662,7 +741,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 			setupMocks: func(m pipelineInitMocks) {
 				m.secretsmanager.EXPECT().CreateSecret("github-token-badgoose-goose", "hunter2").Return("some-arn", nil)
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
 				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
 					Name: "badgoose",
 				}, nil)
@@ -674,6 +753,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 		},
 		"returns an error if buildspec cannot be parsed": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -686,8 +766,8 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 				m.secretsmanager.EXPECT().CreateSecret("github-token-badgoose-goose", "hunter2").Return("some-arn", nil)
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
 				m.workspace.EXPECT().WritePipelineBuildspec(gomock.Any(), wantedName).Times(0)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
-				m.parser.EXPECT().Parse(buildspecTemplatePath, gomock.Any()).Return(nil, errors.New("some error"))
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
+				m.parser.EXPECT().Parse(workloadsPipelineBuildspecTemplatePath, gomock.Any()).Return(nil, errors.New("some error"))
 				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
 					Name: "badgoose",
 				}, nil)
@@ -704,6 +784,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 		},
 		"does not return an error if buildspec and manifest already exists": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -716,8 +797,8 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 				m.secretsmanager.EXPECT().CreateSecret("github-token-badgoose-goose", "hunter2").Return("some-arn", nil)
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return("", manifestExistsErr)
 				m.workspace.EXPECT().WritePipelineBuildspec(gomock.Any(), wantedName).Return("", buildspecExistsErr)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
-				m.parser.EXPECT().Parse(buildspecTemplatePath, gomock.Any()).Return(&template.Content{
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
+				m.parser.EXPECT().Parse(workloadsPipelineBuildspecTemplatePath, gomock.Any()).Return(&template.Content{
 					Buffer: bytes.NewBufferString("hello"),
 				}, nil)
 				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
@@ -736,6 +817,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 		},
 		"returns an error if can't write buildspec": {
 			inName: wantedName,
+			inType: pipelineTypeWorkloads,
 			inEnvConfigs: []*config.Environment{
 				{
 					Name: "test",
@@ -747,9 +829,9 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 			setupMocks: func(m pipelineInitMocks) {
 				m.secretsmanager.EXPECT().CreateSecret("github-token-badgoose-goose", "hunter2").Return("some-arn", nil)
 				m.workspace.EXPECT().WritePipelineManifest(gomock.Any(), wantedName).Return(wantedManifestFile, nil)
-				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedRelativePath, nil)
+				m.workspace.EXPECT().Rel(wantedManifestFile).Return(wantedManifestRelPath, nil)
 				m.workspace.EXPECT().WritePipelineBuildspec(gomock.Any(), wantedName).Return("", errors.New("some error"))
-				m.parser.EXPECT().Parse(buildspecTemplatePath, gomock.Any()).Return(&template.Content{
+				m.parser.EXPECT().Parse(workloadsPipelineBuildspecTemplatePath, gomock.Any()).Return(&template.Content{
 					Buffer: bytes.NewBufferString("hello"),
 				}, nil)
 				m.store.EXPECT().GetApplication("badgoose").Return(&config.Application{
@@ -795,6 +877,7 @@ func TestInitPipelineOpts_Execute(t *testing.T) {
 					appName:           tc.inAppName,
 					repoBranch:        tc.inBranch,
 					repoURL:           tc.inRepoURL,
+					pipelineType:      tc.inType,
 				},
 				workspace:      mocks.workspace,
 				secretsmanager: mocks.secretsmanager,
