@@ -8,7 +8,6 @@ import (
 	"fmt"
 	"testing"
 
-	"github.com/aws/copilot-cli/internal/pkg/config"
 	"github.com/aws/copilot-cli/internal/pkg/term/prompt"
 
 	"github.com/aws/copilot-cli/internal/pkg/cli/mocks"
@@ -45,7 +44,7 @@ func TestStorageInitOpts_Validate(t *testing.T) {
 		},
 		"svc not in workspace": {
 			mockWs: func(m *mocks.MockwsAddonManager) {
-				m.EXPECT().WorkloadNames().Return([]string{"bad", "workspace"}, nil)
+				m.EXPECT().ListWorkloads().Return([]string{"bad", "workspace"}, nil)
 			},
 			mockStore: func(m *mocks.Mockstore) {},
 
@@ -57,7 +56,7 @@ func TestStorageInitOpts_Validate(t *testing.T) {
 		},
 		"workspace error": {
 			mockWs: func(m *mocks.MockwsAddonManager) {
-				m.EXPECT().WorkloadNames().Return(nil, errors.New("wanted err"))
+				m.EXPECT().ListWorkloads().Return(nil, errors.New("wanted err"))
 			},
 			mockStore: func(m *mocks.Mockstore) {},
 
@@ -226,12 +225,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 		wantedInitialDBName = "mydb"
 		wantedDBEngine      = engineTypePostgreSQL
 	)
-	mockWl := config.Workload{
-		App:  "ddos",
-		Name: "frontend",
-		Type: "Load Balanced Web Service",
-	}
-	mockError := errors.New("some error")
 	testCases := map[string]struct {
 		inAppName     string
 		inStorageType string
@@ -248,27 +241,12 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 
 		mockPrompt func(m *mocks.Mockprompter)
 		mockCfg    func(m *mocks.MockwsSelector)
-		mockStore  func(m *mocks.Mockstore)
+		mockWS     func(m *mocks.MockwsAddonManager)
 
 		wantedErr error
 
 		wantedVars *initStorageVars
 	}{
-		"error if fail to get workload type": {
-			inAppName:     wantedAppName,
-			inStorageName: wantedBucketName,
-			inStorageType: s3StorageType,
-
-			mockPrompt: func(m *mocks.Mockprompter) {},
-			mockCfg: func(m *mocks.MockwsSelector) {
-				m.EXPECT().Workload(gomock.Eq(storageInitSvcPrompt), gomock.Any()).Return(wantedSvcName, nil)
-			},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(nil, mockError)
-			},
-
-			wantedErr: fmt.Errorf("get workload: some error"),
-		},
 		"Asks for storage type": {
 			inAppName:     wantedAppName,
 			inSvcName:     wantedSvcName,
@@ -292,9 +270,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				m.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Eq(options), gomock.Any()).Return(s3StorageType, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: nil,
 		},
@@ -306,8 +281,7 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 			mockPrompt: func(m *mocks.Mockprompter) {
 				m.EXPECT().SelectOption(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any()).Return("", errors.New("some error"))
 			},
-			mockCfg:   func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {},
+			mockCfg: func(m *mocks.MockwsSelector) {},
 
 			wantedErr: fmt.Errorf("select storage type: some error"),
 		},
@@ -320,9 +294,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 			mockCfg: func(m *mocks.MockwsSelector) {
 				m.EXPECT().Workload(gomock.Eq(storageInitSvcPrompt), gomock.Any()).Return(wantedSvcName, nil)
 			},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: nil,
 		},
@@ -335,7 +306,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 			mockCfg: func(m *mocks.MockwsSelector) {
 				m.EXPECT().Workload(gomock.Any(), gomock.Any()).Return("", errors.New("some error"))
 			},
-			mockStore: func(m *mocks.Mockstore) {},
 
 			wantedErr: fmt.Errorf("retrieve local workload names: some error"),
 		},
@@ -356,9 +326,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return(wantedBucketName, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: nil,
 		},
@@ -378,8 +345,8 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return(wantedBucketName, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
+			mockWS: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return(workspace.WorkloadManifest("type: Load Balanced Web Service"), nil)
 			},
 
 			wantedErr: nil,
@@ -400,9 +367,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				m.EXPECT().Get(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any()).Return("", errors.New("some error"))
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: fmt.Errorf("input storage name: some error"),
 		},
@@ -432,9 +396,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return(ddbStringType, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: nil,
 		},
@@ -451,11 +412,7 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 					gomock.Any(),
 				).Return("", errors.New("some error"))
 			},
-			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
-
+			mockCfg:   func(m *mocks.MockwsSelector) {},
 			wantedErr: fmt.Errorf("get DDB partition key: some error"),
 		},
 		"error if fail to return partition key type": {
@@ -478,9 +435,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return("", errors.New("some error"))
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: fmt.Errorf("get DDB partition key datatype: some error"),
 		},
@@ -515,9 +469,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return(ddbStringType, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: nil,
 		},
@@ -535,11 +486,7 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 					gomock.Any(),
 				).Return(false, errors.New("some error"))
 			},
-			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
-
+			mockCfg:   func(m *mocks.MockwsSelector) {},
 			wantedErr: fmt.Errorf("confirm DDB sort key: some error"),
 		},
 		"error if fail to return sort key": {
@@ -562,9 +509,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return("", errors.New("some error"))
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: fmt.Errorf("get DDB sort key: some error"),
 		},
@@ -593,9 +537,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return("", errors.New("some error"))
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: fmt.Errorf("get DDB sort key datatype: some error"),
 		},
@@ -610,9 +551,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 
 			mockPrompt: func(m *mocks.Mockprompter) {},
 			mockCfg:    func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: nil,
 		},
@@ -627,10 +565,7 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 
 			mockPrompt: func(m *mocks.Mockprompter) {},
 			mockCfg:    func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
-			wantedErr: nil,
+			wantedErr:  nil,
 		},
 		"don't ask about LSI if no-sort is specified": {
 			inAppName:     wantedAppName,
@@ -642,9 +577,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 
 			mockPrompt: func(m *mocks.Mockprompter) {},
 			mockCfg:    func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: nil,
 		},
@@ -683,10 +615,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return(false, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
-
 			wantedVars: &initStorageVars{
 				storageName:  wantedTableName,
 				workloadName: wantedSvcName,
@@ -714,10 +642,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return(false, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
-
 			wantedVars: &initStorageVars{
 				storageName:  wantedTableName,
 				workloadName: wantedSvcName,
@@ -743,9 +667,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return(false, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedVars: &initStorageVars{
 				storageName:  wantedTableName,
@@ -778,9 +699,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return("", errors.New("some error"))
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: fmt.Errorf("get DDB alternate sort key name: some error"),
 		},
@@ -800,9 +718,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				).Return(false, errors.New("some error"))
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: fmt.Errorf("confirm add alternate sort key: some error"),
 		},
@@ -833,9 +748,6 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
 
 			wantedErr: fmt.Errorf("get DDB alternate sort key type: some error"),
 		},
@@ -850,11 +762,7 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 
 			mockPrompt: func(m *mocks.Mockprompter) {},
 			mockCfg:    func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
-			},
-
-			wantedErr: nil,
+			wantedErr:  nil,
 		},
 		"asks for engine if not specified": {
 			inAppName:     wantedAppName,
@@ -869,8 +777,8 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 					Return(wantedDBEngine, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
+			mockWS: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return(workspace.WorkloadManifest("type: Load Balanced Web Service"), nil)
 			},
 
 			wantedVars: &initStorageVars{
@@ -894,10 +802,9 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 					Return("", errors.New("some error"))
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
+			mockWS: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return(workspace.WorkloadManifest("type: Load Balanced Web Service"), nil)
 			},
-
 			wantedErr: errors.New("select database engine: some error"),
 		},
 		"asks for initial database name": {
@@ -913,8 +820,8 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 					Return(wantedInitialDBName, nil)
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
+			mockWS: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return(workspace.WorkloadManifest("type: Load Balanced Web Service"), nil)
 			},
 
 			wantedVars: &initStorageVars{
@@ -938,8 +845,8 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 					Return("", errors.New("some error"))
 			},
 			mockCfg: func(m *mocks.MockwsSelector) {},
-			mockStore: func(m *mocks.Mockstore) {
-				m.EXPECT().GetWorkload(wantedAppName, wantedSvcName).Return(&mockWl, nil)
+			mockWS: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return(workspace.WorkloadManifest("type: Load Balanced Web Service"), nil)
 			},
 
 			wantedErr: fmt.Errorf("input initial database name: some error"),
@@ -953,7 +860,7 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 
 			mockPrompt := mocks.NewMockprompter(ctrl)
 			mockConfig := mocks.NewMockwsSelector(ctrl)
-			mockStore := mocks.NewMockstore(ctrl)
+			mockWS := mocks.NewMockwsAddonManager(ctrl)
 			opts := initStorageOpts{
 				initStorageVars: initStorageVars{
 					storageType:  tc.inStorageType,
@@ -971,11 +878,13 @@ func TestStorageInitOpts_Ask(t *testing.T) {
 				appName: tc.inAppName,
 				sel:     mockConfig,
 				prompt:  mockPrompt,
-				store:   mockStore,
+				ws:      mockWS,
 			}
 			tc.mockPrompt(mockPrompt)
 			tc.mockCfg(mockConfig)
-			tc.mockStore(mockStore)
+			if tc.mockWS != nil {
+				tc.mockWS(mockWS)
+			}
 			// WHEN
 			err := opts.Ask()
 
@@ -1028,6 +937,7 @@ func TestStorageInitOpts_Execute(t *testing.T) {
 			inStorageName: "my-bucket",
 
 			mockWs: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return([]byte("type: Worker Service"), nil)
 				m.EXPECT().WriteAddon(gomock.Any(), wantedSvcName, "my-bucket").Return("/frontend/addons/my-bucket.yml", nil)
 			},
 
@@ -1043,6 +953,7 @@ func TestStorageInitOpts_Execute(t *testing.T) {
 			inPartition:   wantedPartitionKey,
 
 			mockWs: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return([]byte("type: Backend Service"), nil)
 				m.EXPECT().WriteAddon(gomock.Any(), wantedSvcName, "my-table").Return("/frontend/addons/my-table.yml", nil)
 			},
 
@@ -1058,20 +969,21 @@ func TestStorageInitOpts_Execute(t *testing.T) {
 			inLSISorts:    []string{"goodness:Number"},
 
 			mockWs: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return([]byte("type: Backend Service"), nil)
 				m.EXPECT().WriteAddon(gomock.Any(), wantedSvcName, "my-table").Return("/frontend/addons/my-table.yml", nil)
 			},
 
 			wantedErr: nil,
 		},
-		"happy calls for RDS": {
-			inSvcName: wantedSvcName,
-
+		"happy calls for RDS with LBWS": {
+			inSvcName:        wantedSvcName,
 			inStorageType:    rdsStorageType,
 			inStorageName:    "mycluster",
 			inEngine:         engineTypeMySQL,
 			inParameterGroup: "mygroup",
 
 			mockWs: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return([]byte("type: Load Balanced Web Service"), nil)
 				m.EXPECT().WriteAddon(gomock.Any(), wantedSvcName, "mycluster").Return("/frontend/addons/mycluster.yml", nil)
 			},
 			mockStore: func(m *mocks.Mockstore) {
@@ -1079,6 +991,23 @@ func TestStorageInitOpts_Execute(t *testing.T) {
 			},
 			wantedErr: nil,
 		},
+		"happy calls for RDS with a RDWS": {
+			inSvcName:        wantedSvcName,
+			inStorageType:    rdsStorageType,
+			inStorageName:    "mycluster",
+			inEngine:         engineTypeMySQL,
+			inParameterGroup: "mygroup",
+
+			mockWs: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return([]byte("type: Request-Driven Web Service"), nil)
+				m.EXPECT().WriteAddon(gomock.Any(), wantedSvcName, "mycluster").Return("/frontend/addons/mycluster.yml", nil)
+				m.EXPECT().WriteAddon(gomock.Any(), wantedSvcName, "addons.parameters").Return("/frontend/addons/addons.parameters.yml", nil)
+			},
+			mockStore: func(m *mocks.Mockstore) {
+				m.EXPECT().ListEnvironments(gomock.Any()).AnyTimes()
+			},
+			wantedErr: nil,
+		},
 		"error addon exists": {
 			inAppName:     wantedAppName,
 			inStorageType: s3StorageType,
@@ -1086,21 +1015,35 @@ func TestStorageInitOpts_Execute(t *testing.T) {
 			inStorageName: "my-bucket",
 
 			mockWs: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return([]byte("type: Load Balanced Web Service"), nil)
 				m.EXPECT().WriteAddon(gomock.Any(), wantedSvcName, "my-bucket").Return("", fileExistsError)
 			},
 			mockStore: func(m *mocks.Mockstore) {
 				m.EXPECT().ListEnvironments(gomock.Any()).AnyTimes()
 			},
 
-			wantedErr: fmt.Errorf("addon already exists: %w", fileExistsError),
+			wantedErr: fmt.Errorf("addon file already exists: %w", fileExistsError),
 		},
-		"unrecognized error handled": {
+		"unexpected read workload manifest error handled": {
 			inAppName:     wantedAppName,
 			inStorageType: s3StorageType,
 			inSvcName:     wantedSvcName,
 			inStorageName: "my-bucket",
 
 			mockWs: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return(nil, errors.New("some error"))
+			},
+
+			wantedErr: errors.New("read manifest for frontend: some error"),
+		},
+		"unexpected write addon error handled": {
+			inAppName:     wantedAppName,
+			inStorageType: s3StorageType,
+			inSvcName:     wantedSvcName,
+			inStorageName: "my-bucket",
+
+			mockWs: func(m *mocks.MockwsAddonManager) {
+				m.EXPECT().ReadWorkloadManifest(wantedSvcName).Return([]byte("type: Load Balanced Web Service"), nil)
 				m.EXPECT().WriteAddon(gomock.Any(), wantedSvcName, "my-bucket").Return("", errors.New("some error"))
 			},
 
