@@ -5,8 +5,11 @@ package cli
 
 import (
 	"fmt"
+	"path/filepath"
+	"strings"
 
 	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/aws/session"
 	"github.com/aws/aws-sdk-go/service/ssm"
 	"github.com/aws/copilot-cli/internal/pkg/aws/identity"
 	"github.com/aws/copilot-cli/internal/pkg/aws/tags"
@@ -47,10 +50,9 @@ type deploySvcOpts struct {
 
 	store                store
 	ws                   wsWlDirReader
-	unmarshal            func([]byte) (manifest.WorkloadManifest, error)
+	unmarshal            func([]byte) (manifest.DynamicWorkload, error)
 	newInterpolator      func(app, env string) interpolator
-	cmd                  runner
-	envUpgradeCmd        actionCommand
+	cmd                  execRunner
 	sessProvider         *sessions.Provider
 	newSvcDeployer       func() (workloadDeployer, error)
 	envFeaturesDescriber versionCompatibilityChecker
@@ -60,12 +62,13 @@ type deploySvcOpts struct {
 	prompt  prompter
 
 	// cached variables
-	targetApp       *config.Application
-	targetEnv       *config.Environment
-	svcType         string
-	appliedManifest interface{}
-	rootUserARN     string
-	deployRecs      clideploy.ActionRecommender
+	targetApp         *config.Application
+	targetEnv         *config.Environment
+	envSess           *session.Session
+	svcType           string
+	appliedDynamicMft manifest.DynamicWorkload
+	rootUserARN       string
+	deployRecs        clideploy.ActionRecommender
 }
 
 func newSvcDeployOpts(vars deployWkldVars) (*deploySvcOpts, error) {
@@ -82,6 +85,7 @@ func newSvcDeployOpts(vars deployWkldVars) (*deploySvcOpts, error) {
 
 	store := config.NewSSMStore(identity.New(defaultSession), ssm.New(defaultSession), aws.StringValue(defaultSession.Config.Region))
 	prompter := prompt.New()
+
 	opts := &deploySvcOpts{
 		deployWkldVars: vars,
 
@@ -107,6 +111,11 @@ func newSvcDeployer(o *deploySvcOpts) (workloadDeployer, error) {
 	if err != nil {
 		return nil, err
 	}
+	raw, err := o.ws.ReadWorkloadManifest(o.name)
+	if err != nil {
+		return nil, fmt.Errorf("read manifest file for %s: %w", o.name, err)
+	}
+	content := o.appliedDynamicMft.Manifest()
 	var deployer workloadDeployer
 	in := clideploy.WorkloadDeployerInput{
 		SessionProvider: o.sessProvider,
@@ -114,11 +123,12 @@ func newSvcDeployer(o *deploySvcOpts) (workloadDeployer, error) {
 		App:             targetApp,
 		Env:             o.targetEnv,
 		ImageTag:        o.imageTag,
-		Mft:             o.appliedManifest,
+		Mft:             content,
+		RawMft:          raw,
 	}
-	switch t := o.appliedManifest.(type) {
+	switch t := content.(type) {
 	case *manifest.LoadBalancedWebService:
-		deployer, err = clideploy.NewLBDeployer(&in)
+		deployer, err = clideploy.NewLBWSDeployer(&in)
 	case *manifest.BackendService:
 		deployer, err = clideploy.NewBackendDeployer(&in)
 	case *manifest.RequestDrivenWebService:
@@ -171,9 +181,6 @@ func (o *deploySvcOpts) Execute() error {
 			return err
 		}
 	}
-	if err := o.envUpgradeCmd.Execute(); err != nil {
-		return fmt.Errorf(`execute "env upgrade --app %s --name %s": %v`, o.appName, o.envName, err)
-	}
 	mft, err := workloadManifest(&workloadManifestInput{
 		name:         o.name,
 		appName:      o.appName,
@@ -181,11 +188,15 @@ func (o *deploySvcOpts) Execute() error {
 		interpolator: o.newInterpolator(o.appName, o.envName),
 		ws:           o.ws,
 		unmarshal:    o.unmarshal,
+		sess:         o.envSess,
 	})
 	if err != nil {
 		return err
 	}
-	o.appliedManifest = mft
+	o.appliedDynamicMft = mft
+	if err := validateWorkloadManifestCompatibilityWithEnv(o.ws, o.envFeaturesDescriber, mft, o.envName); err != nil {
+		return err
+	}
 	deployer, err := o.newSvcDeployer()
 	if err != nil {
 		return err
@@ -209,11 +220,12 @@ func (o *deploySvcOpts) Execute() error {
 	}
 	deployRecs, err := deployer.DeployWorkload(&clideploy.DeployWorkloadInput{
 		StackRuntimeConfiguration: clideploy.StackRuntimeConfiguration{
-			ImageDigest: uploadOut.ImageDigest,
-			EnvFileARN:  uploadOut.EnvFileARN,
-			AddonsURL:   uploadOut.AddonsURL,
-			RootUserARN: o.rootUserARN,
-			Tags:        tags.Merge(targetApp.Tags, o.resourceTags),
+			ImageDigest:        uploadOut.ImageDigest,
+			EnvFileARN:         uploadOut.EnvFileARN,
+			AddonsURL:          uploadOut.AddonsURL,
+			RootUserARN:        o.rootUserARN,
+			Tags:               tags.Merge(targetApp.Tags, o.resourceTags),
+			CustomResourceURLs: uploadOut.CustomResourceURLs,
 		},
 		Options: clideploy.Options{
 			ForceNewUpdate:  o.forceNewUpdate,
@@ -314,20 +326,16 @@ func (o *deploySvcOpts) configureClients() error {
 	}
 	o.svcType = svc.Type
 
-	cmd, err := newEnvUpgradeOpts(envUpgradeVars{
-		appName: o.appName,
-		name:    env.Name,
-	})
-	if err != nil {
-		return fmt.Errorf("new env upgrade command: %v", err)
-	}
-	o.envUpgradeCmd = cmd
-
 	// client to retrieve an application's resources created with CloudFormation.
 	defaultSess, err := o.sessProvider.Default()
 	if err != nil {
 		return fmt.Errorf("create default session: %w", err)
 	}
+	envSess, err := o.sessProvider.FromRole(env.ManagerRoleARN, env.Region)
+	if err != nil {
+		return err
+	}
+	o.envSess = envSess
 
 	// client to retrieve caller identity.
 	caller, err := identity.New(defaultSess).Get()
@@ -336,6 +344,15 @@ func (o *deploySvcOpts) configureClients() error {
 	}
 	o.rootUserARN = caller.RootUserARN
 
+	envDescriber, err := describe.NewEnvDescriber(describe.NewEnvDescriberConfig{
+		App:         o.appName,
+		Env:         o.envName,
+		ConfigStore: o.store,
+	})
+	if err != nil {
+		return err
+	}
+	o.envFeaturesDescriber = envDescriber
 	return nil
 }
 
@@ -345,10 +362,11 @@ type workloadManifestInput struct {
 	envName      string
 	ws           wsWlDirReader
 	interpolator interpolator
-	unmarshal    func([]byte) (manifest.WorkloadManifest, error)
+	sess         *session.Session
+	unmarshal    func([]byte) (manifest.DynamicWorkload, error)
 }
 
-func workloadManifest(in *workloadManifestInput) (interface{}, error) {
+func workloadManifest(in *workloadManifestInput) (manifest.DynamicWorkload, error) {
 	raw, err := in.ws.ReadWorkloadManifest(in.name)
 	if err != nil {
 		return nil, fmt.Errorf("read manifest file for %s: %w", in.name, err)
@@ -363,15 +381,18 @@ func workloadManifest(in *workloadManifestInput) (interface{}, error) {
 	}
 	envMft, err := mft.ApplyEnv(in.envName)
 	if err != nil {
-		return nil, fmt.Errorf("apply environment %s override: %s", in.envName, err)
+		return nil, fmt.Errorf("apply environment %s override: %w", in.envName, err)
 	}
 	if err := envMft.Validate(); err != nil {
-		return nil, fmt.Errorf("validate manifest against environment %s: %s", in.envName, err)
+		return nil, fmt.Errorf("validate manifest against environment %q: %w", in.envName, err)
+	}
+	if err := envMft.Load(in.sess); err != nil {
+		return nil, fmt.Errorf("load dynamic content: %w", err)
 	}
 	return envMft, nil
 }
 
-func isManifestCompatibleWithEnvironment(mft manifest.WorkloadManifest, envName string, env versionCompatibilityChecker) error {
+func validateWorkloadManifestCompatibilityWithEnv(ws wsEnvironmentsLister, env versionCompatibilityChecker, mft manifest.DynamicWorkload, envName string) error {
 	availableFeatures, err := env.AvailableFeatures()
 	if err != nil {
 		return fmt.Errorf("get available features of the %s environment stack: %w", envName, err)
@@ -387,14 +408,19 @@ func isManifestCompatibleWithEnvironment(mft manifest.WorkloadManifest, envName
 		if _, ok := available[f]; !ok {
 			logMsg := fmt.Sprintf(`Your manifest configuration requires your environment %q to have the feature %q available.`, envName, template.FriendlyEnvFeatureName(f))
 			if v := template.LeastVersionForFeature(f); v != "" {
-				logMsg += fmt.Sprintf(`The least environment version that supports the feature is %s.`, v)
+				logMsg += fmt.Sprintf(` The least environment version that supports the feature is %s.`, v)
 			}
-			if currVersion, err := env.Version(); err == nil {
-				logMsg += fmt.Sprintf(" Your environment is on %s.\n", currVersion)
+			currVersion, err := env.Version()
+			if err == nil {
+				logMsg += fmt.Sprintf(" Your environment is on %s.", currVersion)
 			}
-			logMsg += fmt.Sprintf(`Please upgrade your environment by running %s.`, color.HighlightCode(fmt.Sprintf("copilot env deploy --name %s", envName)))
 			log.Errorln(logMsg)
-			return fmt.Errorf("environment %q is not on a version that supports the %q feature", envName, template.FriendlyEnvFeatureName(f))
+			return &errFeatureIncompatibleWithEnvironment{
+				ws:             ws,
+				missingFeature: f,
+				envName:        envName,
+				curVersion:     currVersion,
+			}
 		}
 	}
 	return nil
@@ -404,7 +430,7 @@ func (o *deploySvcOpts) uriRecommendedActions() ([]string, error) {
 	type reachable interface {
 		Port() (uint16, bool)
 	}
-	mft, ok := o.appliedManifest.(reachable)
+	mft, ok := o.appliedDynamicMft.Manifest().(reachable)
 	if !ok {
 		return nil, nil
 	}
@@ -438,7 +464,7 @@ func (o *deploySvcOpts) publishRecommendedActions() []string {
 	type publisher interface {
 		Publish() []manifest.Topic
 	}
-	mft, ok := o.appliedManifest.(publisher)
+	mft, ok := o.appliedDynamicMft.Manifest().(publisher)
 	if !ok {
 		return nil
 	}
@@ -466,6 +492,42 @@ func (o *deploySvcOpts) getTargetApp() (*config.Application, error) {
 	return o.targetApp, nil
 }
 
+type errFeatureIncompatibleWithEnvironment struct {
+	ws             wsEnvironmentsLister
+	missingFeature string
+	envName        string
+	curVersion     string
+}
+
+func (e *errFeatureIncompatibleWithEnvironment) Error() string {
+	if e.curVersion == "" {
+		return fmt.Sprintf("environment %q is not on a version that supports the %q feature", e.envName, template.FriendlyEnvFeatureName(e.missingFeature))
+	}
+	return fmt.Sprintf("environment %q is on version %q which does not support the %q feature", e.envName, e.curVersion, template.FriendlyEnvFeatureName(e.missingFeature))
+}
+
+// RecommendActions returns recommended actions to be taken after the error.
+// Implements main.actionRecommender interface.
+func (e *errFeatureIncompatibleWithEnvironment) RecommendActions() string {
+	envs, _ := e.ws.ListEnvironments() // Best effort try to detect if env manifest exists.
+	for _, env := range envs {
+		if e.envName == env {
+			return fmt.Sprintf("You can upgrade the %q environment template by running %s.", e.envName, color.HighlightCode(fmt.Sprintf("copilot env deploy --name %s", e.envName)))
+		}
+	}
+	msgs := []string{
+		"You can upgrade your environment template by running:",
+		fmt.Sprintf("1. Create the directory to store your environment manifest %s.",
+			color.HighlightCode(fmt.Sprintf("mkdir -p %s", filepath.Join("copilot", "environments", e.envName)))),
+		fmt.Sprintf("2. Generate the manifest %s.",
+			color.HighlightCode(fmt.Sprintf("copilot env show -n %s --manifest > %s", e.envName, filepath.Join("copilot", "environments", e.envName, "manifest.yml")))),
+		fmt.Sprintf("3. Deploy the environment stack %s.",
+			color.HighlightCode(fmt.Sprintf("copilot env deploy --name %s", e.envName))),
+	}
+	return strings.Join(msgs, "\n")
+
+}
+
 // buildSvcDeployCmd builds the `svc deploy` subcommand.
 func buildSvcDeployCmd() *cobra.Command {
 	vars := deployWkldVars{}
