@@ -130,7 +130,8 @@ type initSvcOpts struct {
 	wsPendingCreation bool
 
 	// Cache variables
-	df dockerfileParser
+	df             dockerfileParser
+	manifestExists bool
 
 	// Init a Dockerfile parser using fs and input path
 	dockerfile func(string) dockerfileParser
@@ -288,7 +289,7 @@ func (o *initSvcOpts) Execute() error {
 		}
 	}
 	// If the user passes in an image, their docker engine isn't necessarily running, and we can't do anything with the platform because we're not building the Docker image.
-	if o.image == "" {
+	if o.image == "" && !o.manifestExists {
 		platform, err := legitimizePlatform(o.dockerEngine, o.wkldType)
 		if err != nil {
 			return err
@@ -392,7 +393,7 @@ func (o *initSvcOpts) askImage() error {
 		return nil
 	}
 
-	var validator prompt.ValidatorFunc
+	validator := prompt.RequireNonEmpty
 	promptHelp := wkldInitImagePromptHelp
 	if o.wkldType == manifest.RequestDrivenWebServiceType {
 		promptHelp = wkldInitAppRunnerImagePromptHelp
@@ -424,6 +425,8 @@ func (o *initSvcOpts) shouldSkipAsking() (bool, error) {
 		}
 		return false, nil
 	}
+	o.manifestExists = true
+
 	svcType, err := localMft.WorkloadType()
 	if err != nil {
 		return false, fmt.Errorf(`read "type" field for service %s from local manifest: %w`, o.name, err)
@@ -531,6 +534,21 @@ func (o *initSvcOpts) askSvcPort() (err error) {
 }
 
 func legitimizePlatform(engine dockerEngine, wkldType string) (manifest.PlatformString, error) {
+	if err := engine.CheckDockerEngineRunning(); err != nil {
+		// This is a best-effort attempt to detect the platform for users.
+		// If docker is not available, we skip this information.
+		var errDaemon *dockerengine.ErrDockerDaemonNotResponsive
+		switch {
+		case errors.Is(err, dockerengine.ErrDockerCommandNotFound):
+			log.Info("Docker command is not found; Copilot won't detect and populate the \"platform\" field in the manifest.\n")
+			return "", nil
+		case errors.As(err, &errDaemon):
+			log.Info("Docker daemon is not responsive; Copilot won't detect and populate the \"platform\" field in the manifest.\n")
+			return "", nil
+		default:
+			return "", fmt.Errorf("check if docker engine is running: %w", err)
+		}
+	}
 	detectedOs, detectedArch, err := engine.GetPlatform()
 	if err != nil {
 		return "", fmt.Errorf("get docker engine platform: %w", err)
