@@ -4,26 +4,22 @@
 package cli
 
 import (
-	"errors"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 
-	"github.com/aws/aws-sdk-go/service/ssm"
-
 	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/aws/session"
+	"github.com/aws/aws-sdk-go/service/ssm"
 	clideploy "github.com/aws/copilot-cli/internal/pkg/cli/deploy"
+	"github.com/aws/copilot-cli/internal/pkg/deploy"
+	"github.com/aws/copilot-cli/internal/pkg/describe"
 	"github.com/aws/copilot-cli/internal/pkg/exec"
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
-
-	"github.com/aws/copilot-cli/internal/pkg/deploy"
-
 	"github.com/spf13/afero"
 	"github.com/spf13/cobra"
 
-	"github.com/aws/copilot-cli/internal/pkg/addon"
 	"github.com/aws/copilot-cli/internal/pkg/aws/identity"
 	"github.com/aws/copilot-cli/internal/pkg/aws/sessions"
 	"github.com/aws/copilot-cli/internal/pkg/config"
@@ -37,15 +33,6 @@ const (
 	svcPackageEnvNamePrompt = "Which environment would you like to package this stack for?"
 )
 
-var initPackageAddonsClient = func(o *packageSvcOpts) error {
-	addonsClient, err := addon.New(o.name)
-	if err != nil {
-		return fmt.Errorf("new addons client: %w", err)
-	}
-	o.addonsClient = addonsClient
-	return nil
-}
-
 type packageSvcVars struct {
 	name         string
 	envName      string
@@ -62,26 +49,26 @@ type packageSvcOpts struct {
 	packageSvcVars
 
 	// Interfaces to interact with dependencies.
-	addonsClient     templater
-	initAddonsClient func(*packageSvcOpts) error // Overridden in tests.
-	ws               wsWlDirReader
-	fs               afero.Fs
-	store            store
-	stackWriter      io.Writer
-	paramsWriter     io.Writer
-	addonsWriter     io.Writer
-	runner           runner
-	sessProvider     *sessions.Provider
-	sel              wsSelector
-	unmarshal        func([]byte) (manifest.WorkloadManifest, error)
-	newInterpolator  func(app, env string) interpolator
-	newTplGenerator  func(*packageSvcOpts) (workloadTemplateGenerator, error)
+	ws                   wsWlDirReader
+	fs                   afero.Fs
+	store                store
+	templateWriter       io.WriteCloser
+	paramsWriter         io.WriteCloser
+	addonsWriter         io.WriteCloser
+	runner               execRunner
+	sessProvider         *sessions.Provider
+	sel                  wsSelector
+	unmarshal            func([]byte) (manifest.DynamicWorkload, error)
+	newInterpolator      func(app, env string) interpolator
+	newStackGenerator    func(*packageSvcOpts) (workloadStackGenerator, error)
+	envFeaturesDescriber versionCompatibilityChecker
 
 	// cached variables
-	targetApp       *config.Application
-	targetEnv       *config.Environment
-	appliedManifest interface{}
-	rootUserARN     string
+	targetApp         *config.Application
+	targetEnv         *config.Environment
+	envSess           *session.Session
+	appliedDynamicMft manifest.DynamicWorkload
+	rootUserARN       string
 }
 
 func newPackageSvcOpts(vars packageSvcVars) (*packageSvcOpts, error) {
@@ -99,25 +86,24 @@ func newPackageSvcOpts(vars packageSvcVars) (*packageSvcOpts, error) {
 	store := config.NewSSMStore(identity.New(defaultSess), ssm.New(defaultSess), aws.StringValue(defaultSess.Config.Region))
 	prompter := prompt.New()
 	opts := &packageSvcOpts{
-		packageSvcVars:   vars,
-		initAddonsClient: initPackageAddonsClient,
-		store:            store,
-		ws:               ws,
-		fs:               &afero.Afero{Fs: afero.NewOsFs()},
-		unmarshal:        manifest.UnmarshalWorkload,
-		runner:           exec.NewCmd(),
-		sel:              selector.NewLocalWorkloadSelector(prompter, store, ws),
-		stackWriter:      os.Stdout,
-		paramsWriter:     ioutil.Discard,
-		addonsWriter:     ioutil.Discard,
-		newInterpolator:  newManifestInterpolator,
-		sessProvider:     sessProvider,
-		newTplGenerator:  newWkldTplGenerator,
+		packageSvcVars:    vars,
+		store:             store,
+		ws:                ws,
+		fs:                &afero.Afero{Fs: afero.NewOsFs()},
+		unmarshal:         manifest.UnmarshalWorkload,
+		runner:            exec.NewCmd(),
+		sel:               selector.NewLocalWorkloadSelector(prompter, store, ws),
+		templateWriter:    os.Stdout,
+		paramsWriter:      discardFile{},
+		addonsWriter:      discardFile{},
+		newInterpolator:   newManifestInterpolator,
+		sessProvider:      sessProvider,
+		newStackGenerator: newWorkloadStackGenerator,
 	}
 	return opts, nil
 }
 
-func newWkldTplGenerator(o *packageSvcOpts) (workloadTemplateGenerator, error) {
+func newWorkloadStackGenerator(o *packageSvcOpts) (workloadStackGenerator, error) {
 	targetApp, err := o.getTargetApp()
 	if err != nil {
 		return nil, err
@@ -126,18 +112,25 @@ func newWkldTplGenerator(o *packageSvcOpts) (workloadTemplateGenerator, error) {
 	if err != nil {
 		return nil, err
 	}
-	var deployer workloadTemplateGenerator
+	raw, err := o.ws.ReadWorkloadManifest(o.name)
+	if err != nil {
+		return nil, fmt.Errorf("read manifest file for %s: %w", o.name, err)
+	}
+
+	content := o.appliedDynamicMft.Manifest()
+	var deployer workloadStackGenerator
 	in := clideploy.WorkloadDeployerInput{
 		SessionProvider: o.sessProvider,
 		Name:            o.name,
 		App:             targetApp,
 		Env:             targetEnv,
 		ImageTag:        o.tag,
-		Mft:             o.appliedManifest,
+		Mft:             content,
+		RawMft:          raw,
 	}
-	switch t := o.appliedManifest.(type) {
+	switch t := content.(type) {
 	case *manifest.LoadBalancedWebService:
-		deployer, err = clideploy.NewLBDeployer(&in)
+		deployer, err = clideploy.NewLBWSDeployer(&in)
 	case *manifest.BackendService:
 		deployer, err = clideploy.NewBackendDeployer(&in)
 	case *manifest.RequestDrivenWebService:
@@ -195,24 +188,26 @@ func (o *packageSvcOpts) Execute() error {
 	if err != nil {
 		return nil
 	}
-	appTemplates, err := o.getSvcTemplates(targetEnv)
+	gen, err := o.getStackGenerator(targetEnv)
 	if err != nil {
 		return err
 	}
-	if _, err = o.stackWriter.Write([]byte(appTemplates.stack)); err != nil {
-		return err
-	}
-	if _, err = o.paramsWriter.Write([]byte(appTemplates.configuration)); err != nil {
-		return err
-	}
-	addonsTemplate, err := o.getAddonsTemplate()
-	// return nil if addons not found.
-	var notFoundErr *addon.ErrAddonsNotFound
-	if errors.As(err, &notFoundErr) {
-		return nil
-	}
+	stack, err := o.getWorkloadStack(gen)
 	if err != nil {
+		return err
+	}
+	if err := o.writeAndClose(o.templateWriter, stack.template); err != nil {
+		return err
+	}
+	if err := o.writeAndClose(o.paramsWriter, stack.parameters); err != nil {
+		return err
+	}
+	addonsTemplate, err := gen.AddonsTemplate()
+	switch {
+	case err != nil:
 		return fmt.Errorf("retrieve addons template: %w", err)
+	case addonsTemplate == "":
+		return nil
 	}
 	// Addons template won't show up without setting --output-dir flag.
 	if o.outputDir != "" {
@@ -220,8 +215,7 @@ func (o *packageSvcOpts) Execute() error {
 			return err
 		}
 	}
-	_, err = o.addonsWriter.Write([]byte(addonsTemplate))
-	return err
+	return o.writeAndClose(o.addonsWriter, addonsTemplate)
 }
 
 func (o *packageSvcOpts) validateOrAskSvcName() error {
@@ -258,13 +252,6 @@ func (o *packageSvcOpts) validateOrAskEnvName() error {
 	return nil
 }
 
-func (o *packageSvcOpts) getAddonsTemplate() (string, error) {
-	if err := o.initAddonsClient(o); err != nil {
-		return "", err
-	}
-	return o.addonsClient.Template()
-}
-
 func (o *packageSvcOpts) configureClients() error {
 	o.tag = imageTagFromGit(o.runner, o.tag) // Best effort assign git tag.
 	// client to retrieve an application's resources created with CloudFormation.
@@ -272,6 +259,15 @@ func (o *packageSvcOpts) configureClients() error {
 	if err != nil {
 		return fmt.Errorf("create default session: %w", err)
 	}
+	targetEnv, err := o.getTargetEnv()
+	if err != nil {
+		return err
+	}
+	envSess, err := o.sessProvider.FromRole(targetEnv.ManagerRoleARN, targetEnv.Region)
+	if err != nil {
+		return err
+	}
+	o.envSess = envSess
 	// client to retrieve caller identity.
 	caller, err := identity.New(defaultSess).Get()
 	if err != nil {
@@ -279,16 +275,24 @@ func (o *packageSvcOpts) configureClients() error {
 	}
 	o.rootUserARN = caller.RootUserARN
 
+	envDescriber, err := describe.NewEnvDescriber(describe.NewEnvDescriberConfig{
+		App:         o.appName,
+		Env:         o.envName,
+		ConfigStore: o.store,
+	})
+	if err != nil {
+		return err
+	}
+	o.envFeaturesDescriber = envDescriber
 	return nil
 }
 
-type wkldCfnTemplates struct {
-	stack         string
-	configuration string
+type cfnStackConfig struct {
+	template   string
+	parameters string
 }
 
-// getSvcTemplates returns the CloudFormation stack's template and its parameters for the service.
-func (o *packageSvcOpts) getSvcTemplates(env *config.Environment) (*wkldCfnTemplates, error) {
+func (o *packageSvcOpts) getStackGenerator(env *config.Environment) (workloadStackGenerator, error) {
 	mft, err := workloadManifest(&workloadManifestInput{
 		name:         o.name,
 		appName:      o.appName,
@@ -296,22 +300,24 @@ func (o *packageSvcOpts) getSvcTemplates(env *config.Environment) (*wkldCfnTempl
 		interpolator: o.newInterpolator(o.appName, o.envName),
 		ws:           o.ws,
 		unmarshal:    o.unmarshal,
+		sess:         o.envSess,
 	})
 	if err != nil {
 		return nil, err
 	}
-	o.appliedManifest = mft
-	generator, err := o.newTplGenerator(o)
+	o.appliedDynamicMft = mft
+	return o.newStackGenerator(o)
+}
+
+// getWorkloadStack returns the CloudFormation stack's template and its parameters for the service.
+func (o *packageSvcOpts) getWorkloadStack(generator workloadStackGenerator) (*cfnStackConfig, error) {
+	targetApp, err := o.getTargetApp()
 	if err != nil {
 		return nil, err
 	}
 	uploadOut := clideploy.UploadArtifactsOutput{
 		ImageDigest: aws.String(""),
 	}
-	targetApp, err := o.getTargetApp()
-	if err != nil {
-		return nil, err
-	}
 	if o.uploadAssets {
 		out, err := generator.UploadArtifacts()
 		if err != nil {
@@ -321,17 +327,20 @@ func (o *packageSvcOpts) getSvcTemplates(env *config.Environment) (*wkldCfnTempl
 	}
 	output, err := generator.GenerateCloudFormationTemplate(&clideploy.GenerateCloudFormationTemplateInput{
 		StackRuntimeConfiguration: clideploy.StackRuntimeConfiguration{
-			RootUserARN: o.rootUserARN,
-			Tags:        targetApp.Tags,
-			ImageDigest: uploadOut.ImageDigest,
-			EnvFileARN:  uploadOut.EnvFileARN,
-			AddonsURL:   uploadOut.AddonsURL,
+			RootUserARN:        o.rootUserARN,
+			Tags:               targetApp.Tags,
+			ImageDigest:        uploadOut.ImageDigest,
+			EnvFileARN:         uploadOut.EnvFileARN,
+			AddonsURL:          uploadOut.AddonsURL,
+			CustomResourceURLs: uploadOut.CustomResourceURLs,
 		},
 	})
 	if err != nil {
 		return nil, fmt.Errorf("generate workload %s template against environment %s: %w", o.name, o.envName, err)
 	}
-	return &wkldCfnTemplates{stack: output.Template, configuration: output.Parameters}, nil
+	return &cfnStackConfig{
+		template:   output.Template,
+		parameters: output.Parameters}, nil
 }
 
 // setOutputFileWriters creates the output directory, and updates the template and param writers to file writers in the directory.
@@ -346,7 +355,7 @@ func (o *packageSvcOpts) setOutputFileWriters() error {
 	if err != nil {
 		return fmt.Errorf("create file %s: %w", templatePath, err)
 	}
-	o.stackWriter = templateFile
+	o.templateWriter = templateFile
 
 	paramsPath := filepath.Join(o.outputDir,
 		fmt.Sprintf(deploy.WorkloadCfnTemplateConfigurationNameFormat, o.name, o.envName))
@@ -395,9 +404,16 @@ func (o *packageSvcOpts) getTargetEnv() (*config.Environment, error) {
 	return o.targetEnv, nil
 }
 
-// RecommendActions is a no-op for this command.
+func (o *packageSvcOpts) writeAndClose(wc io.WriteCloser, dat string) error {
+	if _, err := wc.Write([]byte(dat)); err != nil {
+		return err
+	}
+	return wc.Close()
+}
+
+// RecommendActions suggests recommended actions before the packaged template is used for deployment.
 func (o *packageSvcOpts) RecommendActions() error {
-	return nil
+	return validateWorkloadManifestCompatibilityWithEnv(o.ws, o.envFeaturesDescriber, o.appliedDynamicMft, o.envName)
 }
 
 func contains(s string, items []string) bool {
@@ -414,16 +430,18 @@ func buildSvcPackageCmd() *cobra.Command {
 	vars := packageSvcVars{}
 	cmd := &cobra.Command{
 		Use:   "package",
-		Short: "Prints the AWS CloudFormation template of a service.",
-		Long:  `Prints the CloudFormation template used to deploy a service to an environment.`,
+		Short: "Print the AWS CloudFormation template of a service.",
+		Long:  `Print the CloudFormation template used to deploy a service to an environment.`,
 		Example: `
   Print the CloudFormation template for the "frontend" service parametrized for the "test" environment.
   /code $ copilot svc package -n frontend -e test
 
-  Write the CloudFormation stack and configuration to a "infrastructure/" sub-directory instead of printing.
-  /code $ copilot svc package -n frontend -e test --output-dir ./infrastructure
-  /code $ ls ./infrastructure
-  /code frontend-test.stack.yml      frontend-test.params.yml`,
+  Write the CloudFormation stack and configuration to a "infrastructure/" sub-directory instead of stdout.
+  /startcodeblock
+  $ copilot svc package -n frontend -e test --output-dir ./infrastructure
+  $ ls ./infrastructure
+  frontend-test.stack.yml      frontend-test.params.json
+  /endcodeblock`,
 		RunE: runCmdE(func(cmd *cobra.Command, args []string) error {
 			opts, err := newPackageSvcOpts(vars)
 			if err != nil {
