@@ -6,6 +6,7 @@ package cli
 import (
 	"errors"
 	"fmt"
+	"github.com/aws/copilot-cli/internal/pkg/aws/s3"
 
 	"github.com/aws/aws-sdk-go/service/ssm"
 	"github.com/aws/copilot-cli/internal/pkg/aws/identity"
@@ -59,10 +60,11 @@ type deleteTaskOpts struct {
 	sel      wsSelector
 
 	// Generators for env-specific clients
-	newTaskSel      func(session *session.Session) cfTaskSelector
-	newTaskStopper  func(session *session.Session) taskStopper
-	newImageRemover func(session *session.Session) imageRemover
-	newStackManager func(session *session.Session) taskStackManager
+	newTaskSel       func(session *session.Session) cfTaskSelector
+	newTaskStopper   func(session *session.Session) taskStopper
+	newImageRemover  func(session *session.Session) imageRemover
+	newBucketEmptier func(session *session.Session) bucketEmptier
+	newStackManager  func(session *session.Session) taskStackManager
 
 	// Cached variables
 	session   *session.Session
@@ -104,6 +106,9 @@ func newDeleteTaskOpts(vars deleteTaskVars) (*deleteTaskOpts, error) {
 		newImageRemover: func(session *session.Session) imageRemover {
 			return ecr.New(session)
 		},
+		newBucketEmptier: func(session *session.Session) bucketEmptier {
+			return s3.New(session)
+		},
 	}, nil
 }
 
@@ -389,10 +394,22 @@ func (o *deleteTaskOpts) clearECRRepository() error {
 	err = o.newImageRemover(defaultSess).ClearRepository(ecrRepoName)
 	if err != nil {
 		o.spinner.Stop(log.Serrorln("Error emptying ECR repository."))
-		return fmt.Errorf("clear ECR repository for task %s: %w", o.name, err)
+		return fmt.Errorf("empty ECR repository for task %s: %w", o.name, err)
 	}
 
-	o.spinner.Stop(log.Ssuccessf("Emptied ECR repositories for task %s.\n", color.HighlightUserInput(o.name)))
+	o.spinner.Stop(log.Ssuccessf("Emptied ECR repository for task %s.\n", color.HighlightUserInput(o.name)))
+	return nil
+}
+
+func (o *deleteTaskOpts) emptyS3Bucket(info *deploy.TaskStackInfo) error {
+	o.spinner.Start(fmt.Sprintf("Emptying S3 bucket for task %s.", color.HighlightUserInput(o.name)))
+	err := o.newBucketEmptier(o.session).EmptyBucket(info.BucketName)
+	if err != nil {
+		o.spinner.Stop(log.Serrorln("Error emptying S3 bucket."))
+		return fmt.Errorf("empty S3 bucket for task %s: %w", o.name, err)
+	}
+
+	o.spinner.Stop(log.Ssuccessf("Emptied S3 bucket for task %s.\n", color.HighlightUserInput(o.name)))
 	return nil
 }
 
@@ -433,6 +450,11 @@ func (o *deleteTaskOpts) deleteStack() error {
 		// Stack does not exist; skip deleting it.
 		return nil
 	}
+	if info.BucketName != "" {
+		if err := o.emptyS3Bucket(info); err != nil {
+			return err
+		}
+	}
 	o.spinner.Start(fmt.Sprintf("Deleting CloudFormation stack for task %s.", color.HighlightUserInput(o.name)))
 	err = o.newStackManager(sess).DeleteTask(*info)
 	if err != nil {
