@@ -4,8 +4,12 @@
 package cli
 
 import (
+	"bytes"
 	"errors"
 	"fmt"
+	"github.com/aws/copilot-cli/internal/pkg/aws/partitions"
+	"github.com/aws/copilot-cli/internal/pkg/aws/s3"
+	"github.com/aws/copilot-cli/internal/pkg/template/artifactpath"
 	"os"
 	"path/filepath"
 	"strings"
@@ -58,6 +62,14 @@ const (
 	shortTaskIDLength     = 8
 )
 
+const (
+	envFileExt = ".env"
+
+	fmtTaskRunEnvUploadStart    = "Uploading env file to S3: %s"
+	fmtTaskRunEnvUploadFailed   = "Failed to upload your env file to S3: %s\n"
+	fmtTaskRunEnvUploadComplete = "Successfully uploaded your env file to S3: %s\n"
+)
+
 const (
 	workloadTypeJob     = "job"
 	workloadTypeSvc     = "svc"
@@ -112,6 +124,7 @@ type runTaskVars struct {
 	useDefaultSubnetsAndCluster bool
 
 	envVars                  map[string]string
+	envFile                  string
 	secrets                  map[string]string
 	acknowledgeSecretsAccess bool
 	command                  string
@@ -158,6 +171,7 @@ type runTaskOpts struct {
 	configureECSServiceDescriber func(session *session.Session) ecs.ECSServiceDescriber
 	configureServiceDescriber    func(session *session.Session) ecs.ServiceDescriber
 	configureJobDescriber        func(session *session.Session) ecs.JobDescriber
+	configureUploader            func(session *session.Session) uploader
 
 	// Functions to generate a task run command.
 	runTaskRequestFromECSService func(client ecs.ECSServiceDescriber, cluster, service string) (*ecs.RunTaskRequest, error)
@@ -167,6 +181,9 @@ type runTaskOpts struct {
 	// Cached variables to hold SSM Param and Secrets Manager Secrets
 	ssmParamSecrets       map[string]string
 	secretsManagerSecrets map[string]string
+
+	// Cached variable for uploaded env file ARN
+	envFileARN string
 }
 
 func newTaskRunOpts(vars runTaskVars) (*runTaskOpts, error) {
@@ -196,7 +213,7 @@ func newTaskRunOpts(vars runTaskVars) (*runTaskOpts, error) {
 		if err != nil {
 			return fmt.Errorf("configure task runner: %w", err)
 		}
-		opts.deployer = cloudformation.New(opts.sess)
+		opts.deployer = cloudformation.New(opts.sess, cloudformation.WithProgressTracker(os.Stderr))
 		opts.defaultClusterGetter = awsecs.New(opts.sess)
 		opts.publicIPGetter = ec2.New(opts.sess)
 		return nil
@@ -221,6 +238,9 @@ func newTaskRunOpts(vars runTaskVars) (*runTaskOpts, error) {
 	opts.configureJobDescriber = func(session *session.Session) ecs.JobDescriber {
 		return ecs.New(session)
 	}
+	opts.configureUploader = func(session *session.Session) uploader {
+		return s3.New(session)
+	}
 
 	opts.runTaskRequestFromECSService = ecs.RunTaskRequestFromECSService
 	opts.runTaskRequestFromService = ecs.RunTaskRequestFromService
@@ -249,6 +269,7 @@ func (o *runTaskOpts) configureRunner() (taskRunner, error) {
 			return nil, fmt.Errorf("create describer for environment %s in application %s: %w", o.env, o.appName, err)
 		}
 
+		ecsClient := ecs.New(o.sess)
 		return &task.EnvRunner{
 			Count:     o.count,
 			GroupName: o.groupName,
@@ -260,10 +281,11 @@ func (o *runTaskOpts) configureRunner() (taskRunner, error) {
 
 			OS: o.os,
 
-			VPCGetter:            vpcGetter,
-			ClusterGetter:        ecs.New(o.sess),
-			Starter:              ecsService,
-			EnvironmentDescriber: d,
+			VPCGetter:             vpcGetter,
+			ClusterGetter:         ecsClient,
+			Starter:               ecsService,
+			EnvironmentDescriber:  d,
+			NonZeroExitCodeGetter: ecsClient,
 		}, nil
 	}
 	return &task.ConfigRunner{
@@ -275,9 +297,10 @@ func (o *runTaskOpts) configureRunner() (taskRunner, error) {
 		SecurityGroups: o.securityGroups,
 		OS:             o.os,
 
-		VPCGetter:     vpcGetter,
-		ClusterGetter: ecsService,
-		Starter:       ecsService,
+		VPCGetter:             vpcGetter,
+		ClusterGetter:         ecsService,
+		Starter:               ecsService,
+		NonZeroExitCodeGetter: ecs.New(o.sess),
 	}, nil
 
 }
@@ -397,6 +420,12 @@ func (o *runTaskOpts) Validate() error {
 		}
 	}
 
+	if o.envFile != "" {
+		if filepath.Ext(o.envFile) != envFileExt {
+			return fmt.Errorf("environment file %s specified in --%s must have a %s file extension", o.envFile, envFileFlag, envFileExt)
+		}
+	}
+
 	return nil
 }
 
@@ -644,6 +673,18 @@ func (o *runTaskOpts) Execute() error {
 		return err
 	}
 
+	var shouldUpdate bool
+
+	if o.envFile != "" {
+		envFileARN, err := o.deployEnvFile()
+		if err != nil {
+			return fmt.Errorf("deploy env file %s: %w", o.envFile, err)
+		}
+		o.envFileARN = envFileARN
+
+		shouldUpdate = true
+	}
+
 	// NOTE: if image is not provided, then we build the image and push to ECR repo
 	if o.image == "" {
 		if err := o.buildAndPushImage(); err != nil {
@@ -659,6 +700,10 @@ func (o *runTaskOpts) Execute() error {
 			return fmt.Errorf("get ECR repository URI: %w", err)
 		}
 		o.image = fmt.Sprintf(fmtImageURI, uri, tag)
+		shouldUpdate = true
+	}
+
+	if shouldUpdate {
 		if err := o.updateTaskResources(); err != nil {
 			return err
 		}
@@ -681,6 +726,9 @@ Did you tag your secrets with the "copilot-application" and "copilot-environment
 		if err := o.displayLogStream(); err != nil {
 			return err
 		}
+		if err := o.runner.CheckNonZeroExitCode(tasks); err != nil {
+			return err
+		}
 	}
 	return nil
 }
@@ -690,7 +738,11 @@ func (o *runTaskOpts) generateCommand() error {
 	if err != nil {
 		return err
 	}
-	log.Infoln(command.CLIString())
+	cliString, err := command.CLIString()
+	if err != nil {
+		return err
+	}
+	log.Infoln(cliString)
 	return nil
 }
 
@@ -924,6 +976,7 @@ func (o *runTaskOpts) deploy() error {
 		Command:               command,
 		EntryPoint:            entrypoint,
 		EnvVars:               o.envVars,
+		EnvFileARN:            o.envFileARN,
 		SSMParamSecrets:       ssmParamSecrets,
 		SecretsManagerSecrets: secretsManagerSecrets,
 		OS:                    o.os,
@@ -932,7 +985,55 @@ func (o *runTaskOpts) deploy() error {
 		Env:                   o.env,
 		AdditionalTags:        o.resourceTags,
 	}
-	return o.deployer.DeployTask(os.Stderr, input, deployOpts...)
+	return o.deployer.DeployTask(input, deployOpts...)
+}
+
+// deployEnvFileIfNeeded uploads the env file if needed, ensures that an S3 bucket is available, and returns the ARN of uploaded file.
+func (o *runTaskOpts) deployEnvFile() (string, error) {
+	if o.envFile == "" {
+		return "", nil
+	}
+
+	info, err := o.deployer.GetTaskStack(o.groupName)
+	if err != nil {
+		return "", fmt.Errorf("deploy env file: %w", err)
+	}
+
+	// push env file
+	o.spinner.Start(fmt.Sprintf(fmtTaskRunEnvUploadStart, color.HighlightUserInput(o.envFile)))
+	envFileARN, err := o.pushEnvFileToS3(info.BucketName)
+	if err != nil {
+		o.spinner.Stop(log.Serrorf(fmtTaskRunEnvUploadFailed, color.HighlightUserInput(o.envFile)))
+		return "", err
+	}
+	o.spinner.Stop(log.Ssuccessf(fmtTaskRunEnvUploadComplete, color.HighlightUserInput(o.envFile)))
+
+	return envFileARN, nil
+}
+
+// pushEnvFileToS3 reads an env file from disk, uploads it to a unique path, and then returns the ARN of the env file.
+func (o *runTaskOpts) pushEnvFileToS3(bucket string) (string, error) {
+	content, err := afero.ReadFile(o.fs, o.envFile)
+	if err != nil {
+		return "", fmt.Errorf("read env file %s: %w", o.envFile, err)
+	}
+	reader := bytes.NewReader(content)
+
+	uploader := o.configureUploader(o.sess)
+	url, err := uploader.Upload(bucket, artifactpath.EnvFiles(o.envFile, content), reader)
+	if err != nil {
+		return "", fmt.Errorf("put env file %s artifact to bucket %s: %w", o.envFile, bucket, err)
+	}
+	bucket, key, err := s3.ParseURL(url)
+	if err != nil {
+		return "", fmt.Errorf("parse s3 url: %w", err)
+	}
+	// The app and environment are always within the same partition.
+	partition, err := partitions.Region(aws.StringValue(o.sess.Config.Region)).Partition()
+	if err != nil {
+		return "", err
+	}
+	return s3.FormatARN(partition.ID(), fmt.Sprintf("%s/%s", bucket, key)), nil
 }
 
 func (o *runTaskOpts) validateAppName() error {
@@ -1061,6 +1162,7 @@ func BuildTaskRunCmd() *cobra.Command {
 	cmd.Flags().StringVar(&vars.os, osFlag, "", osFlagDescription)
 	cmd.Flags().StringVar(&vars.arch, archFlag, "", archFlagDescription)
 	cmd.Flags().StringToStringVar(&vars.envVars, envVarsFlag, nil, envVarsFlagDescription)
+	cmd.Flags().StringVar(&vars.envFile, envFileFlag, "", envFileFlagDescription)
 	cmd.Flags().StringToStringVar(&vars.secrets, secretsFlag, nil, secretsFlagDescription)
 	cmd.Flags().StringVar(&vars.command, commandFlag, "", runCommandFlagDescription)
 	cmd.Flags().StringVar(&vars.entrypoint, entrypointFlag, "", entrypointFlagDescription)
@@ -1096,6 +1198,7 @@ func BuildTaskRunCmd() *cobra.Command {
 	taskFlags.AddFlag(cmd.Flags().Lookup(osFlag))
 	taskFlags.AddFlag(cmd.Flags().Lookup(archFlag))
 	taskFlags.AddFlag(cmd.Flags().Lookup(envVarsFlag))
+	taskFlags.AddFlag(cmd.Flags().Lookup(envFileFlag))
 	taskFlags.AddFlag(cmd.Flags().Lookup(secretsFlag))
 	taskFlags.AddFlag(cmd.Flags().Lookup(commandFlag))
 	taskFlags.AddFlag(cmd.Flags().Lookup(entrypointFlag))
