@@ -7,6 +7,7 @@ package cloudformation_test
 
 import (
 	"fmt"
+	"io"
 	"math/rand"
 	"os"
 	"strings"
@@ -25,6 +26,9 @@ import (
 	"github.com/aws/copilot-cli/internal/pkg/config"
 	"github.com/aws/copilot-cli/internal/pkg/deploy"
 	"github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation"
+	"github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation/stack"
+	"github.com/aws/copilot-cli/internal/pkg/deploy/upload/customresource"
+	"github.com/aws/copilot-cli/internal/pkg/manifest"
 	"github.com/aws/copilot-cli/internal/pkg/template"
 	"github.com/stretchr/testify/require"
 )
@@ -46,7 +50,7 @@ func Test_App_Infrastructure(t *testing.T) {
 	callerInfo, err := identity.Get()
 	require.NoError(t, err)
 	require.NoError(t, err)
-	deployer := cloudformation.New(sess)
+	deployer := cloudformation.New(sess, cloudformation.WithProgressTracker(os.Stderr))
 	cfClient := awsCF.New(sess)
 	require.NoError(t, err)
 
@@ -208,7 +212,7 @@ func Test_App_Infrastructure(t *testing.T) {
 			&cloudformation.AddEnvToAppOpts{
 				App:          &app,
 				EnvName:      "test",
-				EnvAccountID: "000312697014",
+				EnvAccountID: callerInfo.Account,
 				EnvRegion:    *sess.Config.Region,
 			},
 		)
@@ -349,7 +353,7 @@ func Test_App_Infrastructure(t *testing.T) {
 		err = deployer.AddEnvToApp(&cloudformation.AddEnvToAppOpts{
 			App:          &app,
 			EnvName:      "test",
-			EnvAccountID: "000312697014",
+			EnvAccountID: callerInfo.Account,
 			EnvRegion:    *sess.Config.Region,
 		})
 
@@ -376,17 +380,15 @@ func Test_App_Infrastructure(t *testing.T) {
 func Test_Environment_Deployment_Integration(t *testing.T) {
 	sess, err := testSession(nil)
 	require.NoError(t, err)
-	deployer := cloudformation.New(sess)
+	deployer := cloudformation.New(sess, cloudformation.WithProgressTracker(os.Stderr))
 	cfClient := awsCF.New(sess)
 	identity := identity.New(sess)
 	s3ManagerClient := s3manager.NewUploader(sess)
 	s3Client := awss3.New(sess)
-	uploader := template.New()
-
 	iamClient := iam.New(sess)
-
 	id, err := identity.Get()
 	require.NoError(t, err)
+
 	envName := randStringBytes(10)
 	appName := randStringBytes(10)
 	bucketName := randStringBytes(10)
@@ -400,50 +402,116 @@ func Test_Environment_Deployment_Integration(t *testing.T) {
 	}
 	envStackName := fmt.Sprintf("%s-%s", environmentToDeploy.App.Name, environmentToDeploy.Name)
 
-	t.Run("Deploys an environment to CloudFormation", func(t *testing.T) {
-		// Given our stack doesn't exist
-		output, err := cfClient.DescribeStacks(&awsCF.DescribeStacksInput{
+	// Given our stack doesn't exist
+	output, err := cfClient.DescribeStacks(&awsCF.DescribeStacksInput{
+		StackName: aws.String(envStackName),
+	})
+	require.True(t, len(output.Stacks) == 0, "Stack %s should not exist.", envStackName)
+
+	// Create a temporary S3 bucket to store custom resource scripts.
+	_, err = s3ManagerClient.S3.CreateBucket(&s3.CreateBucketInput{
+		Bucket: aws.String(bucketName),
+	})
+	require.NoError(t, err)
+
+	// Make sure we delete the stack after the test is done.
+	defer func() {
+		_, err := cfClient.DeleteStack(&awsCF.DeleteStackInput{
 			StackName: aws.String(envStackName),
 		})
-		require.True(t, len(output.Stacks) == 0, "Stack %s should not exist.", envStackName)
+		require.NoError(t, err)
 
-		// Create a temporary S3 bucket to store custom resource scripts.
-		_, err = s3ManagerClient.S3.CreateBucket(&s3.CreateBucketInput{
+		err = deleteEnvRoles(appName, envName, id.Account, iamClient)
+		require.NoError(t, err)
+
+		err = s3Client.EmptyBucket(bucketName)
+		require.NoError(t, err)
+		_, err = s3ManagerClient.S3.DeleteBucket(&s3.DeleteBucketInput{
 			Bucket: aws.String(bucketName),
 		})
 		require.NoError(t, err)
+	}()
 
-		// Make sure we delete the stack after the test is done
-		defer func() {
-			_, err := cfClient.DeleteStack(&awsCF.DeleteStackInput{
-				StackName: aws.String(envStackName),
-			})
-			require.NoError(t, err)
-
-			err = deleteEnvRoles(appName, envName, id.Account, iamClient)
-			require.NoError(t, err)
-
-			err = s3Client.EmptyBucket(bucketName)
-			require.NoError(t, err)
-			_, err = s3ManagerClient.S3.DeleteBucket(&s3.DeleteBucketInput{
-				Bucket: aws.String(bucketName),
-			})
-			require.NoError(t, err)
-		}()
-
-		urls, err := uploader.UploadEnvironmentCustomResources(func(key string, objects ...awss3.NamedBinary) (string, error) {
-			return s3Client.ZipAndUpload(bucketName, key, objects...)
-		})
-		require.NoError(t, err)
-		environmentToDeploy.CustomResourcesURLs = urls
+	t.Run("Deploys bootstrap resources for the environment to CloudFormation", func(t *testing.T) {
 		environmentToDeploy.ArtifactBucketKeyARN = "arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"
 		environmentToDeploy.ArtifactBucketARN = fmt.Sprintf("arn:aws:s3:::%s", bucketName)
 
 		// Deploy the environment and wait for it to be complete
-		require.NoError(t, deployer.CreateAndRenderEnvironment(os.Stderr, &environmentToDeploy))
+		require.NoError(t, deployer.CreateAndRenderEnvironment(&environmentToDeploy))
 
 		// Ensure that the new stack exists
-		output, err = cfClient.DescribeStacks(&awsCF.DescribeStacksInput{
+		output, err := cfClient.DescribeStacks(&awsCF.DescribeStacksInput{
+			StackName: aws.String(envStackName),
+		})
+		require.NoError(t, err)
+		require.True(t, len(output.Stacks) == 1, "Stack %s should have been deployed.", envStackName)
+
+		deployedStack := output.Stacks[0]
+		expectedResultsForKey := map[string]func(*awsCF.Output){
+			"EnvironmentManagerRoleARN": func(output *awsCF.Output) {
+				require.Equal(t,
+					fmt.Sprintf("%s-EnvironmentManagerRoleARN", envStackName),
+					*output.ExportName,
+					"Should export EnvironmentManagerRole ARN")
+
+				require.True(t,
+					strings.HasSuffix(*output.OutputValue, fmt.Sprintf("role/%s-EnvManagerRole", envStackName)),
+					"EnvironmentManagerRole ARN value should not be nil.")
+			},
+			"CFNExecutionRoleARN": func(output *awsCF.Output) {
+				require.Equal(t,
+					fmt.Sprintf("%s-CFNExecutionRoleARN", envStackName),
+					*output.ExportName,
+					"Should export CRNExecutionRole ARN")
+
+				require.True(t,
+					strings.HasSuffix(*output.OutputValue, fmt.Sprintf("role/%s-CFNExecutionRole", envStackName)),
+					"CRNExecutionRole ARN value should not be nil.")
+			},
+		}
+		require.True(t, len(deployedStack.Outputs) == len(expectedResultsForKey),
+			"There should have been %d output values - instead there were %d. The value of the CF stack was %s",
+			len(expectedResultsForKey),
+			len(deployedStack.Outputs),
+			deployedStack.GoString(),
+		)
+		for _, output := range deployedStack.Outputs {
+			key := *output.OutputKey
+			validationFunction := expectedResultsForKey[key]
+			require.NotNil(t, validationFunction, "Unexpected output key %s in stack.", key)
+			validationFunction(output)
+		}
+	})
+
+	t.Run("Deploys an environment to CloudFormation", func(t *testing.T) {
+		crs, err := customresource.Env(template.New())
+		require.NoError(t, err)
+		urls, err := customresource.Upload(func(key string, dat io.Reader) (url string, err error) {
+			return s3Client.Upload(bucketName, key, dat)
+		}, crs)
+		require.NoError(t, err)
+		environmentToDeploy.CustomResourcesURLs = urls
+		environmentToDeploy.ArtifactBucketKeyARN = "arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"
+		environmentToDeploy.ArtifactBucketARN = fmt.Sprintf("arn:aws:s3:::%s", bucketName)
+		environmentToDeploy.Mft = &manifest.Environment{
+			Workload: manifest.Workload{
+				Name: aws.String(envName),
+				Type: aws.String("Environment"),
+			},
+		}
+
+		// Deploy the environment and wait for it to be complete.
+		oldParams, err := deployer.EnvironmentParameters(environmentToDeploy.App.Name, environmentToDeploy.Name)
+		require.NoError(t, err)
+		lastForceUpdateID, err := deployer.ForceUpdateOutputID(environmentToDeploy.App.Name, environmentToDeploy.Name)
+		require.NoError(t, err)
+		conf := stack.NewEnvConfigFromExistingStack(&environmentToDeploy, lastForceUpdateID, oldParams)
+
+		// Deploy the environment and wait for it to be complete.
+		require.NoError(t, deployer.UpdateAndRenderEnvironment(conf, environmentToDeploy.ArtifactBucketARN))
+
+		// Ensure that the updated stack still exists.
+		output, err := cfClient.DescribeStacks(&awsCF.DescribeStacksInput{
 			StackName: aws.String(envStackName),
 		})
 		require.NoError(t, err)
@@ -452,7 +520,7 @@ func Test_Environment_Deployment_Integration(t *testing.T) {
 		deployedStack := output.Stacks[0]
 		expectedResultsForKey := map[string]func(*awsCF.Output){
 			"EnabledFeatures": func(output *awsCF.Output) {
-				require.Equal(t, ",,,", aws.StringValue(output.OutputValue), "no env features enabled by default")
+				require.Equal(t, ",,,,", aws.StringValue(output.OutputValue), "no env features enabled by default")
 			},
 			"EnvironmentManagerRoleARN": func(output *awsCF.Output) {
 				require.Equal(t,
@@ -554,6 +622,9 @@ func Test_Environment_Deployment_Integration(t *testing.T) {
 					output.OutputValue,
 					"EnvironmentSecurityGroup value should not be nil")
 			},
+			"LastForceDeployID": func(output *awsCF.Output) {
+				require.Equal(t, lastForceUpdateID, aws.StringValue(output.OutputValue), "last force update id does not change by default")
+			},
 		}
 		require.True(t, len(deployedStack.Outputs) == len(expectedResultsForKey),
 			"There should have been %d output values - instead there were %d. The value of the CF stack was %s",
