@@ -13,16 +13,17 @@ import (
 	"testing"
 	"time"
 
+	"github.com/aws/copilot-cli/internal/pkg/config"
+	"gopkg.in/yaml.v3"
+
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/service/cloudformation"
 	"github.com/aws/copilot-cli/internal/pkg/addon"
-	"github.com/aws/copilot-cli/internal/pkg/config"
 	"github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation/stack/mocks"
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
 	"github.com/aws/copilot-cli/internal/pkg/template"
 	"github.com/golang/mock/gomock"
 	"github.com/stretchr/testify/require"
-	"gopkg.in/yaml.v3"
 )
 
 // Test settings for container healthchecks in the backend service manifest.
@@ -51,54 +52,14 @@ func TestBackendService_Template(t *testing.T) {
 		wantedTemplate   string
 		wantedErr        error
 	}{
-		"unavailable rule priority lambda template": {
-			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
-				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(nil, errors.New("some error"))
-				svc.parser = m
-			},
-			wantedTemplate: "",
-			wantedErr:      fmt.Errorf("read rule priority lambda: some error"),
-		},
-		"unavailable desired count lambda template": {
-			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
-				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(desiredCountGeneratorPath).Return(nil, errors.New("some error"))
-				svc.parser = m
-			},
-			wantedTemplate: "",
-			wantedErr:      fmt.Errorf("read desired count lambda: some error"),
-		},
-		"unavailable env controller lambda template": {
-			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
-				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(desiredCountGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(envControllerPath).Return(nil, errors.New("some error"))
-				svc.parser = m
-			},
-			wantedTemplate: "",
-			wantedErr:      fmt.Errorf("read env controller lambda: some error"),
-		},
 		"unexpected addons parsing error": {
 			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
-				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(desiredCountGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(envControllerPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				svc.parser = m
 				svc.addons = mockAddons{tplErr: errors.New("some error")}
 			},
 			wantedErr: fmt.Errorf("generate addons template for %s: %w", testServiceName, errors.New("some error")),
 		},
 		"unexpected addons parameter parsing error": {
 			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
-				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(desiredCountGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(envControllerPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				svc.parser = m
 				svc.addons = mockAddons{paramsErr: errors.New("some error")}
 			},
 			wantedErr: fmt.Errorf("parse addons parameters for %s: %w", testServiceName, errors.New("some error")),
@@ -115,9 +76,6 @@ func TestBackendService_Template(t *testing.T) {
 			},
 			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
 				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(desiredCountGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(envControllerPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
 				svc.parser = m
 				svc.addons = mockAddons{
 					tpl: `
@@ -144,9 +102,6 @@ Outputs:
 			},
 			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
 				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(desiredCountGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(envControllerPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
 				svc.parser = m
 				svc.addons = mockAddons{
 					tpl: `
@@ -166,9 +121,6 @@ Outputs:
 			},
 			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
 				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(desiredCountGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(envControllerPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
 				m.EXPECT().ParseBackendService(gomock.Any()).Return(nil, errors.New("some error"))
 				svc.parser = m
 				svc.addons = mockAddons{
@@ -214,44 +166,54 @@ Outputs:
 			},
 			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
 				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(desiredCountGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(envControllerPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().ParseBackendService(template.WorkloadOpts{
-					WorkloadType: manifest.BackendServiceType,
-					HealthCheck: &template.ContainerHealthCheck{
-						Command:     []string{"CMD-SHELL", "curl -f http://localhost/ || exit 1"},
-						Interval:    aws.Int64(5),
-						Retries:     aws.Int64(3),
-						StartPeriod: aws.Int64(0),
-						Timeout:     aws.Int64(10),
-					},
-					HostedZoneAliases: make(template.AliasesForHostedZone),
-					HTTPHealthCheck: template.HTTPHealthCheckOpts{
-						HealthCheckPath: manifest.DefaultHealthCheckPath,
-						GracePeriod:     aws.Int64(manifest.DefaultHealthCheckGracePeriod),
-					},
-					DeregistrationDelay: aws.Int64(60), // defaults to 60
-					RulePriorityLambda:  "something",
-					DesiredCountLambda:  "something",
-					EnvControllerLambda: "something",
-					ExecuteCommand:      &template.ExecuteCommandOpts{},
-					NestedStack: &template.WorkloadNestedStackOpts{
-						StackName:       addon.StackName,
-						VariableOutputs: []string{"MyTable"},
-					},
-					Network: template.NetworkOpts{
-						AssignPublicIP: template.DisablePublicIP,
-						SubnetsType:    template.PrivateSubnetsPlacement,
-						SecurityGroups: []string{"sg-1234"},
-					},
-					DeploymentConfiguration: template.DeploymentConfigurationOpts{
-						MinHealthyPercent: 0,
-						MaxPercent:        100,
-					},
-					EntryPoint: []string{"enter", "from"},
-					Command:    []string{"here"},
-				}).Return(&template.Content{Buffer: bytes.NewBufferString("template")}, nil)
+				m.EXPECT().ParseBackendService(gomock.Any()).DoAndReturn(func(actual template.WorkloadOpts) (*template.Content, error) {
+					require.Equal(t, template.WorkloadOpts{
+						AppName:      "phonetool",
+						EnvName:      "test",
+						WorkloadName: "frontend",
+						WorkloadType: manifest.BackendServiceType,
+						HealthCheck: &template.ContainerHealthCheck{
+							Command:     []string{"CMD-SHELL", "curl -f http://localhost/ || exit 1"},
+							Interval:    aws.Int64(5),
+							Retries:     aws.Int64(3),
+							StartPeriod: aws.Int64(0),
+							Timeout:     aws.Int64(10),
+						},
+						HostedZoneAliases: make(template.AliasesForHostedZone),
+						HTTPHealthCheck: template.HTTPHealthCheckOpts{
+							HealthCheckPath: manifest.DefaultHealthCheckPath,
+							GracePeriod:     aws.Int64(manifest.DefaultHealthCheckGracePeriod),
+						},
+						DeregistrationDelay: aws.Int64(60), // defaults to 60
+						CustomResources: map[string]template.S3ObjectLocation{
+							"EnvControllerFunction": {
+								Bucket: "my-bucket",
+								Key:    "sha1/envcontroller.zip",
+							},
+							"DynamicDesiredCountFunction": {
+								Bucket: "my-bucket",
+								Key:    "sha2/count.zip",
+							},
+						},
+						ExecuteCommand: &template.ExecuteCommandOpts{},
+						NestedStack: &template.WorkloadNestedStackOpts{
+							StackName:       addon.StackName,
+							VariableOutputs: []string{"MyTable"},
+						},
+						Network: template.NetworkOpts{
+							AssignPublicIP: template.DisablePublicIP,
+							SubnetsType:    template.PrivateSubnetsPlacement,
+							SecurityGroups: []string{"sg-1234"},
+						},
+						DeploymentConfiguration: template.DeploymentConfigurationOpts{
+							MinHealthyPercent: 0,
+							MaxPercent:        100,
+						},
+						EntryPoint: []string{"enter", "from"},
+						Command:    []string{"here"},
+					}, actual)
+					return &template.Content{Buffer: bytes.NewBufferString("template")}, nil
+				})
 				svc.parser = m
 				svc.addons = mockAddons{
 					tpl: `
@@ -316,52 +278,62 @@ Outputs:
 			},
 			mockDependencies: func(t *testing.T, ctrl *gomock.Controller, svc *BackendService) {
 				m := mocks.NewMockbackendSvcReadParser(ctrl)
-				m.EXPECT().Read(albRulePriorityGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(desiredCountGeneratorPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().Read(envControllerPath).Return(&template.Content{Buffer: bytes.NewBufferString("something")}, nil)
-				m.EXPECT().ParseBackendService(template.WorkloadOpts{
-					WorkloadType: manifest.BackendServiceType,
-					HealthCheck: &template.ContainerHealthCheck{
-						Command:     []string{"CMD-SHELL", "curl -f http://localhost/ || exit 1"},
-						Interval:    aws.Int64(5),
-						Retries:     aws.Int64(3),
-						StartPeriod: aws.Int64(0),
-						Timeout:     aws.Int64(10),
-					},
-					HTTPHealthCheck: template.HTTPHealthCheckOpts{
-						HealthCheckPath:    "/healthz",
-						Port:               "4200",
-						SuccessCodes:       "418",
-						HealthyThreshold:   aws.Int64(64),
-						UnhealthyThreshold: aws.Int64(63),
-						Timeout:            aws.Int64(62),
-						Interval:           aws.Int64(61),
-						GracePeriod:        aws.Int64(60),
-					},
-					HostedZoneAliases:   make(template.AliasesForHostedZone),
-					DeregistrationDelay: aws.Int64(59),
-					AllowedSourceIps:    []string{"10.0.1.0/24"},
-					RulePriorityLambda:  "something",
-					DesiredCountLambda:  "something",
-					EnvControllerLambda: "something",
-					ExecuteCommand:      &template.ExecuteCommandOpts{},
-					NestedStack: &template.WorkloadNestedStackOpts{
-						StackName:       addon.StackName,
-						VariableOutputs: []string{"MyTable"},
-					},
-					Network: template.NetworkOpts{
-						AssignPublicIP: template.DisablePublicIP,
-						SubnetsType:    template.PrivateSubnetsPlacement,
-						SecurityGroups: []string{"sg-1234"},
-					},
-					DeploymentConfiguration: template.DeploymentConfigurationOpts{
-						MinHealthyPercent: 0,
-						MaxPercent:        100,
-					},
-					EntryPoint: []string{"enter", "from"},
-					Command:    []string{"here"},
-					ALBEnabled: true,
-				}).Return(&template.Content{Buffer: bytes.NewBufferString("template")}, nil)
+				m.EXPECT().ParseBackendService(gomock.Any()).DoAndReturn(func(actual template.WorkloadOpts) (*template.Content, error) {
+					require.Equal(t, template.WorkloadOpts{
+						AppName:      "phonetool",
+						EnvName:      "test",
+						WorkloadName: "frontend",
+						WorkloadType: manifest.BackendServiceType,
+						HealthCheck: &template.ContainerHealthCheck{
+							Command:     []string{"CMD-SHELL", "curl -f http://localhost/ || exit 1"},
+							Interval:    aws.Int64(5),
+							Retries:     aws.Int64(3),
+							StartPeriod: aws.Int64(0),
+							Timeout:     aws.Int64(10),
+						},
+						HTTPHealthCheck: template.HTTPHealthCheckOpts{
+							HealthCheckPath:    "/healthz",
+							Port:               "4200",
+							SuccessCodes:       "418",
+							HealthyThreshold:   aws.Int64(64),
+							UnhealthyThreshold: aws.Int64(63),
+							Timeout:            aws.Int64(62),
+							Interval:           aws.Int64(61),
+							GracePeriod:        aws.Int64(60),
+						},
+						HostedZoneAliases:   make(template.AliasesForHostedZone),
+						DeregistrationDelay: aws.Int64(59),
+						AllowedSourceIps:    []string{"10.0.1.0/24"},
+						CustomResources: map[string]template.S3ObjectLocation{
+							"EnvControllerFunction": {
+								Bucket: "my-bucket",
+								Key:    "sha1/envcontroller.zip",
+							},
+							"DynamicDesiredCountFunction": {
+								Bucket: "my-bucket",
+								Key:    "sha2/count.zip",
+							},
+						},
+						ExecuteCommand: &template.ExecuteCommandOpts{},
+						NestedStack: &template.WorkloadNestedStackOpts{
+							StackName:       addon.StackName,
+							VariableOutputs: []string{"MyTable"},
+						},
+						Network: template.NetworkOpts{
+							AssignPublicIP: template.DisablePublicIP,
+							SubnetsType:    template.PrivateSubnetsPlacement,
+							SecurityGroups: []string{"sg-1234"},
+						},
+						DeploymentConfiguration: template.DeploymentConfigurationOpts{
+							MinHealthyPercent: 0,
+							MaxPercent:        100,
+						},
+						EntryPoint: []string{"enter", "from"},
+						Command:    []string{"here"},
+						ALBEnabled: true,
+					}, actual)
+					return &template.Content{Buffer: bytes.NewBufferString("template")}, nil
+				})
 				svc.parser = m
 				svc.addons = mockAddons{
 					tpl: `
@@ -394,6 +366,10 @@ Outputs:
 								RepoURL:  testImageRepoURL,
 								ImageTag: testImageTag,
 							},
+							CustomResourcesURL: map[string]string{
+								"EnvControllerFunction":       "https://my-bucket.s3.Region.amazonaws.com/sha1/envcontroller.zip",
+								"DynamicDesiredCountFunction": "https://my-bucket.s3.Region.amazonaws.com/sha2/count.zip",
+							},
 						},
 					},
 					taskDefOverrideFunc: mockCloudFormationOverrideFunc,
@@ -406,7 +382,9 @@ Outputs:
 				conf.manifest.Network.VPC.Placement = manifest.PlacementArgOrString{
 					PlacementString: &privatePlacement,
 				}
-				conf.manifest.Network.VPC.SecurityGroups = []string{"sg-1234"}
+				conf.manifest.Network.VPC.SecurityGroups = manifest.SecurityGroupsIDsOrConfig{
+					IDs: []string{"sg-1234"},
+				}
 			}
 
 			tc.mockDependencies(t, ctrl, conf)
@@ -511,8 +489,8 @@ func TestBackendService_Parameters(t *testing.T) {
 func TestBackendService_TemplateAndParamsGeneration(t *testing.T) {
 	const (
 		appName = "my-app"
-		envName = "my-env"
 	)
+	envName := "my-env"
 
 	testDir := filepath.Join("testdata", "workloads", "backend")
 
@@ -564,21 +542,23 @@ func TestBackendService_TemplateAndParamsGeneration(t *testing.T) {
 			paramsBytes, err := ioutil.ReadFile(tc.ParamsPath)
 			require.NoError(t, err)
 
-			mft, err := manifest.UnmarshalWorkload([]byte(manifestBytes))
+			dynamicMft, err := manifest.UnmarshalWorkload([]byte(manifestBytes))
 			require.NoError(t, err)
-			require.NoError(t, mft.Validate())
+			require.NoError(t, dynamicMft.Validate())
+			mft := dynamicMft.Manifest()
 
+			envConfig := &manifest.Environment{
+				Workload: manifest.Workload{
+					Name: &envName,
+				},
+			}
+			envConfig.HTTPConfig.Private.Certificates = tc.EnvImportedCertARNs
 			serializer, err := NewBackendService(BackendServiceConfig{
 				App: &config.Application{
 					Name: appName,
 				},
-				Env: &config.Environment{
-					Name: envName,
-					CustomConfig: &config.CustomizeEnv{
-						ImportCertARNs: tc.EnvImportedCertARNs,
-					},
-				},
-				Manifest: mft.(*manifest.BackendService),
+				EnvManifest: envConfig,
+				Manifest:    mft.(*manifest.BackendService),
 				RuntimeConfig: RuntimeConfig{
 					ServiceDiscoveryEndpoint: fmt.Sprintf("%s.%s.local", envName, appName),
 				},
@@ -588,9 +568,6 @@ func TestBackendService_TemplateAndParamsGeneration(t *testing.T) {
 			// mock parser for lambda functions
 			realParser := serializer.parser
 			mockParser := mocks.NewMockbackendSvcReadParser(ctrl)
-			mockParser.EXPECT().Read(albRulePriorityGeneratorPath).Return(templateContent("albRulePriorityGenerator"), nil)
-			mockParser.EXPECT().Read(desiredCountGeneratorPath).Return(templateContent("desiredCountGenerator"), nil)
-			mockParser.EXPECT().Read(envControllerPath).Return(templateContent("envController"), nil)
 			mockParser.EXPECT().ParseBackendService(gomock.Any()).DoAndReturn(func(data template.WorkloadOpts) (*template.Content, error) {
 				// pass call to real parser
 				return realParser.ParseBackendService(data)
@@ -632,9 +609,3 @@ func TestBackendService_TemplateAndParamsGeneration(t *testing.T) {
 		})
 	}
 }
-
-func templateContent(str string) *template.Content {
-	return &template.Content{
-		Buffer: bytes.NewBuffer([]byte(str)),
-	}
-}
