@@ -8,6 +8,8 @@ import (
 	"path/filepath"
 	"testing"
 
+	"github.com/aws/copilot-cli/internal/pkg/config"
+
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
 	"gopkg.in/yaml.v3"
 
@@ -18,6 +20,18 @@ import (
 
 // TestGHv1Pipeline_Template ensures that the CloudFormation template generated for a pipeline matches our pre-defined template.
 func TestGHv1Pipeline_Template(t *testing.T) {
+	var stage deploy.PipelineStage
+	stage.Init(&config.Environment{
+		App:              "phonetool",
+		Name:             "test",
+		Region:           "us-west-2",
+		AccountID:        "1111",
+		ExecutionRoleARN: "arn:aws:iam::1111:role/phonetool-test-CFNExecutionRole",
+		ManagerRoleARN:   "arn:aws:iam::1111:role/phonetool-test-EnvManagerRole",
+	}, &manifest.PipelineStage{
+		Name:         "test",
+		TestCommands: []string{`echo "test"`},
+	}, []string{"api"})
 	ps := stack.NewPipelineStackConfig(&deploy.CreatePipelineInput{
 		AppName: "phonetool",
 		Name:    "phonetool-pipeline",
@@ -30,19 +44,9 @@ func TestGHv1Pipeline_Template(t *testing.T) {
 		Build: &deploy.Build{
 			Image:           "aws/codebuild/amazonlinux2-x86_64-standard:3.0",
 			EnvironmentType: "LINUX_CONTAINER",
+			BuildspecPath:   "copilot/buildspec.yml",
 		},
-		Stages: []deploy.PipelineStage{
-			{
-				AssociatedEnvironment: &deploy.AssociatedEnvironment{
-					Name:      "test",
-					Region:    "us-west-2",
-					AccountID: "1111",
-				},
-				LocalWorkloads:   []string{"api"},
-				RequiresApproval: false,
-				TestCommands:     []string{`echo "test"`},
-			},
-		},
+		Stages: []deploy.PipelineStage{stage},
 		ArtifactBuckets: []deploy.ArtifactBucket{
 			{
 				BucketName: "fancy-bucket",
