@@ -11,12 +11,14 @@ import (
 	"os"
 	"path/filepath"
 	"regexp"
-	"strings"
 	"testing"
 
+	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/aws/session"
+	"github.com/aws/copilot-cli/internal/pkg/addon"
 	"github.com/aws/copilot-cli/internal/pkg/config"
 	"github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation/stack"
-	"github.com/aws/copilot-cli/internal/pkg/template"
+	"github.com/aws/copilot-cli/internal/pkg/workspace"
 	"gopkg.in/yaml.v3"
 
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
@@ -26,9 +28,6 @@ import (
 
 const (
 	svcManifestPath = "svc-manifest.yml"
-
-	dynamicDesiredCountPath = "custom-resources/desired-count-delegation.js"
-	rulePriorityPath        = "custom-resources/alb-rule-priority-generator.js"
 )
 
 func TestLoadBalancedWebService_Template(t *testing.T) {
@@ -72,23 +71,33 @@ func TestLoadBalancedWebService_Template(t *testing.T) {
 		require.NoError(t, err)
 		envMft, err := mft.ApplyEnv(tc.envName)
 		require.NoError(t, err)
-
 		err = envMft.Validate()
 		require.NoError(t, err)
+		err = envMft.Load(session.New())
+		require.NoError(t, err)
+		content := envMft.Manifest()
 
-		v, ok := envMft.(*manifest.LoadBalancedWebService)
+		v, ok := content.(*manifest.LoadBalancedWebService)
 		require.True(t, ok)
 
+		ws, err := workspace.New()
+		require.NoError(t, err)
+
+		_, err = addon.Parse(aws.StringValue(v.Name), ws)
+		var notFound *addon.ErrAddonsNotFound
+		require.ErrorAs(t, err, &notFound)
+
 		svcDiscoveryEndpointName := fmt.Sprintf("%s.%s.local", tc.envName, appName)
-		serializer, err := stack.NewLoadBalancedWebService(stack.LoadBalancedWebServiceConfig{
-			App: &config.Application{Name: appName},
-			Env: &config.Environment{
-				Name: tc.envName,
-				CustomConfig: &config.CustomizeEnv{
-					ImportCertARNs: []string{"mockCertARN"},
-				},
+		envConfig := &manifest.Environment{
+			Workload: manifest.Workload{
+				Name: &tc.envName,
 			},
-			Manifest: v,
+		}
+		envConfig.HTTPConfig.Public.Certificates = []string{"mockCertARN"}
+		serializer, err := stack.NewLoadBalancedWebService(stack.LoadBalancedWebServiceConfig{
+			App:         &config.Application{Name: appName},
+			EnvManifest: envConfig,
+			Manifest:    v,
 			RuntimeConfig: stack.RuntimeConfig{
 				ServiceDiscoveryEndpoint: svcDiscoveryEndpointName,
 				AccountID:                "123456789123",
@@ -99,28 +108,11 @@ func TestLoadBalancedWebService_Template(t *testing.T) {
 		require.NoError(t, err, "template should render")
 		regExpGUID := regexp.MustCompile(`([a-f\d]{8}-)([a-f\d]{4}-){3}([a-f\d]{12})`) // Matches random guids
 		testName := fmt.Sprintf("CF Template should be equal/%s", name)
-		parser := template.New()
-		envController, err := parser.Read(envControllerPath)
-		require.NoError(t, err)
-		envControllerZipFile := envController.String()
-		dynamicDesiredCount, err := parser.Read(dynamicDesiredCountPath)
-		require.NoError(t, err)
-		dynamicDesiredCountZipFile := dynamicDesiredCount.String()
-		rulePriority, err := parser.Read(rulePriorityPath)
-		require.NoError(t, err)
-		rulePriorityZipFile := rulePriority.String()
 
 		t.Run(testName, func(t *testing.T) {
 			actualBytes := []byte(tpl)
 			// Cut random GUID from template.
 			actualBytes = regExpGUID.ReplaceAll(actualBytes, []byte("RandomGUID"))
-			actualString := string(actualBytes)
-			// Cut out zip file for more readable output
-			actualString = strings.ReplaceAll(actualString, envControllerZipFile, "mockEnvControllerZipFile")
-			actualString = strings.ReplaceAll(actualString, dynamicDesiredCountZipFile, "mockDynamicDesiredCountZipFile")
-			actualString = strings.ReplaceAll(actualString, rulePriorityZipFile, "mockRulePriorityZipFile")
-
-			actualBytes = []byte(actualString)
 			mActual := make(map[interface{}]interface{})
 			require.NoError(t, yaml.Unmarshal(actualBytes, mActual))
 
