@@ -10,7 +10,6 @@ import (
 
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/aws/aws-sdk-go/service/cloudformation"
-	"github.com/aws/copilot-cli/internal/pkg/addon"
 	"github.com/aws/copilot-cli/internal/pkg/config"
 	"github.com/aws/copilot-cli/internal/pkg/deploy"
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
@@ -18,15 +17,6 @@ import (
 	"github.com/aws/copilot-cli/internal/pkg/template/override"
 )
 
-// Template rendering configuration.
-const (
-	albRulePriorityGeneratorPath = "custom-resources/alb-rule-priority-generator.js"
-	desiredCountGeneratorPath    = "custom-resources/desired-count-delegation.js"
-	envControllerPath            = "custom-resources/env-controller.js"
-	nlbCertValidatorPath         = "custom-resources/nlb-cert-validator.js"
-	nlbCustomDomainPath          = "custom-resources/nlb-custom-domain.js"
-)
-
 // Parameter logical IDs for a load balanced web service.
 const (
 	LBWebServiceDNSDelegatedParamKey = "DNSDelegated"
@@ -44,7 +34,6 @@ type LoadBalancedWebService struct {
 	*ecsWkld
 	manifest               *manifest.LoadBalancedWebService
 	httpsEnabled           bool
-	certImported           bool
 	dnsDelegationEnabled   bool
 	publicSubnetCIDRBlocks []string
 	appInfo                deploy.AppInformation
@@ -65,20 +54,18 @@ func WithNLB(cidrBlocks []string) func(s *LoadBalancedWebService) {
 // LoadBalancedWebServiceConfig contains fields to configure LoadBalancedWebService.
 type LoadBalancedWebServiceConfig struct {
 	App           *config.Application
-	Env           *config.Environment
+	EnvManifest   *manifest.Environment
 	Manifest      *manifest.LoadBalancedWebService
+	RawManifest   []byte // Content of the manifest file without any transformations.
 	RuntimeConfig RuntimeConfig
 	RootUserARN   string
+	Addons        addons
 }
 
 // NewLoadBalancedWebService creates a new CFN stack with an ECS service from a manifest file, given the options.
 func NewLoadBalancedWebService(conf LoadBalancedWebServiceConfig,
 	opts ...LoadBalancedWebServiceOption) (*LoadBalancedWebService, error) {
 	parser := template.New()
-	addons, err := addon.New(aws.StringValue(conf.Manifest.Name))
-	if err != nil {
-		return nil, fmt.Errorf("new addons: %w", err)
-	}
 	var dnsDelegationEnabled, httpsEnabled bool
 	var appInfo deploy.AppInformation
 
@@ -91,7 +78,8 @@ func NewLoadBalancedWebService(conf LoadBalancedWebServiceConfig,
 		}
 		httpsEnabled = true
 	}
-	if conf.Env.HasImportedCerts() {
+	certImported := len(conf.EnvManifest.HTTPConfig.Public.Certificates) != 0
+	if certImported {
 		httpsEnabled = true
 		dnsDelegationEnabled = false
 	}
@@ -101,20 +89,20 @@ func NewLoadBalancedWebService(conf LoadBalancedWebServiceConfig,
 	s := &LoadBalancedWebService{
 		ecsWkld: &ecsWkld{
 			wkld: &wkld{
-				name:   aws.StringValue(conf.Manifest.Name),
-				env:    conf.Env.Name,
-				app:    conf.App.Name,
-				rc:     conf.RuntimeConfig,
-				image:  conf.Manifest.ImageConfig.Image,
-				parser: parser,
-				addons: addons,
+				name:        aws.StringValue(conf.Manifest.Name),
+				env:         aws.StringValue(conf.EnvManifest.Name),
+				app:         conf.App.Name,
+				rc:          conf.RuntimeConfig,
+				image:       conf.Manifest.ImageConfig.Image,
+				rawManifest: conf.RawManifest,
+				parser:      parser,
+				addons:      conf.Addons,
 			},
 			logRetention:        conf.Manifest.Logging.Retention,
 			tc:                  conf.Manifest.TaskConfig,
 			taskDefOverrideFunc: override.CloudFormationTemplate,
 		},
 		manifest:             conf.Manifest,
-		certImported:         conf.Env.HasImportedCerts(),
 		httpsEnabled:         httpsEnabled,
 		appInfo:              appInfo,
 		dnsDelegationEnabled: dnsDelegationEnabled,
@@ -129,17 +117,9 @@ func NewLoadBalancedWebService(conf LoadBalancedWebServiceConfig,
 
 // Template returns the CloudFormation template for the service parametrized for the environment.
 func (s *LoadBalancedWebService) Template() (string, error) {
-	rulePriorityLambda, err := s.parser.Read(albRulePriorityGeneratorPath)
+	crs, err := convertCustomResources(s.rc.CustomResourcesURL)
 	if err != nil {
-		return "", fmt.Errorf("read rule priority lambda: %w", err)
-	}
-	desiredCountLambda, err := s.parser.Read(desiredCountGeneratorPath)
-	if err != nil {
-		return "", fmt.Errorf("read desired count lambda: %w", err)
-	}
-	envControllerLambda, err := s.parser.Read(envControllerPath)
-	if err != nil {
-		return "", fmt.Errorf("read env controller lambda: %w", err)
+		return "", err
 	}
 	addonsParams, err := s.addonsParameters()
 	if err != nil {
@@ -193,64 +173,58 @@ func (s *LoadBalancedWebService) Template() (string, error) {
 	if err != nil {
 		return "", err
 	}
-	if len(aliasesFor) != 0 && !s.certImported {
-		return "", fmt.Errorf("cannot specify alias hosted zones when env certificates are managed by Copilot")
-	}
-
 	var deregistrationDelay *int64 = aws.Int64(60)
 	if s.manifest.RoutingRule.DeregistrationDelay != nil {
 		deregistrationDelay = aws.Int64(int64(s.manifest.RoutingRule.DeregistrationDelay.Seconds()))
 	}
-
 	var allowedSourceIPs []string
 	for _, ipNet := range s.manifest.RoutingRule.AllowedSourceIps {
 		allowedSourceIPs = append(allowedSourceIPs, string(ipNet))
 	}
-
 	nlbConfig, err := s.convertNetworkLoadBalancer()
 	if err != nil {
 		return "", err
 	}
 	content, err := s.parser.ParseLoadBalancedWebService(template.WorkloadOpts{
-		Variables:                      s.manifest.TaskConfig.Variables,
-		Secrets:                        convertSecrets(s.manifest.TaskConfig.Secrets),
-		Aliases:                        aliases,
-		HTTPSListener:                  s.httpsEnabled,
-		UseImportedCerts:               s.certImported,
-		NestedStack:                    addonsOutputs,
-		AddonsExtraParams:              addonsParams,
-		Sidecars:                       sidecars,
-		LogConfig:                      convertLogging(s.manifest.Logging),
-		DockerLabels:                   s.manifest.ImageConfig.Image.DockerLabels,
-		Autoscaling:                    autoscaling,
-		CapacityProviders:              capacityProviders,
-		DesiredCountOnSpot:             desiredCountOnSpot,
-		ExecuteCommand:                 convertExecuteCommand(&s.manifest.ExecuteCommand),
-		WorkloadType:                   manifest.LoadBalancedWebServiceType,
-		HealthCheck:                    convertContainerHealthCheck(s.manifest.ImageConfig.HealthCheck),
-		HTTPHealthCheck:                convertHTTPHealthCheck(&s.manifest.RoutingRule.HealthCheck),
-		DeregistrationDelay:            deregistrationDelay,
-		AllowedSourceIps:               allowedSourceIPs,
-		RulePriorityLambda:             rulePriorityLambda.String(),
-		DesiredCountLambda:             desiredCountLambda.String(),
-		EnvControllerLambda:            envControllerLambda.String(),
-		Storage:                        convertStorageOpts(s.manifest.Name, s.manifest.Storage),
-		Network:                        convertNetworkConfig(s.manifest.Network),
-		EntryPoint:                     entrypoint,
-		Command:                        command,
-		DependsOn:                      convertDependsOn(s.manifest.ImageConfig.Image.DependsOn),
-		CredentialsParameter:           aws.StringValue(s.manifest.ImageConfig.Image.Credentials),
-		ServiceDiscoveryEndpoint:       s.rc.ServiceDiscoveryEndpoint,
-		Publish:                        publishers,
-		Platform:                       convertPlatform(s.manifest.Platform),
-		HTTPVersion:                    convertHTTPVersion(s.manifest.RoutingRule.ProtocolVersion),
-		NLB:                            nlbConfig.settings,
-		DeploymentConfiguration:        convertDeploymentConfig(s.manifest.DeployConfig),
-		AppDNSName:                     nlbConfig.appDNSName,
-		AppDNSDelegationRole:           nlbConfig.appDNSDelegationRole,
-		NLBCertValidatorFunctionLambda: nlbConfig.certValidatorLambda,
-		NLBCustomDomainFunctionLambda:  nlbConfig.customDomainLambda,
-		ALBEnabled:                     !s.manifest.RoutingRule.Disabled(),
+		AppName:            s.app,
+		EnvName:            s.env,
+		WorkloadName:       s.name,
+		SerializedManifest: string(s.rawManifest),
+
+		Variables:                s.manifest.TaskConfig.Variables,
+		Secrets:                  convertSecrets(s.manifest.TaskConfig.Secrets),
+		Aliases:                  aliases,
+		HTTPSListener:            s.httpsEnabled,
+		NestedStack:              addonsOutputs,
+		AddonsExtraParams:        addonsParams,
+		Sidecars:                 sidecars,
+		LogConfig:                convertLogging(s.manifest.Logging),
+		DockerLabels:             s.manifest.ImageConfig.Image.DockerLabels,
+		Autoscaling:              autoscaling,
+		CapacityProviders:        capacityProviders,
+		DesiredCountOnSpot:       desiredCountOnSpot,
+		ExecuteCommand:           convertExecuteCommand(&s.manifest.ExecuteCommand),
+		WorkloadType:             manifest.LoadBalancedWebServiceType,
+		HealthCheck:              convertContainerHealthCheck(s.manifest.ImageConfig.HealthCheck),
+		HTTPHealthCheck:          convertHTTPHealthCheck(&s.manifest.RoutingRule.HealthCheck),
+		DeregistrationDelay:      deregistrationDelay,
+		AllowedSourceIps:         allowedSourceIPs,
+		CustomResources:          crs,
+		Storage:                  convertStorageOpts(s.manifest.Name, s.manifest.Storage),
+		Network:                  convertNetworkConfig(s.manifest.Network),
+		EntryPoint:               entrypoint,
+		Command:                  command,
+		DependsOn:                convertDependsOn(s.manifest.ImageConfig.Image.DependsOn),
+		CredentialsParameter:     aws.StringValue(s.manifest.ImageConfig.Image.Credentials),
+		ServiceDiscoveryEndpoint: s.rc.ServiceDiscoveryEndpoint,
+		Publish:                  publishers,
+		Platform:                 convertPlatform(s.manifest.Platform),
+		HTTPVersion:              convertHTTPVersion(s.manifest.RoutingRule.ProtocolVersion),
+		NLB:                      nlbConfig.settings,
+		DeploymentConfiguration:  convertDeploymentConfig(s.manifest.DeployConfig),
+		AppDNSName:               nlbConfig.appDNSName,
+		AppDNSDelegationRole:     nlbConfig.appDNSDelegationRole,
+		ALBEnabled:               !s.manifest.RoutingRule.Disabled(),
 		Observability: template.ObservabilityOpts{
 			Tracing: strings.ToUpper(aws.StringValue(s.manifest.Observability.Tracing)),
 		},
@@ -349,8 +323,7 @@ func (s *LoadBalancedWebService) Parameters() ([]*cloudformation.Parameter, erro
 	return wkldParams, nil
 }
 
-// SerializedParameters returns the CloudFormation stack's parameters serialized
-// to a YAML document annotated with comments for readability to users.
+// SerializedParameters returns the CloudFormation stack's parameters serialized to a JSON document.
 func (s *LoadBalancedWebService) SerializedParameters() (string, error) {
-	return s.templateConfiguration(s)
+	return serializeTemplateConfig(s.wkld.parser, s)
 }
