@@ -1,5 +1,4 @@
 //go:build integration || localintegration
-// +build integration localintegration
 
 // Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
@@ -9,15 +8,17 @@ package stack_test
 import (
 	"io/ioutil"
 	"path/filepath"
-	"strings"
 	"testing"
 
 	"gopkg.in/yaml.v3"
 
+	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/aws/session"
+	"github.com/aws/copilot-cli/internal/pkg/addon"
 	"github.com/aws/copilot-cli/internal/pkg/deploy"
 	"github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation/stack"
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
-	"github.com/aws/copilot-cli/internal/pkg/template"
+	"github.com/aws/copilot-cli/internal/pkg/workspace"
 	"github.com/stretchr/testify/require"
 )
 
@@ -40,40 +41,46 @@ func TestRDWS_Template(t *testing.T) {
 	// Read manifest.
 	manifestBytes, err := ioutil.ReadFile(filepath.Join("testdata", "workloads", manifestFileName))
 	require.NoError(t, err, "read manifest file")
-
 	mft, err := manifest.UnmarshalWorkload(manifestBytes)
 	require.NoError(t, err, "unmarshal manifest file")
 	for _, tc := range testCases {
 		envMft, err := mft.ApplyEnv(tc.envName)
 		require.NoError(t, err, "apply test env to manifest")
-
 		err = envMft.Validate()
 		require.NoError(t, err)
+		err = envMft.Load(session.New())
+		require.NoError(t, err)
+		content := envMft.Manifest()
 
-		v, ok := envMft.(*manifest.RequestDrivenWebService)
+		v, ok := content.(*manifest.RequestDrivenWebService)
 		require.True(t, ok)
 
+		ws, err := workspace.New()
+		require.NoError(t, err)
+
+		_, err = addon.Parse(aws.StringValue(v.Name), ws)
+		var notFound *addon.ErrAddonsNotFound
+		require.ErrorAs(t, err, &notFound)
+
 		// Read wanted stack template.
 		wantedTemplate, err := ioutil.ReadFile(filepath.Join("testdata", "workloads", tc.svcStackPath))
 		require.NoError(t, err, "read cloudformation stack")
 
 		// Read actual stack template.
-		serializer, err := stack.NewRequestDrivenWebService(v, tc.envName, deploy.AppInformation{
-			Name: appName,
-		}, stack.RuntimeConfig{
-			AccountID: "123456789123",
-			Region:    "us-west-2",
+		serializer, err := stack.NewRequestDrivenWebService(stack.RequestDrivenWebServiceConfig{
+			App: deploy.AppInformation{
+				Name: appName,
+			},
+			Env:      tc.envName,
+			Manifest: v,
+			RuntimeConfig: stack.RuntimeConfig{
+				AccountID: "123456789123",
+				Region:    "us-west-2",
+			},
 		})
 		require.NoError(t, err, "create rdws serializer")
 		actualTemplate, err := serializer.Template()
 		require.NoError(t, err, "get cloudformation template for rdws")
-		parser := template.New()
-		envController, err := parser.Read(envControllerPath)
-		require.NoError(t, err)
-		envControllerZipFile := envController.String()
-		// Cut out zip file for more readable output
-		actualTemplate = strings.ReplaceAll(actualTemplate, envControllerZipFile, "mockEnvControllerZipFile")
-
 		// Compare the two.
 		wanted := make(map[interface{}]interface{})
 		require.NoError(t, yaml.Unmarshal(wantedTemplate, wanted), "unmarshal wanted template to map[interface{}]interface{}")
