@@ -1,5 +1,4 @@
 //go:build integration || localintegration
-// +build integration localintegration
 
 // Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
@@ -9,16 +8,17 @@ package stack_test
 import (
 	"io/ioutil"
 	"path/filepath"
-	"strings"
 	"testing"
 
 	"gopkg.in/yaml.v3"
 
+	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/aws/session"
+	"github.com/aws/copilot-cli/internal/pkg/addon"
 	"github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation/stack"
+	"github.com/aws/copilot-cli/internal/pkg/workspace"
 
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
-	"github.com/aws/copilot-cli/internal/pkg/template"
-
 	"github.com/stretchr/testify/require"
 )
 
@@ -33,35 +33,39 @@ func TestScheduledJob_Template(t *testing.T) {
 	path := filepath.Join("testdata", "workloads", jobManifestPath)
 	manifestBytes, err := ioutil.ReadFile(path)
 	require.NoError(t, err)
-
 	mft, err := manifest.UnmarshalWorkload(manifestBytes)
 	require.NoError(t, err)
-
 	envMft, err := mft.ApplyEnv(envName)
 	require.NoError(t, err)
-
 	err = envMft.Validate()
 	require.NoError(t, err)
+	err = envMft.Load(session.New())
+	require.NoError(t, err)
+	content := envMft.Manifest()
 
-	v, ok := envMft.(*manifest.ScheduledJob)
+	v, ok := content.(*manifest.ScheduledJob)
 	require.True(t, ok)
 
-	serializer, err := stack.NewScheduledJob(v, envName, appName, stack.RuntimeConfig{
-		ServiceDiscoveryEndpoint: "test.my-app.local",
+	ws, err := workspace.New()
+	require.NoError(t, err)
+
+	_, err = addon.Parse(aws.StringValue(v.Name), ws)
+	var notFound *addon.ErrAddonsNotFound
+	require.ErrorAs(t, err, &notFound)
+
+	serializer, err := stack.NewScheduledJob(stack.ScheduledJobConfig{
+		App:      appName,
+		Env:      envName,
+		Manifest: v,
+		RuntimeConfig: stack.RuntimeConfig{
+			ServiceDiscoveryEndpoint: "test.my-app.local",
+		},
 	})
 
 	tpl, err := serializer.Template()
 	require.NoError(t, err, "template should render")
-	parser := template.New()
-	envController, err := parser.Read(envControllerPath)
-	require.NoError(t, err)
-	zipFile := envController.String()
 	t.Run("CF Template should be equal", func(t *testing.T) {
 		actualBytes := []byte(tpl)
-		actualString := string(actualBytes)
-		// Cut out zip file from EnvControllerAction for more readable output
-		actualString = strings.ReplaceAll(actualString, zipFile, "Abracadabra")
-		actualBytes = []byte(actualString)
 		mActual := make(map[interface{}]interface{})
 		require.NoError(t, yaml.Unmarshal(actualBytes, mActual))
 
