@@ -1,5 +1,4 @@
 //go:build integration || localintegration
-// +build integration localintegration
 
 // Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
@@ -11,9 +10,12 @@ import (
 	"path/filepath"
 	"testing"
 
+	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/copilot-cli/internal/pkg/addon"
 	"github.com/aws/copilot-cli/internal/pkg/config"
 	"github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation/stack"
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
+	"github.com/aws/copilot-cli/internal/pkg/workspace"
 	"github.com/stretchr/testify/require"
 )
 
@@ -21,34 +23,46 @@ const (
 	autoScalingManifestPath = "manifest.yml"
 
 	appName  = "my-app"
-	envName  = "test"
 	imageURL = "mockImageURL"
 	imageTag = "latest"
 )
 
+var envName = "test"
+
 func Test_Stack_Local_Integration(t *testing.T) {
 	const (
 		wantedAutoScalingCFNTemplatePath  = "autoscaling-svc-cf.yml"
 		wantedAutoScalingCFNParameterPath = "cf.params.json"
 		wantedOverrideCFNTemplatePath     = "override-cf.yml"
 	)
+
 	path := filepath.Join("testdata", "stacklocal", autoScalingManifestPath)
 	wantedManifestBytes, err := ioutil.ReadFile(path)
 	require.NoError(t, err)
 	mft, err := manifest.UnmarshalWorkload(wantedManifestBytes)
 	require.NoError(t, err)
-	v, ok := mft.(*manifest.LoadBalancedWebService)
+	content := mft.Manifest()
+
+	v, ok := content.(*manifest.LoadBalancedWebService)
 	require.Equal(t, ok, true)
 
-	serializer, err := stack.NewLoadBalancedWebService(stack.LoadBalancedWebServiceConfig{
-		App: &config.Application{Name: appName},
-		Env: &config.Environment{
-			Name: envName,
-			CustomConfig: &config.CustomizeEnv{
-				ImportCertARNs: []string{"mockCertARN"},
-			},
+	ws, err := workspace.New()
+	require.NoError(t, err)
+
+	_, err = addon.Parse(aws.StringValue(v.Name), ws)
+	var notFound *addon.ErrAddonsNotFound
+	require.ErrorAs(t, err, &notFound)
+
+	envConfig := &manifest.Environment{
+		Workload: manifest.Workload{
+			Name: &envName,
 		},
-		Manifest: v,
+	}
+	envConfig.HTTPConfig.Public.Certificates = []string{"mockCertARN"}
+	serializer, err := stack.NewLoadBalancedWebService(stack.LoadBalancedWebServiceConfig{
+		App:         &config.Application{Name: appName},
+		EnvManifest: envConfig,
+		Manifest:    v,
 		RuntimeConfig: stack.RuntimeConfig{
 			Image: &stack.ECRImage{
 				RepoURL:  imageURL,
