@@ -7,6 +7,7 @@ import (
 	"encoding/json"
 	"fmt"
 	"hash/crc32"
+	"sort"
 	"strconv"
 	"strings"
 	"time"
@@ -64,8 +65,17 @@ func convertSidecar(s map[string]*manifest.SidecarConfig) ([]*template.SidecarOp
 	if s == nil {
 		return nil, nil
 	}
+
+	// Sort the sidecars so that the order is consistent and the integration test won't be flaky.
+	keys := make([]string, 0, len(s))
+	for k := range s {
+		keys = append(keys, k)
+	}
+	sort.Strings(keys)
+
 	var sidecars []*template.SidecarOpts
-	for name, config := range s {
+	for _, name := range keys {
+		config := s[name]
 		port, protocol, err := manifest.ParsePortMapping(config.Port)
 		if err != nil {
 			return nil, err
@@ -204,6 +214,43 @@ func convertCapacityProviders(a manifest.AdvancedCount) []*template.CapacityProv
 	return cps
 }
 
+// convertCooldown converts a service manifest cooldown struct into a format parsable
+// by the templates pkg.
+func convertCooldown(c manifest.Cooldown) template.Cooldown {
+	if c.IsEmpty() {
+		return template.Cooldown{}
+	}
+
+	cooldown := template.Cooldown{}
+
+	if c.ScaleInCooldown != nil {
+		scaleInTime := float64(*c.ScaleInCooldown) / float64(time.Second)
+		cooldown.ScaleInCooldown = aws.Float64(scaleInTime)
+	}
+	if c.ScaleOutCooldown != nil {
+		scaleOutTime := float64(*c.ScaleOutCooldown) / float64(time.Second)
+		cooldown.ScaleOutCooldown = aws.Float64(scaleOutTime)
+	}
+
+	return cooldown
+}
+
+// convertScalingCooldown handles the logic of converting generalized and specific cooldowns set
+// into the scaling cooldown used in the Auto Scaling configuration.
+func convertScalingCooldown(specCooldown, genCooldown manifest.Cooldown) template.Cooldown {
+	cooldown := convertCooldown(genCooldown)
+
+	specTemplateCooldown := convertCooldown(specCooldown)
+	if specCooldown.ScaleInCooldown != nil {
+		cooldown.ScaleInCooldown = specTemplateCooldown.ScaleInCooldown
+	}
+	if specCooldown.ScaleOutCooldown != nil {
+		cooldown.ScaleOutCooldown = specTemplateCooldown.ScaleOutCooldown
+	}
+
+	return cooldown
+}
+
 // convertAutoscaling converts the service's Auto Scaling configuration into a format parsable
 // by the templates pkg.
 func convertAutoscaling(a manifest.AdvancedCount) (*template.AutoscalingOpts, error) {
@@ -235,14 +282,27 @@ func convertAutoscaling(a manifest.AdvancedCount) (*template.AutoscalingOpts, er
 	if a.Memory.ScalingConfig.Value != nil {
 		autoscalingOpts.Memory = aws.Float64(float64(*a.Memory.ScalingConfig.Value))
 	}
-
-	if a.Requests != nil {
-		autoscalingOpts.Requests = aws.Float64(float64(*a.Requests))
+	if a.Requests.Value != nil {
+		autoscalingOpts.Requests = aws.Float64(float64(*a.Requests.Value))
 	}
-	if a.ResponseTime != nil {
-		responseTime := float64(*a.ResponseTime) / float64(time.Second)
+	if a.Requests.ScalingConfig.Value != nil {
+		autoscalingOpts.Requests = aws.Float64(float64(*a.Requests.ScalingConfig.Value))
+	}
+	if a.ResponseTime.Value != nil {
+		responseTime := float64(*a.ResponseTime.Value) / float64(time.Second)
 		autoscalingOpts.ResponseTime = aws.Float64(responseTime)
 	}
+	if a.ResponseTime.ScalingConfig.Value != nil {
+		responseTime := float64(*a.ResponseTime.ScalingConfig.Value) / float64(time.Second)
+		autoscalingOpts.ResponseTime = aws.Float64(responseTime)
+	}
+
+	autoscalingOpts.CPUCooldown = convertScalingCooldown(a.CPU.ScalingConfig.Cooldown, a.Cooldown)
+	autoscalingOpts.MemCooldown = convertScalingCooldown(a.Memory.ScalingConfig.Cooldown, a.Cooldown)
+	autoscalingOpts.ReqCooldown = convertScalingCooldown(a.Requests.ScalingConfig.Cooldown, a.Cooldown)
+	autoscalingOpts.RespTimeCooldown = convertScalingCooldown(a.ResponseTime.ScalingConfig.Cooldown, a.Cooldown)
+	autoscalingOpts.QueueDelayCooldown = convertScalingCooldown(a.QueueScaling.Cooldown, a.Cooldown)
+
 	if !a.QueueScaling.IsEmpty() {
 		acceptableBacklog, err := a.QueueScaling.AcceptableBacklogPerTask()
 		if err != nil {
@@ -292,8 +352,55 @@ type networkLoadBalancerConfig struct {
 	// If a domain is associated these values are not empty.
 	appDNSDelegationRole *string
 	appDNSName           *string
-	certValidatorLambda  string
-	customDomainLambda   string
+}
+
+func convertELBAccessLogsConfig(mft *manifest.Environment) (*template.ELBAccessLogs, error) {
+	elbAccessLogsArgs, isELBAccessLogsSet := mft.ELBAccessLogs()
+	if !isELBAccessLogsSet {
+		return nil, nil
+	}
+
+	if elbAccessLogsArgs == nil {
+		return &template.ELBAccessLogs{}, nil
+	}
+
+	return &template.ELBAccessLogs{
+		BucketName: aws.StringValue(elbAccessLogsArgs.BucketName),
+		Prefix:     aws.StringValue(elbAccessLogsArgs.Prefix),
+	}, nil
+}
+
+func convertEnvSecurityGroupCfg(mft *manifest.Environment) (*template.SecurityGroupConfig, error) {
+	securityGroupConfig, isSecurityConfigSet := mft.EnvSecurityGroup()
+	if !isSecurityConfigSet {
+		return nil, nil
+	}
+	var ingress = make([]template.SecurityGroupRule, len(securityGroupConfig.Ingress))
+	var egress = make([]template.SecurityGroupRule, len(securityGroupConfig.Egress))
+	for idx, ingressValue := range securityGroupConfig.Ingress {
+		ingress[idx].IpProtocol = ingressValue.IpProtocol
+		ingress[idx].CidrIP = ingressValue.CidrIP
+		if fromPort, toPort, err := ingressValue.GetPorts(); err != nil {
+			return nil, err
+		} else {
+			ingress[idx].ToPort = toPort
+			ingress[idx].FromPort = fromPort
+		}
+	}
+	for idx, egressValue := range securityGroupConfig.Egress {
+		egress[idx].IpProtocol = egressValue.IpProtocol
+		egress[idx].CidrIP = egressValue.CidrIP
+		if fromPort, toPort, err := egressValue.GetPorts(); err != nil {
+			return nil, err
+		} else {
+			egress[idx].ToPort = toPort
+			egress[idx].FromPort = fromPort
+		}
+	}
+	return &template.SecurityGroupConfig{
+		Ingress: ingress,
+		Egress:  egress,
+	}, nil
 }
 
 func (s *LoadBalancedWebService) convertNetworkLoadBalancer() (networkLoadBalancerConfig, error) {
@@ -372,17 +479,6 @@ func (s *LoadBalancedWebService) convertNetworkLoadBalancer() (networkLoadBalanc
 		dnsDelegationRole, dnsName := convertAppInformation(s.appInfo)
 		config.appDNSName = dnsName
 		config.appDNSDelegationRole = dnsDelegationRole
-
-		nlbCertValidatorLambda, err := s.parser.Read(nlbCertValidatorPath)
-		if err != nil {
-			return networkLoadBalancerConfig{}, fmt.Errorf("read nlb certificate validator lambda: %w", err)
-		}
-		nlbCustomDomainLambda, err := s.parser.Read(nlbCustomDomainPath)
-		if err != nil {
-			return networkLoadBalancerConfig{}, fmt.Errorf("read nlb custom domain lambda: %w", err)
-		}
-		config.certValidatorLambda = nlbCertValidatorLambda.String()
-		config.customDomainLambda = nlbCustomDomainLambda.String()
 	}
 	return config, nil
 }
@@ -616,21 +712,24 @@ func convertNetworkConfig(network manifest.NetworkConfig) template.NetworkOpts {
 	opts := template.NetworkOpts{
 		AssignPublicIP: template.EnablePublicIP,
 		SubnetsType:    template.PublicSubnetsPlacement,
-		SecurityGroups: network.VPC.SecurityGroups,
 	}
+	opts.SecurityGroups = network.VPC.SecurityGroups.GetIDs()
+	opts.DenyDefaultSecurityGroup = network.VPC.SecurityGroups.IsDefaultSecurityGroupDenied()
+
 	placement := network.VPC.Placement
 	if placement.IsEmpty() {
 		return opts
 	}
-	if placement.PlacementString == nil {
-		opts.AssignPublicIP = template.DisablePublicIP
-		opts.SubnetIDs = placement.PlacementArgs.Subnets
+	if placement.PlacementString != nil {
+		if *placement.PlacementString == manifest.PrivateSubnetPlacement {
+			opts.AssignPublicIP = template.DisablePublicIP
+		}
+		opts.SubnetsType = subnetPlacementForTemplate[*placement.PlacementString]
 		return opts
 	}
-	if *placement.PlacementString == manifest.PrivateSubnetPlacement {
-		opts.AssignPublicIP = template.DisablePublicIP
-	}
-	opts.SubnetsType = subnetPlacementForTemplate[*placement.PlacementString]
+	opts.AssignPublicIP = template.DisablePublicIP
+	opts.SubnetsType = ""
+	opts.SubnetIDs = placement.PlacementArgs.Subnets.IDs
 	return opts
 }
 
@@ -643,11 +742,11 @@ func convertRDWSNetworkConfig(network manifest.RequestDrivenWebServiceNetworkCon
 	if placement.IsEmpty() {
 		return opts
 	}
-	if placement.PlacementString == nil {
-		opts.SubnetIDs = placement.PlacementArgs.Subnets
+	if placement.PlacementString != nil {
+		opts.SubnetsType = subnetPlacementForTemplate[*placement.PlacementString]
 		return opts
 	}
-	opts.SubnetsType = subnetPlacementForTemplate[*placement.PlacementString]
+	opts.SubnetIDs = placement.PlacementArgs.Subnets.IDs
 	return opts
 }
 
@@ -801,22 +900,6 @@ func convertDeadLetter(d manifest.DeadLetterQueue) *template.DeadLetterQueue {
 	}
 }
 
-func parseS3URLs(nameToS3URL map[string]string) (bucket *string, s3ObjectKeys map[string]*string, err error) {
-	if len(nameToS3URL) == 0 {
-		return nil, nil, nil
-	}
-	s3ObjectKeys = make(map[string]*string)
-	for fname, s3url := range nameToS3URL {
-		bucketName, key, err := s3.ParseURL(s3url)
-		if err != nil {
-			return nil, nil, err
-		}
-		s3ObjectKeys[fname] = &key
-		bucket = &bucketName
-	}
-	return
-}
-
 func convertAppInformation(app deploy.AppInformation) (delegationRole *string, domain *string) {
 	role := app.DNSDelegationRole()
 	if role != "" {
@@ -873,3 +956,18 @@ func convertSecrets(secrets map[string]manifest.Secret) map[string]template.Secr
 	}
 	return m
 }
+
+func convertCustomResources(urlForFunc map[string]string) (map[string]template.S3ObjectLocation, error) {
+	out := make(map[string]template.S3ObjectLocation)
+	for fn, url := range urlForFunc {
+		bucket, key, err := s3.ParseURL(url)
+		if err != nil {
+			return nil, fmt.Errorf("convert custom resource %q url: %w", fn, err)
+		}
+		out[fn] = template.S3ObjectLocation{
+			Bucket: bucket,
+			Key:    key,
+		}
+	}
+	return out, nil
+}
