@@ -1,5 +1,4 @@
 //go:build integration || localintegration
-// +build integration localintegration
 
 // Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0
@@ -10,84 +9,78 @@ import (
 	"io/ioutil"
 	"path/filepath"
 	"regexp"
-	"strings"
 	"testing"
 
 	"gopkg.in/yaml.v3"
 
+	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/aws/session"
+	"github.com/aws/copilot-cli/internal/pkg/addon"
 	"github.com/aws/copilot-cli/internal/pkg/deploy/cloudformation/stack"
-	"github.com/aws/copilot-cli/internal/pkg/template"
-
 	"github.com/aws/copilot-cli/internal/pkg/manifest"
+	"github.com/aws/copilot-cli/internal/pkg/workspace"
 
 	"github.com/stretchr/testify/require"
 )
 
 const (
-	workerManifestPath       = "worker-manifest.yml"
-	workerStackPath          = "worker-test.stack.yml"
-	workerParamsPath         = "worker-test.params.json"
-	backlogPerTaskLambdaPath = "custom-resources/backlog-per-task-calculator.js"
+	workerManifestPath = "worker-manifest.yml"
+	workerStackPath    = "worker-test.stack.yml"
+	workerParamsPath   = "worker-test.params.json"
 )
 
 func TestWorkerService_Template(t *testing.T) {
 	path := filepath.Join("testdata", "workloads", workerManifestPath)
 	manifestBytes, err := ioutil.ReadFile(path)
 	require.NoError(t, err)
-
 	mft, err := manifest.UnmarshalWorkload(manifestBytes)
 	require.NoError(t, err)
-
 	envMft, err := mft.ApplyEnv(envName)
 	require.NoError(t, err)
-
 	err = envMft.Validate()
 	require.NoError(t, err)
+	err = envMft.Load(session.New())
+	require.NoError(t, err)
+	content := envMft.Manifest()
 
-	v, ok := envMft.(*manifest.WorkerService)
+	v, ok := content.(*manifest.WorkerService)
 	require.True(t, ok)
 
-	serializer, err := stack.NewWorkerService(v, envName, appName, stack.RuntimeConfig{
-		ServiceDiscoveryEndpoint: "test.my-app.local",
-		AccountID:                "123456789123",
-		Region:                   "us-west-2",
+	ws, err := workspace.New()
+	require.NoError(t, err)
+
+	_, err = addon.Parse(aws.StringValue(v.Name), ws)
+	var notFound *addon.ErrAddonsNotFound
+	require.ErrorAs(t, err, &notFound)
+
+	serializer, err := stack.NewWorkerService(stack.WorkerServiceConfig{
+		App:         appName,
+		Env:         envName,
+		Manifest:    v,
+		RawManifest: manifestBytes,
+		RuntimeConfig: stack.RuntimeConfig{
+			ServiceDiscoveryEndpoint: "test.my-app.local",
+			AccountID:                "123456789123",
+			Region:                   "us-west-2",
+		},
 	})
 
 	tpl, err := serializer.Template()
 	require.NoError(t, err, "template should render")
 	regExpGUID := regexp.MustCompile(`([a-f\d]{8}-)([a-f\d]{4}-){3}([a-f\d]{12})`) // Matches random guids
 
-	parser := template.New()
-	envController, err := parser.Read(envControllerPath)
-	require.NoError(t, err)
-	envControllerZipFile := envController.String()
-
-	dynamicDesiredCount, err := parser.Read(dynamicDesiredCountPath)
-	require.NoError(t, err)
-	dynamicDesiredCountZipFile := dynamicDesiredCount.String()
-
-	backlogPerTaskLambda, err := parser.Read(backlogPerTaskLambdaPath)
-	require.NoError(t, err)
-
 	t.Run("CF Template should be equal", func(t *testing.T) {
 		actualBytes := []byte(tpl)
 		// Cut random GUID from template.
 		actualBytes = regExpGUID.ReplaceAll(actualBytes, []byte("RandomGUID"))
-		actualString := string(actualBytes)
-		// Cut out zip file for more readable output
-		actualString = strings.ReplaceAll(actualString, envControllerZipFile, "mockEnvControllerZipFile")
-		actualString = strings.ReplaceAll(actualString, dynamicDesiredCountZipFile, "mockDynamicDesiredCountZipFile")
-		actualString = strings.ReplaceAll(actualString, backlogPerTaskLambda.String(), "mockBacklogPerTaskLambda")
-		actualBytes = []byte(actualString)
 		mActual := make(map[interface{}]interface{})
 		require.NoError(t, yaml.Unmarshal(actualBytes, mActual))
 
 		expected, err := ioutil.ReadFile(filepath.Join("testdata", "workloads", workerStackPath))
 		require.NoError(t, err, "should be able to read expected bytes")
-		expectedBytes := []byte(expected)
 		mExpected := make(map[interface{}]interface{})
 
-		require.NoError(t, yaml.Unmarshal(expectedBytes, mExpected))
+		require.NoError(t, yaml.Unmarshal(expected, mExpected))
 		require.Equal(t, mExpected, mActual)
 	})
 
