@@ -4,7 +4,6 @@
 package stack
 
 import (
-	"errors"
 	"fmt"
 	"strconv"
 
@@ -71,10 +70,11 @@ const (
 // RuntimeConfig represents configuration that's defined outside of the manifest file
 // that is needed to create a CloudFormation stack.
 type RuntimeConfig struct {
-	Image             *ECRImage         // Optional. Image location in an ECR repository.
-	AddonsTemplateURL string            // Optional. S3 object URL for the addons template.
-	EnvFileARN        string            // Optional. S3 object ARN for the env file.
-	AdditionalTags    map[string]string // AdditionalTags are labels applied to resources in the workload stack.
+	Image              *ECRImage         // Optional. Image location in an ECR repository.
+	AddonsTemplateURL  string            // Optional. S3 object URL for the addons template.
+	EnvFileARN         string            // Optional. S3 object ARN for the env file.
+	AdditionalTags     map[string]string // AdditionalTags are labels applied to resources in the workload stack.
+	CustomResourcesURL map[string]string // Mapping of Custom Resource Function Name to the S3 URL where the function zip file is stored.
 
 	// The target environment metadata.
 	ServiceDiscoveryEndpoint string // Endpoint for the service discovery namespace in the environment.
@@ -116,11 +116,12 @@ type location interface {
 // wkld represents a generic containerized workload.
 // A workload can be a long-running service, an ephemeral task, or a periodic task.
 type wkld struct {
-	name  string
-	env   string
-	app   string
-	rc    RuntimeConfig
-	image location
+	name        string
+	env         string
+	app         string
+	rc          RuntimeConfig
+	image       location
+	rawManifest []byte // Content of the manifest file without any transformations.
 
 	parser template.Parser
 	addons addons
@@ -178,17 +179,17 @@ type templateConfigurer interface {
 	Tags() []*cloudformation.Tag
 }
 
-func (w *wkld) templateConfiguration(tc templateConfigurer) (string, error) {
-	params, err := tc.Parameters()
+func serializeTemplateConfig(parser template.Parser, stack templateConfigurer) (string, error) {
+	params, err := stack.Parameters()
 	if err != nil {
 		return "", err
 	}
-	doc, err := w.parser.Parse(wkldParamsTemplatePath, struct {
+	doc, err := parser.Parse(wkldParamsTemplatePath, struct {
 		Parameters []*cloudformation.Parameter
 		Tags       []*cloudformation.Tag
 	}{
 		Parameters: params,
-		Tags:       tc.Tags(),
+		Tags:       stack.Tags(),
 	}, template.WithFuncs(map[string]interface{}{
 		"inc": template.IncFunc,
 	}))
@@ -199,16 +200,19 @@ func (w *wkld) templateConfiguration(tc templateConfigurer) (string, error) {
 }
 
 func (w *wkld) addonsOutputs() (*template.WorkloadNestedStackOpts, error) {
-	stack, err := w.addons.Template()
-	if err != nil {
-		var notFoundErr *addon.ErrAddonsNotFound
-		if !errors.As(err, &notFoundErr) {
-			return nil, fmt.Errorf("generate addons template for %s: %w", w.name, err)
-		}
-		return nil, nil // No addons found, so there are no outputs and error.
+	if w.addons == nil {
+		return nil, nil
 	}
 
-	out, err := addon.Outputs(stack)
+	tmpl, err := w.addons.Template()
+	switch {
+	case err != nil:
+		return nil, fmt.Errorf("generate addons template for %s: %w", w.name, err)
+	case tmpl == "":
+		return nil, nil
+	}
+
+	out, err := addon.Outputs(tmpl)
 	if err != nil {
 		return nil, fmt.Errorf("get addons outputs for %s: %w", w.name, err)
 	}
@@ -222,13 +226,13 @@ func (w *wkld) addonsOutputs() (*template.WorkloadNestedStackOpts, error) {
 }
 
 func (w *wkld) addonsParameters() (string, error) {
+	if w.addons == nil {
+		return "", nil
+	}
+
 	params, err := w.addons.Parameters()
 	if err != nil {
-		var notFoundErr *addon.ErrAddonsNotFound
-		if !errors.As(err, &notFoundErr) {
-			return "", fmt.Errorf("parse addons parameters for %s: %w", w.name, err)
-		}
-		return "", nil
+		return "", fmt.Errorf("parse addons parameters for %s: %w", w.name, err)
 	}
 	return params, nil
 }
