@@ -19,7 +19,7 @@ import (
 )
 
 type storeMock struct {
-	rgGetter    *mocks.MockresourceGetter
+	rgGetter    *mocks.MockResourceGetter
 	configStore *mocks.MockConfigStoreClient
 }
 
@@ -121,7 +121,7 @@ func TestStore_ListDeployedServices(t *testing.T) {
 			defer ctrl.Finish()
 
 			mockConfigStore := mocks.NewMockConfigStoreClient(ctrl)
-			mockRgGetter := mocks.NewMockresourceGetter(ctrl)
+			mockRgGetter := mocks.NewMockResourceGetter(ctrl)
 
 			mocks := storeMock{
 				rgGetter:    mockRgGetter,
@@ -132,7 +132,7 @@ func TestStore_ListDeployedServices(t *testing.T) {
 
 			store := &Store{
 				configStore:        mockConfigStore,
-				newRgClientFromIDs: func(string, string) (resourceGetter, error) { return mockRgGetter, nil },
+				newRgClientFromIDs: func(string, string) (ResourceGetter, error) { return mockRgGetter, nil },
 			}
 
 			// WHEN
@@ -217,7 +217,7 @@ func TestStore_ListDeployedJobs(t *testing.T) {
 			defer ctrl.Finish()
 
 			mockConfigStore := mocks.NewMockConfigStoreClient(ctrl)
-			mockRgGetter := mocks.NewMockresourceGetter(ctrl)
+			mockRgGetter := mocks.NewMockResourceGetter(ctrl)
 
 			mocks := storeMock{
 				rgGetter:    mockRgGetter,
@@ -228,7 +228,7 @@ func TestStore_ListDeployedJobs(t *testing.T) {
 
 			store := &Store{
 				configStore:        mockConfigStore,
-				newRgClientFromIDs: func(string, string) (resourceGetter, error) { return mockRgGetter, nil },
+				newRgClientFromIDs: func(string, string) (ResourceGetter, error) { return mockRgGetter, nil },
 			}
 			// WHEN
 			jobs, err := store.ListDeployedJobs(tc.inputApp, tc.inputEnv)
@@ -324,7 +324,7 @@ func TestStore_ListEnvironmentsDeployedTo(t *testing.T) {
 			defer ctrl.Finish()
 
 			mockConfigStore := mocks.NewMockConfigStoreClient(ctrl)
-			mockRgGetter := mocks.NewMockresourceGetter(ctrl)
+			mockRgGetter := mocks.NewMockResourceGetter(ctrl)
 
 			mocks := storeMock{
 				rgGetter:    mockRgGetter,
@@ -335,7 +335,7 @@ func TestStore_ListEnvironmentsDeployedTo(t *testing.T) {
 
 			store := &Store{
 				configStore:         mockConfigStore,
-				newRgClientFromRole: func(string, string) (resourceGetter, error) { return mockRgGetter, nil },
+				newRgClientFromRole: func(string, string) (ResourceGetter, error) { return mockRgGetter, nil },
 			}
 
 			// WHEN
@@ -421,7 +421,7 @@ func TestStore_IsServiceDeployed(t *testing.T) {
 			defer ctrl.Finish()
 
 			mockConfigStore := mocks.NewMockConfigStoreClient(ctrl)
-			mockRgGetter := mocks.NewMockresourceGetter(ctrl)
+			mockRgGetter := mocks.NewMockResourceGetter(ctrl)
 
 			mocks := storeMock{
 				rgGetter:    mockRgGetter,
@@ -432,7 +432,7 @@ func TestStore_IsServiceDeployed(t *testing.T) {
 
 			store := &Store{
 				configStore:        mockConfigStore,
-				newRgClientFromIDs: func(string, string) (resourceGetter, error) { return mockRgGetter, nil },
+				newRgClientFromIDs: func(string, string) (ResourceGetter, error) { return mockRgGetter, nil },
 			}
 
 			// WHEN
@@ -516,7 +516,7 @@ func Test_IsJobDeployed(t *testing.T) {
 			defer ctrl.Finish()
 
 			mockConfigStore := mocks.NewMockConfigStoreClient(ctrl)
-			mockRgGetter := mocks.NewMockresourceGetter(ctrl)
+			mockRgGetter := mocks.NewMockResourceGetter(ctrl)
 
 			mocks := storeMock{
 				rgGetter:    mockRgGetter,
@@ -527,7 +527,7 @@ func Test_IsJobDeployed(t *testing.T) {
 
 			store := &Store{
 				configStore:        mockConfigStore,
-				newRgClientFromIDs: func(string, string) (resourceGetter, error) { return mockRgGetter, nil },
+				newRgClientFromIDs: func(string, string) (ResourceGetter, error) { return mockRgGetter, nil },
 			}
 
 			// WHEN
@@ -661,7 +661,7 @@ func TestStore_ListSNSTopics(t *testing.T) {
 			defer ctrl.Finish()
 
 			mockConfigStore := mocks.NewMockConfigStoreClient(ctrl)
-			mockRgGetter := mocks.NewMockresourceGetter(ctrl)
+			mockRgGetter := mocks.NewMockResourceGetter(ctrl)
 
 			mocks := storeMock{
 				rgGetter:    mockRgGetter,
@@ -672,7 +672,7 @@ func TestStore_ListSNSTopics(t *testing.T) {
 
 			store := &Store{
 				configStore:        mockConfigStore,
-				newRgClientFromIDs: func(string, string) (resourceGetter, error) { return mockRgGetter, nil },
+				newRgClientFromIDs: func(string, string) (ResourceGetter, error) { return mockRgGetter, nil },
 			}
 
 			// WHEN
@@ -688,3 +688,104 @@ func TestStore_ListSNSTopics(t *testing.T) {
 		})
 	}
 }
+
+func TestPipelineStore_ListDeployedPipelines(t *testing.T) {
+	const (
+		mockAppName           = "mockApp"
+		mockLegacyPipelineARN = "arn:aws:codepipeline:us-west-2:1234567890:pipeline-dinder-badgoose-repo"
+		mockPipelineARN       = "arn:aws:codepipeline:us-west-2:1234567890:pipeline-my-pipeline-repo"
+	)
+	testCases := map[string]struct {
+		setupMocks func(mocks storeMock)
+
+		wantedError     error
+		wantedPipelines []Pipeline
+	}{
+		"return error if fail to get resources by tag": {
+			setupMocks: func(m storeMock) {
+				m.rgGetter.EXPECT().GetResourcesByTags(pipelineResourceType, map[string]string{
+					AppTagKey: "mockApp",
+				}).Return(nil, errors.New("some error"))
+			},
+
+			wantedError: fmt.Errorf("get pipeline resources by tags for app mockApp: some error"),
+		},
+		"return error if fail to parse pipeline ARN": {
+			setupMocks: func(m storeMock) {
+				m.rgGetter.EXPECT().GetResourcesByTags(pipelineResourceType, map[string]string{
+					AppTagKey: "mockApp",
+				}).Return([]*rg.Resource{
+					{
+						ARN: "badARN",
+					},
+				}, nil)
+			},
+
+			wantedError: fmt.Errorf("parse pipeline ARN: badARN"),
+		},
+		"success": {
+			setupMocks: func(m storeMock) {
+				m.rgGetter.EXPECT().GetResourcesByTags(pipelineResourceType, map[string]string{
+					AppTagKey: "mockApp",
+				}).Return([]*rg.Resource{
+					{
+						ARN: mockLegacyPipelineARN,
+						Tags: map[string]string{
+							AppTagKey: mockAppName,
+						},
+					},
+					{
+						ARN: mockPipelineARN,
+						Tags: map[string]string{
+							AppTagKey:      mockAppName,
+							PipelineTagKey: "my-pipeline-repo",
+						},
+					},
+				}, nil)
+			},
+
+			wantedPipelines: []Pipeline{
+				{
+					ResourceName: "pipeline-dinder-badgoose-repo",
+					IsLegacy:     true,
+					AppName:      mockAppName,
+				},
+				{
+					ResourceName: "my-pipeline-repo",
+					IsLegacy:     false,
+					AppName:      mockAppName,
+				},
+			},
+		},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			ctrl := gomock.NewController(t)
+			defer ctrl.Finish()
+
+			mockRgGetter := mocks.NewMockResourceGetter(ctrl)
+
+			mocks := storeMock{
+				rgGetter: mockRgGetter,
+			}
+
+			tc.setupMocks(mocks)
+
+			store := &PipelineStore{
+				getter: mocks.rgGetter,
+			}
+
+			// WHEN
+			pipelines, err := store.ListDeployedPipelines(mockAppName)
+
+			// THEN
+			if tc.wantedError != nil {
+				require.EqualError(t, err, tc.wantedError.Error())
+				require.ElementsMatch(t, pipelines, tc.wantedPipelines)
+			} else {
+				require.NoError(t, err)
+			}
+		})
+	}
+}
