@@ -136,6 +136,16 @@ func (d *RDWebServiceDescriber) Describe() (HumanJSONStringer, error) {
 	}, nil
 }
 
+// Manifest returns the contents of the manifest used to deploy a request-driven web service stack.
+// If the Manifest metadata doesn't exist in the stack template, then returns ErrManifestNotFoundInTemplate.
+func (d *RDWebServiceDescriber) Manifest(env string) ([]byte, error) {
+	cfn, err := d.initAppRunnerDescriber(env)
+	if err != nil {
+		return nil, err
+	}
+	return cfn.Manifest()
+}
+
 func formatTracingConfiguration(configuration *apprunner.TraceConfiguration) *tracing {
 	if configuration == nil {
 		return nil
