@@ -18,12 +18,15 @@ import (
 
 func TestLBWebServiceDescriber_URI(t *testing.T) {
 	const (
-		testApp          = "phonetool"
-		testEnv          = "test"
-		testSvc          = "jobs"
-		testEnvSubdomain = "test.phonetool.com"
-		testEnvLBDNSName = "abc.us-west-1.elb.amazonaws.com"
-		testSvcPath      = "/"
+		testApp                  = "phonetool"
+		testEnv                  = "test"
+		testSvc                  = "jobs"
+		testEnvSubdomain         = "test.phonetool.com"
+		testEnvLBDNSName         = "abc.us-west-1.elb.amazonaws.com"
+		testSvcPath              = "/"
+		testALBAccessible        = "true"
+		testALBInaccessible      = "false"
+		testCloudFrontDomainName = "test.cloudfront.com"
 
 		testNLBDNSName = "def.us-west-2.elb.amazonaws.com"
 	)
@@ -86,7 +89,7 @@ func TestLBWebServiceDescriber_URI(t *testing.T) {
 					m.ecsDescriber.EXPECT().ServiceStackResources().Return([]*describeStack.Resource{
 						{
 							LogicalID:  svcStackResourceHTTPSListenerRuleLogicalID,
-							Type:       svcStackResourceHTTPSListenerRuleResourceType,
+							Type:       svcStackResourceListenerRuleResourceType,
 							PhysicalID: "mockRuleARN",
 						},
 					}, nil),
@@ -112,7 +115,7 @@ func TestLBWebServiceDescriber_URI(t *testing.T) {
 					m.ecsDescriber.EXPECT().ServiceStackResources().Return([]*describeStack.Resource{
 						{
 							LogicalID:  svcStackResourceHTTPSListenerRuleLogicalID,
-							Type:       svcStackResourceHTTPSListenerRuleResourceType,
+							Type:       svcStackResourceListenerRuleResourceType,
 							PhysicalID: "mockRuleARN",
 						},
 					}, nil),
@@ -141,6 +144,48 @@ func TestLBWebServiceDescriber_URI(t *testing.T) {
 
 			wantedURI: "http://abc.us-west-1.elb.amazonaws.com/mySvc",
 		},
+		"http web service with cloudfront": {
+			setupMocks: func(m lbWebSvcDescriberMocks) {
+				gomock.InOrder(
+					m.ecsDescriber.EXPECT().ServiceStackResources().Return([]*describeStack.Resource{
+						{
+							LogicalID: svcStackResourceALBTargetGroupLogicalID,
+						},
+					}, nil),
+					m.ecsDescriber.EXPECT().Params().Return(map[string]string{
+						stack.WorkloadRulePathParamKey: "mySvc",
+					}, nil),
+					m.envDescriber.EXPECT().Outputs().Return(map[string]string{
+						envOutputPublicLoadBalancerDNSName: testEnvLBDNSName,
+						envOutputPublicALBAccessible:       testALBAccessible,
+						envOutputCloudFrontDomainName:      testCloudFrontDomainName,
+					}, nil),
+				)
+			},
+
+			wantedURI: "http://abc.us-west-1.elb.amazonaws.com/mySvc or http://test.cloudfront.com/mySvc",
+		},
+		"http web service with cloudfront and alb disabled": {
+			setupMocks: func(m lbWebSvcDescriberMocks) {
+				gomock.InOrder(
+					m.ecsDescriber.EXPECT().ServiceStackResources().Return([]*describeStack.Resource{
+						{
+							LogicalID: svcStackResourceALBTargetGroupLogicalID,
+						},
+					}, nil),
+					m.ecsDescriber.EXPECT().Params().Return(map[string]string{
+						stack.WorkloadRulePathParamKey: "mySvc",
+					}, nil),
+					m.envDescriber.EXPECT().Outputs().Return(map[string]string{
+						envOutputPublicLoadBalancerDNSName: testEnvLBDNSName,
+						envOutputPublicALBAccessible:       testALBInaccessible,
+						envOutputCloudFrontDomainName:      testCloudFrontDomainName,
+					}, nil),
+				)
+			},
+
+			wantedURI: "http://test.cloudfront.com/mySvc",
+		},
 		"fail to get parameters of service stack when fetching NLB uris": {
 			setupMocks: func(m lbWebSvcDescriberMocks) {
 				m.ecsDescriber.EXPECT().ServiceStackResources().Return([]*describeStack.Resource{
@@ -255,7 +300,7 @@ func TestLBWebServiceDescriber_URI(t *testing.T) {
 					m.ecsDescriber.EXPECT().ServiceStackResources().Return([]*describeStack.Resource{
 						{
 							LogicalID:  svcStackResourceHTTPSListenerRuleLogicalID,
-							Type:       svcStackResourceHTTPSListenerRuleResourceType,
+							Type:       svcStackResourceListenerRuleResourceType,
 							PhysicalID: "mockRuleARN",
 						},
 					}, nil),
@@ -345,21 +390,32 @@ func TestBackendServiceDescriber_URI(t *testing.T) {
 		},
 		"internal url http": {
 			setupMocks: func(m lbWebSvcDescriberMocks) {
+				resources := []*describeStack.Resource{
+					{
+						Type:       "AWS::ElasticLoadBalancingV2::TargetGroup",
+						LogicalID:  svcStackResourceALBTargetGroupLogicalID,
+						PhysicalID: "targetGroupARN",
+					},
+					{
+						Type:       svcStackResourceListenerRuleResourceType,
+						LogicalID:  svcStackResourceHTTPListenerRuleLogicalID,
+						PhysicalID: "mockRuleARN",
+					},
+				}
 				gomock.InOrder(
-					m.ecsDescriber.EXPECT().ServiceStackResources().Return([]*describeStack.Resource{
-						{
-							LogicalID: svcStackResourceALBTargetGroupLogicalID,
-						},
-					}, nil),
+					m.ecsDescriber.EXPECT().ServiceStackResources().Return(resources, nil),
 					m.ecsDescriber.EXPECT().Params().Return(map[string]string{
 						stack.WorkloadRulePathParamKey: "mySvc",
 					}, nil),
+					m.ecsDescriber.EXPECT().ServiceStackResources().Return(resources, nil),
+					m.lbDescriber.EXPECT().ListenerRuleHostHeaders("mockRuleARN").
+						Return([]string{"jobs.test.phonetool.internal", "1234.us-west-2.internal.aws.com"}, nil),
 					m.envDescriber.EXPECT().Outputs().Return(map[string]string{
-						envOutputInternalLoadBalancerDNSName: testEnvInternalDNSName,
+						envOutputInternalLoadBalancerDNSName: "1234.us-west-2.internal.aws.com",
 					}, nil),
 				)
 			},
-			wantedURI: "http://abc.us-west-1.elb.amazonaws.internal/mySvc",
+			wantedURI: "http://jobs.test.phonetool.internal/mySvc",
 		},
 		"internal url https": {
 			setupMocks: func(m lbWebSvcDescriberMocks) {
@@ -376,7 +432,7 @@ func TestBackendServiceDescriber_URI(t *testing.T) {
 					m.ecsDescriber.EXPECT().ServiceStackResources().Return([]*describeStack.Resource{
 						{
 							LogicalID:  svcStackResourceHTTPSListenerRuleLogicalID,
-							Type:       svcStackResourceHTTPSListenerRuleResourceType,
+							Type:       svcStackResourceListenerRuleResourceType,
 							PhysicalID: "mockRuleARN",
 						},
 					}, nil),
@@ -494,35 +550,48 @@ func TestRDWebServiceDescriber_URI(t *testing.T) {
 
 func TestLBWebServiceURI_String(t *testing.T) {
 	testCases := map[string]struct {
-		albDNSNames []string
-		albPath     string
-		albHTTPS    bool
+		accessDNSNames []string
+		accessPath     string
+		accessHTTPS    bool
 
 		wanted string
 	}{
 		"http": {
-			albDNSNames: []string{"abc.us-west-1.elb.amazonaws.com"},
-			albPath:     "svc",
+			accessDNSNames: []string{"abc.us-west-1.elb.amazonaws.com"},
+			accessPath:     "svc",
 
 			wanted: "http://abc.us-west-1.elb.amazonaws.com/svc",
 		},
 		"http with / path": {
-			albDNSNames: []string{"jobs.test.phonetool.com"},
-			albPath:     "/",
+			accessDNSNames: []string{"jobs.test.phonetool.com"},
+			accessPath:     "/",
 
 			wanted: "http://jobs.test.phonetool.com",
 		},
+		"cloudfront": {
+			accessDNSNames: []string{"abc.cloudfront.net"},
+			accessPath:     "svc",
+
+			wanted: "http://abc.cloudfront.net/svc",
+		},
+		"cloudfront with https": {
+			accessDNSNames: []string{"abc.cloudfront.net"},
+			accessPath:     "svc",
+			accessHTTPS:    true,
+
+			wanted: "https://abc.cloudfront.net/svc",
+		},
 		"https": {
-			albDNSNames: []string{"jobs.test.phonetool.com"},
-			albPath:     "svc",
-			albHTTPS:    true,
+			accessDNSNames: []string{"jobs.test.phonetool.com"},
+			accessPath:     "svc",
+			accessHTTPS:    true,
 
 			wanted: "https://jobs.test.phonetool.com/svc",
 		},
 		"https with / path": {
-			albDNSNames: []string{"jobs.test.phonetool.com"},
-			albPath:     "/",
-			albHTTPS:    true,
+			accessDNSNames: []string{"jobs.test.phonetool.com"},
+			accessPath:     "/",
+			accessHTTPS:    true,
 
 			wanted: "https://jobs.test.phonetool.com",
 		},
@@ -531,10 +600,10 @@ func TestLBWebServiceURI_String(t *testing.T) {
 	for name, tc := range testCases {
 		t.Run(name, func(t *testing.T) {
 			uri := &LBWebServiceURI{
-				albURI: albURI{
-					DNSNames: tc.albDNSNames,
-					Path:     tc.albPath,
-					HTTPS:    tc.albHTTPS,
+				access: accessURI{
+					DNSNames: tc.accessDNSNames,
+					Path:     tc.accessPath,
+					HTTPS:    tc.accessHTTPS,
 				},
 			}
 
