@@ -46,7 +46,10 @@ func TestWorkloadInitializer_Job(t *testing.T) {
 			inSchedule: "@hourly",
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/resizer/copilot", nil)
+				// workspace root: "/resizer/copilot"
+				gomock.InOrder(
+					m.EXPECT().Rel("resizer/Dockerfile").Return("../Dockerfile", nil),
+					m.EXPECT().Rel("/resizer/copilot/manifest.yml").Return("manifest.yml", nil))
 				m.EXPECT().WriteJobManifest(gomock.Any(), "resizer").Return("/resizer/copilot/manifest.yml", nil)
 			},
 			mockstore: func(m *mocks.MockStore) {
@@ -84,6 +87,7 @@ func TestWorkloadInitializer_Job(t *testing.T) {
 			inSchedule: "@hourly",
 
 			mockWriter: func(m *mocks.MockWorkspace) {
+				m.EXPECT().Rel("/resizer/manifest.yml").Return("manifest.yml", nil)
 				m.EXPECT().WriteJobManifest(gomock.Any(), "resizer").Do(func(m *manifest.ScheduledJob, _ string) {
 					require.Equal(t, *m.Workload.Type, manifest.ScheduledJobType)
 					require.Equal(t, *m.ImageConfig.Image.Location, "mockImage")
@@ -124,7 +128,7 @@ func TestWorkloadInitializer_Job(t *testing.T) {
 			inSchedule: "@hourly",
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/resizer", nil)
+				m.EXPECT().Rel("resizer/Dockerfile").Return("Dockerfile", nil)
 				m.EXPECT().WriteJobManifest(gomock.Any(), "resizer").Return("/resizer/manifest.yml", errors.New("some error"))
 			},
 			mockstore: func(m *mocks.MockStore) {},
@@ -138,7 +142,8 @@ func TestWorkloadInitializer_Job(t *testing.T) {
 
 			inSchedule: "@hourly",
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/copilot", nil)
+				// workspace root: "/copilot"
+				m.EXPECT().Rel("resizer/Dockerfile").Return("resizer/Dockerfile", nil)
 				m.EXPECT().WriteJobManifest(gomock.Any(), "resizer").Return("/copilot/resizer/manifest.yml", nil)
 			},
 			mockstore: func(m *mocks.MockStore) {
@@ -155,7 +160,7 @@ func TestWorkloadInitializer_Job(t *testing.T) {
 			inSchedule: "@hourly",
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/resizer", nil)
+				m.EXPECT().Rel("frontend/Dockerfile").Return("frontend/Dockerfile", nil)
 				m.EXPECT().WriteJobManifest(gomock.Any(), "resizer").Return("/resizer/manifest.yml", nil)
 			},
 			mockstore: func(m *mocks.MockStore) {
@@ -182,7 +187,7 @@ func TestWorkloadInitializer_Job(t *testing.T) {
 			inSchedule: "@hourly",
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/resizer", nil)
+				m.EXPECT().Rel("resizer/Dockerfile").Return("Dockerfile", nil)
 				m.EXPECT().WriteJobManifest(gomock.Any(), "resizer").Return("/resizer/manifest.yml", nil)
 			},
 			mockstore: func(m *mocks.MockStore) {
@@ -462,7 +467,10 @@ func TestWorkloadInitializer_Service(t *testing.T) {
 			inSvcPort:        80,
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/frontend", nil)
+				// workspace root: "/frontend"
+				gomock.InOrder(
+					m.EXPECT().Rel("frontend/Dockerfile").Return("Dockerfile", nil),
+					m.EXPECT().Rel("/frontend/manifest.yml").Return("manifest.yml", nil))
 				m.EXPECT().WriteServiceManifest(gomock.Any(), "frontend").Return("/frontend/manifest.yml", nil)
 			},
 			mockstore: func(m *mocks.MockStore) {
@@ -500,7 +508,8 @@ func TestWorkloadInitializer_Service(t *testing.T) {
 			inDockerfilePath: "frontend/Dockerfile",
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/frontend", nil)
+				// workspace root: "/frontend"
+				m.EXPECT().Rel("frontend/Dockerfile").Return("Dockerfile", nil)
 			},
 			mockstore: func(m *mocks.MockStore) {
 				m.EXPECT().GetApplication("app").Return(nil, errors.New("some error"))
@@ -515,7 +524,8 @@ func TestWorkloadInitializer_Service(t *testing.T) {
 			inSvcPort:        80,
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/frontend", nil)
+				// workspace root: "/frontend"
+				m.EXPECT().Rel("frontend/Dockerfile").Return("Dockerfile", nil)
 				m.EXPECT().WriteServiceManifest(gomock.Any(), "frontend").Return("/frontend/manifest.yml", errors.New("some error"))
 			},
 			mockstore: func(m *mocks.MockStore) {
@@ -535,7 +545,8 @@ func TestWorkloadInitializer_Service(t *testing.T) {
 			inDockerfilePath: "frontend/Dockerfile",
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/frontend", nil)
+				// workspace root: "/frontend"
+				m.EXPECT().Rel("frontend/Dockerfile").Return("Dockerfile", nil)
 				m.EXPECT().WriteServiceManifest(gomock.Any(), "frontend").Return("/frontend/manifest.yml", nil)
 			},
 			mockstore: func(m *mocks.MockStore) {
@@ -561,7 +572,8 @@ func TestWorkloadInitializer_Service(t *testing.T) {
 			inDockerfilePath: "frontend/Dockerfile",
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/frontend", nil)
+				// workspace root: "/frontend"
+				m.EXPECT().Rel("frontend/Dockerfile").Return("Dockerfile", nil)
 				m.EXPECT().WriteServiceManifest(gomock.Any(), "frontend").Return("/frontend/manifest.yml", nil)
 			},
 			mockstore: func(m *mocks.MockStore) {
@@ -587,6 +599,8 @@ func TestWorkloadInitializer_Service(t *testing.T) {
 			inSvcPort: 80,
 
 			mockWriter: func(m *mocks.MockWorkspace) {
+				// workspace root: "/backend"
+				m.EXPECT().Rel("/backend/manifest.yml").Return("manifest.yml", nil)
 				m.EXPECT().WriteServiceManifest(gomock.Any(), "backend").
 					Do(func(m *manifest.BackendService, _ string) {
 						require.Equal(t, *m.Workload.Type, manifest.BackendServiceType)
@@ -629,7 +643,10 @@ func TestWorkloadInitializer_Service(t *testing.T) {
 			inSvcPort:        80,
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/backend", nil)
+				// workspace root: "/backend"
+				gomock.InOrder(
+					m.EXPECT().Rel("backend/Dockerfile").Return("Dockerfile", nil),
+					m.EXPECT().Rel("/backend/manifest.yml").Return("manifest.yml", nil))
 				m.EXPECT().WriteServiceManifest(gomock.Any(), "backend").
 					Do(func(m *manifest.BackendService, _ string) {
 						require.Equal(t, *m.Workload.Type, manifest.BackendServiceType)
@@ -678,7 +695,10 @@ func TestWorkloadInitializer_Service(t *testing.T) {
 			},
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/backend", nil)
+				// workspace root: "/backend"
+				gomock.InOrder(
+					m.EXPECT().Rel("backend/Dockerfile").Return("Dockerfile", nil),
+					m.EXPECT().Rel("/backend/manifest.yml").Return("manifest.yml", nil))
 				m.EXPECT().WriteServiceManifest(gomock.Any(), "backend").
 					Do(func(m *manifest.BackendService, _ string) {
 						require.Equal(t, *m.Workload.Type, manifest.BackendServiceType)
@@ -730,7 +750,10 @@ func TestWorkloadInitializer_Service(t *testing.T) {
 			},
 
 			mockWriter: func(m *mocks.MockWorkspace) {
-				m.EXPECT().Path().Return("/worker", nil)
+				// workspace root: "/worker"
+				gomock.InOrder(
+					m.EXPECT().Rel("worker/Dockerfile").Return("Dockerfile", nil),
+					m.EXPECT().Rel("/worker/manifest.yml").Return("manifest.yml", nil))
 				m.EXPECT().WriteServiceManifest(gomock.Any(), "worker").
 					Do(func(m *manifest.WorkerService, _ string) {
 						require.Equal(t, *m.Workload.Type, manifest.WorkerServiceType)
