@@ -209,8 +209,8 @@ func TestFromEnvConfig(t *testing.T) {
 							},
 						},
 					},
-					HTTPConfig: environmentHTTPConfig{
-						Public: publicHTTPConfig{
+					HTTPConfig: EnvironmentHTTPConfig{
+						Public: PublicHTTPConfig{
 							Certificates: []string{"arn:aws:acm:region:account:certificate/certificate_ID_1", "arn:aws:acm:region:account:certificate/certificate_ID_2"},
 						},
 					},
@@ -232,8 +232,8 @@ func TestFromEnvConfig(t *testing.T) {
 					Type: stringP("Environment"),
 				},
 				EnvironmentConfig: EnvironmentConfig{
-					HTTPConfig: environmentHTTPConfig{
-						Public: publicHTTPConfig{
+					HTTPConfig: EnvironmentHTTPConfig{
+						Public: PublicHTTPConfig{
 							Certificates: []string{"arn:aws:acm:region:account:certificate/certificate_ID_1", "arn:aws:acm:region:account:certificate/certificate_ID_2"},
 						},
 					},
@@ -274,10 +274,15 @@ func TestFromEnvConfig(t *testing.T) {
 							},
 						},
 					},
-					HTTPConfig: environmentHTTPConfig{
+					HTTPConfig: EnvironmentHTTPConfig{
 						Private: privateHTTPConfig{
 							InternalALBSubnets: []string{"subnet2"},
 							Certificates:       []string{"arn:aws:acm:region:account:certificate/certificate_ID_1", "arn:aws:acm:region:account:certificate/certificate_ID_2"},
+							SecurityGroupsConfig: securityGroupsConfig{
+								Ingress: Ingress{
+									VPCIngress: aws.Bool(false),
+								},
+							},
 						},
 					},
 				},
@@ -381,6 +386,57 @@ network:
 				},
 			},
 		},
+		"unmarshal with enable access logs": {
+			inContent: `name: prod
+type: Environment
+
+http:
+  public:
+    access_logs: true`,
+			wantedStruct: &Environment{
+				Workload: Workload{
+					Name: aws.String("prod"),
+					Type: aws.String("Environment"),
+				},
+				EnvironmentConfig: EnvironmentConfig{
+					HTTPConfig: EnvironmentHTTPConfig{
+						Public: PublicHTTPConfig{
+							ELBAccessLogs: ELBAccessLogsArgsOrBool{
+								Enabled: aws.Bool(true),
+							},
+						},
+					},
+				},
+			},
+		},
+		"unmarshal with advanced access logs": {
+			inContent: `name: prod
+type: Environment
+
+http:
+  public:
+    access_logs:
+      bucket_name: testbucket
+      prefix: prefix`,
+			wantedStruct: &Environment{
+				Workload: Workload{
+					Name: aws.String("prod"),
+					Type: aws.String("Environment"),
+				},
+				EnvironmentConfig: EnvironmentConfig{
+					HTTPConfig: EnvironmentHTTPConfig{
+						Public: PublicHTTPConfig{
+							ELBAccessLogs: ELBAccessLogsArgsOrBool{
+								AdvancedConfig: ELBAccessLogsArgs{
+									Prefix:     aws.String("prefix"),
+									BucketName: aws.String("testbucket"),
+								},
+							},
+						},
+					},
+				},
+			},
+		},
 		"unmarshal with observability": {
 			inContent: `name: prod
 type: Environment
@@ -400,6 +456,24 @@ observability:
 				},
 			},
 		},
+		"unmarshal with content delivery network bool": {
+			inContent: `name: prod
+type: Environment
+
+cdn: true
+`,
+			wantedStruct: &Environment{
+				Workload: Workload{
+					Name: aws.String("prod"),
+					Type: aws.String("Environment"),
+				},
+				EnvironmentConfig: EnvironmentConfig{
+					CDNConfig: environmentCDNConfig{
+						Enabled: aws.Bool(true),
+					},
+				},
+			},
+		},
 		"unmarshal with http": {
 			inContent: `name: prod
 type: Environment
@@ -409,6 +483,10 @@ http:
         certificates:
             - cert-1
             - cert-2
+    private:
+        security_groups:
+            ingress:
+                from_vpc: false
 `,
 			wantedStruct: &Environment{
 				Workload: Workload{
@@ -416,10 +494,17 @@ http:
 					Type: aws.String("Environment"),
 				},
 				EnvironmentConfig: EnvironmentConfig{
-					HTTPConfig: environmentHTTPConfig{
-						Public: publicHTTPConfig{
+					HTTPConfig: EnvironmentHTTPConfig{
+						Public: PublicHTTPConfig{
 							Certificates: []string{"cert-1", "cert-2"},
 						},
+						Private: privateHTTPConfig{
+							SecurityGroupsConfig: securityGroupsConfig{
+								Ingress: Ingress{
+									VPCIngress: aws.Bool(false),
+								},
+							},
+						},
 					},
 				},
 			},
@@ -584,6 +669,40 @@ func TestEnvironmentVPCConfig_ManagedVPC(t *testing.T) {
 				PrivateSubnetCIDRs: []string{string(mockPrivateSubnet1CIDR), string(mockPrivateSubnet2CIDR), string(mockPrivateSubnet3CIDR)},
 			},
 		},
+		"managed vpc without explicitly configured azs": {
+			inVPCConfig: environmentVPCConfig{
+				CIDR: &mockVPCCIDR,
+				Subnets: subnetsConfiguration{
+					Public: []subnetConfiguration{
+						{
+							CIDR: &mockPublicSubnet1CIDR,
+						},
+						{
+							CIDR: &mockPublicSubnet3CIDR,
+						},
+						{
+							CIDR: &mockPublicSubnet2CIDR,
+						},
+					},
+					Private: []subnetConfiguration{
+						{
+							CIDR: &mockPrivateSubnet2CIDR,
+						},
+						{
+							CIDR: &mockPrivateSubnet1CIDR,
+						},
+						{
+							CIDR: &mockPrivateSubnet3CIDR,
+						},
+					},
+				},
+			},
+			wanted: &template.ManagedVPC{
+				CIDR:               string(mockVPCCIDR),
+				PublicSubnetCIDRs:  []string{string(mockPublicSubnet1CIDR), string(mockPublicSubnet3CIDR), string(mockPublicSubnet2CIDR)},
+				PrivateSubnetCIDRs: []string{string(mockPrivateSubnet2CIDR), string(mockPrivateSubnet1CIDR), string(mockPrivateSubnet3CIDR)},
+			},
+		},
 	}
 	for name, tc := range testCases {
 		t.Run(name, func(t *testing.T) {
@@ -593,6 +712,122 @@ func TestEnvironmentVPCConfig_ManagedVPC(t *testing.T) {
 	}
 }
 
+func TestEnvironmentVPCConfig_IsEmpty(t *testing.T) {
+	testCases := map[string]struct {
+		in     environmentVPCConfig
+		wanted bool
+	}{
+		"empty": {
+			wanted: true,
+		},
+		"not empty": {
+			in: environmentVPCConfig{
+				ID: aws.String("mock-vpc-id"),
+			},
+		},
+	}
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			got := tc.in.IsEmpty()
+			require.Equal(t, tc.wanted, got)
+		})
+	}
+}
+
+func TestSubnetsConfiguration_IsEmpty(t *testing.T) {
+	testCases := map[string]struct {
+		in     subnetsConfiguration
+		wanted bool
+	}{
+		"empty": {
+			wanted: true,
+		},
+		"not empty": {
+			in: subnetsConfiguration{
+				Public: []subnetConfiguration{
+					{
+						SubnetID: aws.String("mock-subnet-id"),
+					},
+				},
+			},
+		},
+	}
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			got := tc.in.IsEmpty()
+			require.Equal(t, tc.wanted, got)
+		})
+	}
+}
+
+func TestEnvironmentHTTPConfig_IsEmpty(t *testing.T) {
+	testCases := map[string]struct {
+		in     EnvironmentHTTPConfig
+		wanted bool
+	}{
+		"empty": {
+			wanted: true,
+		},
+		"not empty": {
+			in: EnvironmentHTTPConfig{
+				Public: PublicHTTPConfig{
+					Certificates: []string{"mock-cert"},
+				},
+			},
+		},
+	}
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			got := tc.in.IsEmpty()
+			require.Equal(t, tc.wanted, got)
+		})
+	}
+}
+
+func TestPublicHTTPConfig_IsEmpty(t *testing.T) {
+	testCases := map[string]struct {
+		in     PublicHTTPConfig
+		wanted bool
+	}{
+		"empty": {
+			wanted: true,
+		},
+		"not empty": {
+			in: PublicHTTPConfig{
+				Certificates: []string{"mock-cert-1"},
+			},
+		},
+	}
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			got := tc.in.IsEmpty()
+			require.Equal(t, tc.wanted, got)
+		})
+	}
+}
+
+func TestPrivateHTTPConfig_IsEmpty(t *testing.T) {
+	testCases := map[string]struct {
+		in     privateHTTPConfig
+		wanted bool
+	}{
+		"empty": {
+			wanted: true,
+		},
+		"not empty": {
+			in: privateHTTPConfig{
+				InternalALBSubnets: []string{"mock-subnet-1"},
+			},
+		},
+	}
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			got := tc.in.IsEmpty()
+			require.Equal(t, tc.wanted, got)
+		})
+	}
+}
+
 func TestEnvironmentObservability_IsEmpty(t *testing.T) {
 	testCases := map[string]struct {
 		in     environmentObservability
@@ -617,3 +852,143 @@ func TestEnvironmentObservability_IsEmpty(t *testing.T) {
 		})
 	}
 }
+
+func TestEnvironmentCDNConfig_IsEmpty(t *testing.T) {
+	testCases := map[string]struct {
+		in     environmentCDNConfig
+		wanted bool
+	}{
+		"empty": {
+			in:     environmentCDNConfig{},
+			wanted: true,
+		},
+		"not empty": {
+			in: environmentCDNConfig{
+				Enabled: aws.Bool(false),
+			},
+			wanted: false,
+		},
+		"advanced not empty": {
+			in: environmentCDNConfig{
+				Config: advancedCDNConfig{
+					Certificate: aws.String("arn:aws:acm:us-east-1:1111111:certificate/look-like-a-good-arn"),
+				},
+			},
+			wanted: false,
+		},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			got := tc.in.IsEmpty()
+			require.Equal(t, tc.wanted, got)
+		})
+	}
+}
+
+func TestEnvironmentConfig_CDNEnabled(t *testing.T) {
+	testCases := map[string]struct {
+		in     EnvironmentConfig
+		wanted bool
+	}{
+		"enabled via bool": {
+			in: EnvironmentConfig{
+				CDNConfig: environmentCDNConfig{
+					Enabled: aws.Bool(true),
+				},
+			},
+			wanted: true,
+		},
+		"enabled via config": {
+			in: EnvironmentConfig{
+				CDNConfig: environmentCDNConfig{
+					Config: advancedCDNConfig{
+						Certificate: aws.String("arn:aws:acm:us-east-1:1111111:certificate/look-like-a-good-arn"),
+					},
+				},
+			},
+			wanted: true,
+		},
+		"not enabled because empty": {
+			in:     EnvironmentConfig{},
+			wanted: false,
+		},
+		"not enabled via bool": {
+			in: EnvironmentConfig{
+				CDNConfig: environmentCDNConfig{
+					Enabled: aws.Bool(false),
+				},
+			},
+			wanted: false,
+		},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			got := tc.in.CDNEnabled()
+			require.Equal(t, tc.wanted, got)
+		})
+	}
+}
+
+func TestEnvironmentConfig_ELBAccessLogs(t *testing.T) {
+	testCases := map[string]struct {
+		in            EnvironmentConfig
+		wantedFlag    bool
+		wantedConfigs *ELBAccessLogsArgs
+	}{
+		"enabled via bool": {
+			in: EnvironmentConfig{
+				HTTPConfig: EnvironmentHTTPConfig{
+					Public: PublicHTTPConfig{
+						ELBAccessLogs: ELBAccessLogsArgsOrBool{
+							Enabled: aws.Bool(true),
+						},
+					},
+				},
+			},
+			wantedFlag:    true,
+			wantedConfigs: nil,
+		},
+		"disabled via bool": {
+			in: EnvironmentConfig{
+				HTTPConfig: EnvironmentHTTPConfig{
+					Public: PublicHTTPConfig{
+						ELBAccessLogs: ELBAccessLogsArgsOrBool{
+							Enabled: aws.Bool(false),
+						},
+					},
+				},
+			},
+			wantedFlag:    false,
+			wantedConfigs: nil,
+		},
+		"advanced access logs config": {
+			in: EnvironmentConfig{
+				HTTPConfig: EnvironmentHTTPConfig{
+					Public: PublicHTTPConfig{
+						ELBAccessLogs: ELBAccessLogsArgsOrBool{
+							AdvancedConfig: ELBAccessLogsArgs{
+								Prefix:     aws.String("prefix"),
+								BucketName: aws.String("bucketname"),
+							},
+						},
+					},
+				},
+			},
+			wantedFlag: true,
+			wantedConfigs: &ELBAccessLogsArgs{
+				BucketName: aws.String("bucketname"),
+				Prefix:     aws.String("prefix"),
+			},
+		},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			elbAccessLogs, flag := tc.in.ELBAccessLogs()
+			require.Equal(t, tc.wantedFlag, flag)
+			require.Equal(t, tc.wantedConfigs, elbAccessLogs)
+		})
+	}
+}
