@@ -4,8 +4,6 @@
 package manifest
 
 import (
-	"io/ioutil"
-	"path/filepath"
 	"testing"
 
 	"github.com/aws/aws-sdk-go/aws"
@@ -13,93 +11,6 @@ import (
 	"github.com/stretchr/testify/require"
 )
 
-func TestScheduledJob_MarshalBinary(t *testing.T) {
-	testCases := map[string]struct {
-		inProps ScheduledJobProps
-
-		wantedTestData string
-	}{
-		"without timeout or retries": {
-			inProps: ScheduledJobProps{
-				WorkloadProps: &WorkloadProps{
-					Name:  "cuteness-aggregator",
-					Image: "copilot/cuteness-aggregator",
-				},
-				Platform: PlatformArgsOrString{
-					PlatformString: nil,
-					PlatformArgs:   PlatformArgs{},
-				},
-				Schedule: "@weekly",
-			},
-			wantedTestData: "scheduled-job-no-timeout-or-retries.yml",
-		},
-		"fully specified using cron schedule": {
-			inProps: ScheduledJobProps{
-				WorkloadProps: &WorkloadProps{
-					Name:       "cuteness-aggregator",
-					Dockerfile: "./cuteness-aggregator/Dockerfile",
-				},
-				Platform: PlatformArgsOrString{
-					PlatformString: nil,
-					PlatformArgs:   PlatformArgs{},
-				},
-				Schedule: "0 */2 * * *",
-				Retries:  3,
-				Timeout:  "1h30m",
-			},
-			wantedTestData: "scheduled-job-fully-specified.yml",
-		},
-		"with timeout and no retries": {
-			inProps: ScheduledJobProps{
-				WorkloadProps: &WorkloadProps{
-					Name:       "cuteness-aggregator",
-					Dockerfile: "./cuteness-aggregator/Dockerfile",
-				},
-				Platform: PlatformArgsOrString{
-					PlatformString: nil,
-					PlatformArgs:   PlatformArgs{},
-				},
-				Schedule: "@every 5h",
-				Retries:  0,
-				Timeout:  "3h",
-			},
-			wantedTestData: "scheduled-job-no-retries.yml",
-		},
-		"with retries and no timeout": {
-			inProps: ScheduledJobProps{
-				WorkloadProps: &WorkloadProps{
-					Name:       "cuteness-aggregator",
-					Dockerfile: "./cuteness-aggregator/Dockerfile",
-				},
-				Platform: PlatformArgsOrString{
-					PlatformString: nil,
-					PlatformArgs:   PlatformArgs{},
-				},
-				Schedule: "@every 5h",
-				Retries:  5,
-			},
-			wantedTestData: "scheduled-job-no-timeout.yml",
-		},
-	}
-
-	for name, tc := range testCases {
-		t.Run(name, func(t *testing.T) {
-			// GIVEN
-			path := filepath.Join("testdata", tc.wantedTestData)
-			wantedBytes, err := ioutil.ReadFile(path)
-			require.NoError(t, err)
-			manifest := NewScheduledJob(&tc.inProps)
-
-			// WHEN
-			tpl, err := manifest.MarshalBinary()
-			require.NoError(t, err)
-
-			// THEN
-			require.Equal(t, string(wantedBytes), string(tpl))
-		})
-	}
-}
-
 func TestScheduledJob_ApplyEnv(t *testing.T) {
 	testCases := map[string]struct {
 		inputManifest *ScheduledJob
@@ -376,7 +287,7 @@ func TestScheduledJob_ApplyEnv(t *testing.T) {
 	for name, tc := range testCases {
 		t.Run(name, func(t *testing.T) {
 			// WHEN
-			actualManifest, actualErr := tc.inputManifest.ApplyEnv(tc.inputEnv)
+			actualManifest, actualErr := tc.inputManifest.applyEnv(tc.inputEnv)
 
 			// THEN
 			if tc.wantedErr != nil {
@@ -455,7 +366,7 @@ func TestScheduledJob_RequiredEnvironmentFeatures(t *testing.T) {
 				},
 			}
 			tc.mft(&inSvc)
-			got := inSvc.RequiredEnvironmentFeatures()
+			got := inSvc.requiredEnvironmentFeatures()
 			require.Equal(t, tc.wanted, got)
 		})
 	}
