@@ -12,10 +12,10 @@ import (
 )
 
 type testVolume struct {
-	EFS *EFSConfigOrBool `yaml:"efs"`
+	EFS EFSConfigOrBool `yaml:"efs"`
 }
 
-func Test_UnmarshalEFS(t *testing.T) {
+func TestEFSConfigOrBool_UnmarshalYAML(t *testing.T) {
 	testCases := map[string]struct {
 		manifest []byte
 		want     testVolume
@@ -26,7 +26,7 @@ func Test_UnmarshalEFS(t *testing.T) {
 efs: 
   id: fs-12345`),
 			want: testVolume{
-				EFS: &EFSConfigOrBool{
+				EFS: EFSConfigOrBool{
 					Advanced: EFSVolumeConfiguration{
 						FileSystemID: aws.String("fs-12345"),
 					},
@@ -39,7 +39,7 @@ efs:
   uid: 1000
   gid: 10000`),
 			want: testVolume{
-				EFS: &EFSConfigOrBool{
+				EFS: EFSConfigOrBool{
 					Advanced: EFSVolumeConfiguration{
 						UID: aws.Uint32(1000),
 						GID: aws.Uint32(10000),
@@ -47,11 +47,11 @@ efs:
 				},
 			},
 		},
-		"with just managed ": {
+		"with just managed": {
 			manifest: []byte(`
 efs: true`),
 			want: testVolume{
-				EFS: &EFSConfigOrBool{
+				EFS: EFSConfigOrBool{
 					Enabled: aws.Bool(true),
 				},
 			},
@@ -65,11 +65,11 @@ efs:
     iam: true
     access_point_id: fsap-1234`),
 			want: testVolume{
-				EFS: &EFSConfigOrBool{
+				EFS: EFSConfigOrBool{
 					Advanced: EFSVolumeConfiguration{
 						FileSystemID:  aws.String("fs-12345"),
 						RootDirectory: aws.String("/"),
-						AuthConfig: &AuthorizationConfig{
+						AuthConfig: AuthorizationConfig{
 							IAM:           aws.Bool(true),
 							AccessPointID: aws.String("fsap-1234"),
 						},
@@ -90,7 +90,7 @@ efs:
 		t.Run(name, func(t *testing.T) {
 			// GIVEN
 			v := testVolume{
-				EFS: &EFSConfigOrBool{},
+				EFS: EFSConfigOrBool{},
 			}
 
 			// WHEN
@@ -112,23 +112,23 @@ efs:
 
 func Test_EmptyVolume(t *testing.T) {
 	testCases := map[string]struct {
-		in   *EFSConfigOrBool
+		in   EFSConfigOrBool
 		want bool
 	}{
 		"with bool set": {
-			in: &EFSConfigOrBool{
+			in: EFSConfigOrBool{
 				Enabled: aws.Bool(true),
 			},
 			want: false,
 		},
 		"with bool set to false": {
-			in: &EFSConfigOrBool{
+			in: EFSConfigOrBool{
 				Enabled: aws.Bool(false),
 			},
 			want: true,
 		},
 		"with uid/gid set": {
-			in: &EFSConfigOrBool{
+			in: EFSConfigOrBool{
 				Advanced: EFSVolumeConfiguration{
 					UID: aws.Uint32(1000),
 					GID: aws.Uint32(10000),
@@ -137,11 +137,11 @@ func Test_EmptyVolume(t *testing.T) {
 			want: false,
 		},
 		"empty": {
-			in:   nil,
+			in:   EFSConfigOrBool{},
 			want: true,
 		},
 		"misconfigured with boolean enabled": {
-			in: &EFSConfigOrBool{
+			in: EFSConfigOrBool{
 				Enabled: aws.Bool(true),
 				Advanced: EFSVolumeConfiguration{
 					FileSystemID: aws.String("fs-1234"),
@@ -150,7 +150,7 @@ func Test_EmptyVolume(t *testing.T) {
 			want: false,
 		},
 		"misconfigured with FSID and UID": {
-			in: &EFSConfigOrBool{
+			in: EFSConfigOrBool{
 				Advanced: EFSVolumeConfiguration{
 					FileSystemID: aws.String("fs-12345"),
 					UID:          aws.Uint32(6777),
@@ -160,7 +160,7 @@ func Test_EmptyVolume(t *testing.T) {
 			want: false,
 		},
 		"misconfigured with bool set to false and extra config (should respect bool)": {
-			in: &EFSConfigOrBool{
+			in: EFSConfigOrBool{
 				Enabled: aws.Bool(false),
 				Advanced: EFSVolumeConfiguration{
 					UID: aws.Uint32(6777),
@@ -247,3 +247,59 @@ func Test_UseManagedFS(t *testing.T) {
 		})
 	}
 }
+
+func TestStorage_IsEmpty(t *testing.T) {
+	testCases := map[string]struct {
+		in     Storage
+		wanted bool
+	}{
+		"empty storage": {
+			in:     Storage{},
+			wanted: true,
+		},
+		"non empty storage": {
+			in: Storage{
+				Volumes: map[string]*Volume{
+					"volume1": nil,
+				},
+			},
+		},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			// WHEN
+			got := tc.in.IsEmpty()
+
+			// THEN
+			require.Equal(t, tc.wanted, got)
+		})
+	}
+}
+
+func TestAuthorizationConfig_IsEmpty(t *testing.T) {
+	testCases := map[string]struct {
+		in     AuthorizationConfig
+		wanted bool
+	}{
+		"empty auth": {
+			in:     AuthorizationConfig{},
+			wanted: true,
+		},
+		"non empty auth": {
+			in: AuthorizationConfig{
+				IAM: aws.Bool(false),
+			},
+		},
+	}
+
+	for name, tc := range testCases {
+		t.Run(name, func(t *testing.T) {
+			// WHEN
+			got := tc.in.IsEmpty()
+
+			// THEN
+			require.Equal(t, tc.wanted, got)
+		})
+	}
+}
