@@ -6,6 +6,7 @@ package manifest
 import (
 	"fmt"
 	"reflect"
+	"time"
 
 	"github.com/imdario/mergo"
 )
@@ -21,17 +22,22 @@ var defaultTransformers = []mergo.Transformers{
 	aliasTransformer{},
 	stringSliceOrStringTransformer{},
 	platformArgsOrStringTransformer{},
+	securityGroupsIDsOrConfigTransformer{},
 	placementArgOrStringTransformer{},
+	subnetListOrArgsTransformer{},
 	healthCheckArgsOrStringTransformer{},
 	countTransformer{},
 	advancedCountTransformer{},
-	scalingConfigOrPercentageTransformer{},
+	scalingConfigOrTTransformer[Percentage]{},
+	scalingConfigOrTTransformer[int]{},
+	scalingConfigOrTTransformer[time.Duration]{},
 	rangeTransformer{},
 	efsConfigOrBoolTransformer{},
 	efsVolumeConfigurationTransformer{},
 	sqsQueueOrBoolTransformer{},
 	routingRuleConfigOrBoolTransformer{},
 	secretTransformer{},
+	environmentCDNConfigTransformer{},
 }
 
 // See a complete list of `reflect.Kind` here: https://pkg.go.dev/reflect#Kind.
@@ -113,7 +119,7 @@ func (t aliasTransformer) Transformer(typ reflect.Type) func(dst, src reflect.Va
 		srcStruct := src.Convert(reflect.TypeOf(Alias{})).Interface().(Alias)
 
 		if len(srcStruct.AdvancedAliases) != 0 {
-			dstStruct.StringSliceOrString = stringSliceOrString{}
+			dstStruct.StringSliceOrString = StringSliceOrString{}
 		}
 
 		if !srcStruct.StringSliceOrString.isEmpty() {
@@ -129,15 +135,15 @@ func (t aliasTransformer) Transformer(typ reflect.Type) func(dst, src reflect.Va
 
 type stringSliceOrStringTransformer struct{}
 
-// Transformer returns custom merge logic for stringSliceOrString's fields.
+// Transformer returns custom merge logic for StringSliceOrString's fields.
 func (t stringSliceOrStringTransformer) Transformer(typ reflect.Type) func(dst, src reflect.Value) error {
-	if !typ.ConvertibleTo(reflect.TypeOf(stringSliceOrString{})) {
+	if !typ.ConvertibleTo(reflect.TypeOf(StringSliceOrString{})) {
 		return nil
 	}
 
 	return func(dst, src reflect.Value) error {
-		dstStruct := dst.Convert(reflect.TypeOf(stringSliceOrString{})).Interface().(stringSliceOrString)
-		srcStruct := src.Convert(reflect.TypeOf(stringSliceOrString{})).Interface().(stringSliceOrString)
+		dstStruct := dst.Convert(reflect.TypeOf(StringSliceOrString{})).Interface().(StringSliceOrString)
+		srcStruct := src.Convert(reflect.TypeOf(StringSliceOrString{})).Interface().(StringSliceOrString)
 
 		if srcStruct.String != nil {
 			dstStruct.StringSlice = nil
@@ -180,6 +186,32 @@ func (t platformArgsOrStringTransformer) Transformer(typ reflect.Type) func(dst,
 	}
 }
 
+type securityGroupsIDsOrConfigTransformer struct{}
+
+// Transformer returns custom merge logic for SecurityGroupsIDsOrConfig's fields.
+func (s securityGroupsIDsOrConfigTransformer) Transformer(typ reflect.Type) func(dst, src reflect.Value) error {
+	if typ != reflect.TypeOf(SecurityGroupsIDsOrConfig{}) {
+		return nil
+	}
+
+	return func(dst, src reflect.Value) error {
+		dstStruct, srcStruct := dst.Interface().(SecurityGroupsIDsOrConfig), src.Interface().(SecurityGroupsIDsOrConfig)
+
+		if !srcStruct.AdvancedConfig.isEmpty() {
+			dstStruct.IDs = nil
+		}
+
+		if srcStruct.IDs != nil {
+			dstStruct.AdvancedConfig = SecurityGroupsConfig{}
+		}
+
+		if dst.CanSet() { // For extra safety to prevent panicking.
+			dst.Set(reflect.ValueOf(dstStruct))
+		}
+		return nil
+	}
+}
+
 type placementArgOrStringTransformer struct{}
 
 // Transformer returns custom merge logic for placementArgOrString's fields.
@@ -206,6 +238,32 @@ func (t placementArgOrStringTransformer) Transformer(typ reflect.Type) func(dst,
 	}
 }
 
+type subnetListOrArgsTransformer struct{}
+
+// Transformer returns custom merge logic for subnetListOrArgsTransformer's fields.
+func (t subnetListOrArgsTransformer) Transformer(typ reflect.Type) func(dst, src reflect.Value) error {
+	if typ != reflect.TypeOf(SubnetListOrArgs{}) {
+		return nil
+	}
+
+	return func(dst, src reflect.Value) error {
+		dstStruct, srcStruct := dst.Interface().(SubnetListOrArgs), src.Interface().(SubnetListOrArgs)
+
+		if len(srcStruct.IDs) != 0 {
+			dstStruct.SubnetArgs = SubnetArgs{}
+		}
+
+		if !srcStruct.SubnetArgs.isEmpty() {
+			dstStruct.IDs = nil
+		}
+
+		if dst.CanSet() { // For extra safety to prevent panicking.
+			dst.Set(reflect.ValueOf(dstStruct))
+		}
+		return nil
+	}
+}
+
 type healthCheckArgsOrStringTransformer struct{}
 
 // Transformer returns custom merge logic for HealthCheckArgsOrString's fields.
@@ -284,23 +342,23 @@ func (t advancedCountTransformer) Transformer(typ reflect.Type) func(dst, src re
 	}
 }
 
-type scalingConfigOrPercentageTransformer struct{}
+type scalingConfigOrTTransformer[T ~int | time.Duration] struct{}
 
 // Transformer returns custom merge logic for ScalingConfigOrPercentage's fields.
-func (t scalingConfigOrPercentageTransformer) Transformer(typ reflect.Type) func(dst, src reflect.Value) error {
-	if typ != reflect.TypeOf(ScalingConfigOrPercentage{}) {
+func (t scalingConfigOrTTransformer[T]) Transformer(typ reflect.Type) func(dst, src reflect.Value) error {
+	if typ != reflect.TypeOf(ScalingConfigOrT[T]{}) {
 		return nil
 	}
 
 	return func(dst, src reflect.Value) error {
-		dstStruct, srcStruct := dst.Interface().(ScalingConfigOrPercentage), src.Interface().(ScalingConfigOrPercentage)
+		dstStruct, srcStruct := dst.Interface().(ScalingConfigOrT[T]), src.Interface().(ScalingConfigOrT[T])
 
 		if !srcStruct.ScalingConfig.IsEmpty() {
 			dstStruct.Value = nil
 		}
 
 		if srcStruct.Value != nil {
-			dstStruct.ScalingConfig = AdvancedScalingConfig{}
+			dstStruct.ScalingConfig = AdvancedScalingConfig[T]{}
 		}
 
 		if dst.CanSet() { // For extra safety to prevent panicking.
@@ -460,6 +518,32 @@ func (t secretTransformer) Transformer(typ reflect.Type) func(dst, src reflect.V
 	}
 }
 
+type environmentCDNConfigTransformer struct{}
+
+// Transformer returns custom merge logic for environmentCDNConfig's fields.
+func (t environmentCDNConfigTransformer) Transformer(typ reflect.Type) func(dst, src reflect.Value) error {
+	if typ != reflect.TypeOf(environmentCDNConfig{}) {
+		return nil
+	}
+
+	return func(dst, src reflect.Value) error {
+		dstStruct, srcStruct := dst.Interface().(environmentCDNConfig), src.Interface().(environmentCDNConfig)
+
+		if !srcStruct.Config.isEmpty() {
+			dstStruct.Enabled = nil
+		}
+
+		if srcStruct.Enabled != nil {
+			dstStruct.Config = advancedCDNConfig{}
+		}
+
+		if dst.CanSet() { // For extra safety to prevent panicking.
+			dst.Set(reflect.ValueOf(dstStruct))
+		}
+		return nil
+	}
+}
+
 type basicTransformer struct{}
 
 // Transformer returns custom merge logic for volume's fields.
