@@ -9,6 +9,8 @@ import (
 	"path/filepath"
 	"strings"
 
+	"github.com/aws/aws-sdk-go/aws/session"
+	"github.com/aws/copilot-cli/internal/pkg/aws/ec2"
 	"github.com/aws/copilot-cli/internal/pkg/docker/dockerengine"
 	"github.com/aws/copilot-cli/internal/pkg/template"
 
@@ -37,11 +39,13 @@ var (
 var (
 	ErrAppRunnerInvalidPlatformWindows = errors.New("Windows is not supported for App Runner services")
 
-	errUnmarshalBuildOpts     = errors.New("unable to unmarshal build field into string or compose-style map")
-	errUnmarshalPlatformOpts  = errors.New("unable to unmarshal platform field into string or compose-style map")
-	errUnmarshalPlacementOpts = errors.New("unable to unmarshal placement field into string or compose-style map")
-	errUnmarshalCountOpts     = errors.New(`unable to unmarshal "count" field to an integer or autoscaling configuration`)
-	errUnmarshalRangeOpts     = errors.New(`unable to unmarshal "range" field`)
+	errUnmarshalBuildOpts         = errors.New("unable to unmarshal build field into string or compose-style map")
+	errUnmarshalPlatformOpts      = errors.New("unable to unmarshal platform field into string or compose-style map")
+	errUnmarshalSecurityGroupOpts = errors.New(`unable to unmarshal "security_groups" field into slice of strings or compose-style map`)
+	errUnmarshalPlacementOpts     = errors.New("unable to unmarshal placement field into string or compose-style map")
+	errUnmarshalSubnetsOpts       = errors.New("unable to unmarshal subnets field into string slice or compose-style map")
+	errUnmarshalCountOpts         = errors.New(`unable to unmarshal "count" field to an integer or autoscaling configuration`)
+	errUnmarshalRangeOpts         = errors.New(`unable to unmarshal "range" field`)
 
 	errUnmarshalExec       = errors.New(`unable to unmarshal "exec" field into boolean or exec configuration`)
 	errUnmarshalEntryPoint = errors.New(`unable to unmarshal "entrypoint" into string or slice of strings`)
@@ -54,26 +58,32 @@ func WorkloadTypes() []string {
 	return append(ServiceTypes(), JobTypes()...)
 }
 
-// WorkloadManifest represents a workload manifest.
-type WorkloadManifest interface {
-	ApplyEnv(envName string) (WorkloadManifest, error)
+// DynamicWorkload represents a dynamically populated workload.
+type DynamicWorkload interface {
+	ApplyEnv(envName string) (DynamicWorkload, error)
 	Validate() error
 	RequiredEnvironmentFeatures() []string
+	Load(sess *session.Session) error
+	Manifest() any
+}
+
+type workloadManifest interface {
+	validate() error
+	applyEnv(envName string) (workloadManifest, error)
+	requiredEnvironmentFeatures() []string
+	subnets() *SubnetListOrArgs
 }
 
 // UnmarshalWorkload deserializes the YAML input stream into a workload manifest object.
 // If an error occurs during deserialization, then returns the error.
 // If the workload type in the manifest is invalid, then returns an ErrInvalidManifestType.
-func UnmarshalWorkload(in []byte) (WorkloadManifest, error) {
-	type manifest interface {
-		WorkloadManifest
-	}
+func UnmarshalWorkload(in []byte) (DynamicWorkload, error) {
 	am := Workload{}
 	if err := yaml.Unmarshal(in, &am); err != nil {
 		return nil, fmt.Errorf("unmarshal to workload manifest: %w", err)
 	}
 	typeVal := aws.StringValue(am.Type)
-	var m manifest
+	var m workloadManifest
 	switch typeVal {
 	case LoadBalancedWebServiceType:
 		m = newDefaultLoadBalancedWebService()
@@ -91,7 +101,7 @@ func UnmarshalWorkload(in []byte) (WorkloadManifest, error) {
 	if err := yaml.Unmarshal(in, m); err != nil {
 		return nil, fmt.Errorf("unmarshal manifest for %s: %w", typeVal, err)
 	}
-	return m, nil
+	return newDynamicWorkloadManifest(m), nil
 }
 
 // WorkloadProps contains properties for creating a new workload manifest.
@@ -220,19 +230,22 @@ type ImageOverride struct {
 	Command    CommandOverride    `yaml:"command"`
 }
 
+// StringSliceOrShellString is either a slice of string or a string using shell-style rules.
+type stringSliceOrShellString StringSliceOrString
+
 // EntryPointOverride is a custom type which supports unmarshalling "entrypoint" yaml which
 // can either be of type string or type slice of string.
-type EntryPointOverride stringSliceOrString
+type EntryPointOverride stringSliceOrShellString
 
 // CommandOverride is a custom type which supports unmarshalling "command" yaml which
 // can either be of type string or type slice of string.
-type CommandOverride stringSliceOrString
+type CommandOverride stringSliceOrShellString
 
 // UnmarshalYAML overrides the default YAML unmarshalling logic for the EntryPointOverride
-// struct, allowing it to perform more complex unmarshalling behavior.
+// struct, allowing it to be unmarshalled into a string slice or a string.
 // This method implements the yaml.Unmarshaler (v3) interface.
 func (e *EntryPointOverride) UnmarshalYAML(value *yaml.Node) error {
-	if err := unmarshalYAMLToStringSliceOrString((*stringSliceOrString)(e), value); err != nil {
+	if err := (*StringSliceOrString)(e).UnmarshalYAML(value); err != nil {
 		return errUnmarshalEntryPoint
 	}
 	return nil
@@ -240,7 +253,7 @@ func (e *EntryPointOverride) UnmarshalYAML(value *yaml.Node) error {
 
 // ToStringSlice converts an EntryPointOverride to a slice of string using shell-style rules.
 func (e *EntryPointOverride) ToStringSlice() ([]string, error) {
-	out, err := toStringSlice((*stringSliceOrString)(e))
+	out, err := (*stringSliceOrShellString)(e).toStringSlice()
 	if err != nil {
 		return nil, err
 	}
@@ -251,7 +264,7 @@ func (e *EntryPointOverride) ToStringSlice() ([]string, error) {
 // struct, allowing it to perform more complex unmarshaling behavior.
 // This method implements the yaml.Unmarshaler (v3) interface.
 func (c *CommandOverride) UnmarshalYAML(value *yaml.Node) error {
-	if err := unmarshalYAMLToStringSliceOrString((*stringSliceOrString)(c), value); err != nil {
+	if err := (*StringSliceOrString)(c).UnmarshalYAML(value); err != nil {
 		return errUnmarshalCommand
 	}
 	return nil
@@ -259,28 +272,26 @@ func (c *CommandOverride) UnmarshalYAML(value *yaml.Node) error {
 
 // ToStringSlice converts an CommandOverride to a slice of string using shell-style rules.
 func (c *CommandOverride) ToStringSlice() ([]string, error) {
-	out, err := toStringSlice((*stringSliceOrString)(c))
+	out, err := (*stringSliceOrShellString)(c).toStringSlice()
 	if err != nil {
 		return nil, err
 	}
 	return out, nil
 }
 
-type stringSliceOrString struct {
+// StringSliceOrString is a custom type that can either be of type string or type slice of string.
+type StringSliceOrString struct {
 	String      *string
 	StringSlice []string
 }
 
-func (s *stringSliceOrString) isEmpty() bool {
-	return s.String == nil && len(s.StringSlice) == 0
-}
-
-func unmarshalYAMLToStringSliceOrString(s *stringSliceOrString, value *yaml.Node) error {
+// UnmarshalYAML overrides the default YAML unmarshaling logic for the StringSliceOrString
+// struct, allowing it to perform more complex unmarshaling behavior.
+// This method implements the yaml.Unmarshaler (v3) interface.
+func (s *StringSliceOrString) UnmarshalYAML(value *yaml.Node) error {
 	if err := value.Decode(&s.StringSlice); err != nil {
-		switch err.(type) {
-		case *yaml.TypeError:
-			break
-		default:
+		var yamlTypeErr *yaml.TypeError
+		if !errors.As(err, &yamlTypeErr) {
 			return err
 		}
 	}
@@ -294,7 +305,22 @@ func unmarshalYAMLToStringSliceOrString(s *stringSliceOrString, value *yaml.Node
 	return value.Decode(&s.String)
 }
 
-func toStringSlice(s *stringSliceOrString) ([]string, error) {
+func (s *StringSliceOrString) isEmpty() bool {
+	return s.String == nil && len(s.StringSlice) == 0
+}
+
+func (s *StringSliceOrString) toStringSlice() []string {
+	if s.StringSlice != nil {
+		return s.StringSlice
+	}
+
+	if s.String == nil {
+		return nil
+	}
+	return []string{*s.String}
+}
+
+func (s *stringSliceOrShellString) toStringSlice() ([]string, error) {
 	if s.StringSlice != nil {
 		return s.StringSlice, nil
 	}
@@ -330,10 +356,8 @@ func (b *BuildArgsOrString) isEmpty() bool {
 // This method implements the yaml.Unmarshaler (v3) interface.
 func (b *BuildArgsOrString) UnmarshalYAML(value *yaml.Node) error {
 	if err := value.Decode(&b.BuildArgs); err != nil {
-		switch err.(type) {
-		case *yaml.TypeError:
-			break
-		default:
+		var yamlTypeErr *yaml.TypeError
+		if !errors.As(err, &yamlTypeErr) {
 			return err
 		}
 	}
@@ -401,6 +425,9 @@ type PlacementArgOrString struct {
 	PlacementArgs
 }
 
+// PlacementString represents what types of subnets (public or private subnets) to place tasks.
+type PlacementString string
+
 // IsEmpty returns empty if the struct has all zero members.
 func (p *PlacementArgOrString) IsEmpty() bool {
 	return p.PlacementString == nil && p.PlacementArgs.isEmpty()
@@ -411,10 +438,8 @@ func (p *PlacementArgOrString) IsEmpty() bool {
 // This method implements the yaml.Unmarshaler (v3) interface.
 func (p *PlacementArgOrString) UnmarshalYAML(value *yaml.Node) error {
 	if err := value.Decode(&p.PlacementArgs); err != nil {
-		switch err.(type) {
-		case *yaml.TypeError:
-			break
-		default:
+		var yamlTypeErr *yaml.TypeError
+		if !errors.As(err, &yamlTypeErr) {
 			return err
 		}
 	}
@@ -429,26 +454,159 @@ func (p *PlacementArgOrString) UnmarshalYAML(value *yaml.Node) error {
 	return nil
 }
 
-// PlacementArgs represents what subnets to place tasks.
+// PlacementArgs represents where to place tasks.
 type PlacementArgs struct {
-	Subnets []string `yaml:"subnets"`
+	Subnets SubnetListOrArgs `yaml:"subnets"`
 }
 
 func (p *PlacementArgs) isEmpty() bool {
-	return p.Subnets == nil || len(p.Subnets) == 0
+	return p.Subnets.isEmpty()
 }
 
-// PlacementString represents what types of subnets (public or private subnets) to place tasks.
-type PlacementString string
+// SubnetListOrArgs represents what subnets to place tasks. It supports unmarshalling
+// yaml which can either be of type SubnetArgs or a list of strings.
+type SubnetListOrArgs struct {
+	IDs []string
+	SubnetArgs
+}
+
+func (s *SubnetListOrArgs) isEmpty() bool {
+	return len(s.IDs) == 0 && s.SubnetArgs.isEmpty()
+}
+
+type dynamicSubnets struct {
+	cfg    *SubnetListOrArgs
+	client subnetIDsGetter
+}
+
+// Load populates the subnet's IDs field if the client is using tags.
+func (dyn *dynamicSubnets) load() error {
+	if dyn.cfg == nil || dyn.cfg.isEmpty() {
+		return nil
+	}
+	if len(dyn.cfg.IDs) > 0 {
+		return nil
+	}
+	var filters []ec2.Filter
+	for k, v := range dyn.cfg.FromTags {
+		values := v.StringSlice
+		if v.String != nil {
+			values = v.toStringSlice()
+		}
+		filters = append(filters, ec2.FilterForTags(k, values...))
+	}
+	ids, err := dyn.client.SubnetIDs(filters...)
+	if err != nil {
+		return fmt.Errorf("get subnet IDs: %w", err)
+	}
+	dyn.cfg.IDs = ids
+	return nil
+}
+
+// Tags represents the aws tags which take string as key and slice of string as values.
+type Tags map[string]StringSliceOrString
+
+// SubnetArgs represents what subnets to place tasks.
+type SubnetArgs struct {
+	FromTags Tags `yaml:"from_tags"`
+}
+
+func (s *SubnetArgs) isEmpty() bool {
+	return len(s.FromTags) == 0
+}
+
+// UnmarshalYAML overrides the default YAML unmarshaling logic for the SubnetListOrArgs
+// struct, allowing it to perform more complex unmarshaling behavior.
+// This method implements the yaml.Unmarshaler (v3) interface.
+func (s *SubnetListOrArgs) UnmarshalYAML(value *yaml.Node) error {
+	if err := value.Decode(&s.SubnetArgs); err != nil {
+		var yamlTypeErr *yaml.TypeError
+		if !errors.As(err, &yamlTypeErr) {
+			return err
+		}
+	}
+	if !s.SubnetArgs.isEmpty() {
+		// Unmarshaled successfully to s.SubnetArgs, unset s.Subnets, and return.
+		s.IDs = nil
+		return nil
+	}
+	if err := value.Decode(&s.IDs); err != nil {
+		return errUnmarshalSubnetsOpts
+	}
+	return nil
+}
+
+// SecurityGroupsIDsOrConfig represents security groups attached to task. It supports unmarshalling
+// yaml which can either be of type SecurityGroupsConfig or a list of strings.
+type SecurityGroupsIDsOrConfig struct {
+	IDs            []string
+	AdvancedConfig SecurityGroupsConfig
+}
+
+func (s *SecurityGroupsIDsOrConfig) isEmpty() bool {
+	return len(s.IDs) == 0 && s.AdvancedConfig.isEmpty()
+}
+
+// SecurityGroupsConfig represents which security groups are attached to a task
+// and if default security group is applied.
+type SecurityGroupsConfig struct {
+	SecurityGroups []string `yaml:"groups"`
+	DenyDefault    *bool    `yaml:"deny_default"`
+}
+
+func (s *SecurityGroupsConfig) isEmpty() bool {
+	return len(s.SecurityGroups) == 0 && s.DenyDefault == nil
+}
+
+// UnmarshalYAML overrides the default YAML unmarshalling logic for the SecurityGroupsIDsOrConfig
+// struct, allowing it to be unmarshalled into a string slice or a string.
+// This method implements the yaml.Unmarshaler (v3) interface.
+func (s *SecurityGroupsIDsOrConfig) UnmarshalYAML(value *yaml.Node) error {
+	if err := value.Decode(&s.AdvancedConfig); err != nil {
+		var yamlTypeErr *yaml.TypeError
+		if !errors.As(err, &yamlTypeErr) {
+			return err
+		}
+	}
+
+	if !s.AdvancedConfig.isEmpty() {
+		// Unmarshalled successfully to s.AdvancedConfig, unset s.IDs, and return.
+		s.IDs = nil
+		return nil
+	}
+
+	if err := value.Decode(&s.IDs); err != nil {
+		return errUnmarshalSecurityGroupOpts
+	}
+	return nil
+}
+
+// GetIDs returns security groups from SecurityGroupsIDsOrConfig that are attached to task.
+// nil is returned if no security groups are specified.
+func (s *SecurityGroupsIDsOrConfig) GetIDs() []string {
+	if !s.AdvancedConfig.isEmpty() {
+		return s.AdvancedConfig.SecurityGroups
+	}
+	return s.IDs
+}
+
+// IsDefaultSecurityGroupDenied returns true if DenyDefault is set to true
+// in SecurityGroupsIDsOrConfig.AdvancedConfig. Otherwise, false is returned.
+func (s *SecurityGroupsIDsOrConfig) IsDefaultSecurityGroupDenied() bool {
+	if !s.AdvancedConfig.isEmpty() {
+		return aws.BoolValue(s.AdvancedConfig.DenyDefault)
+	}
+	return false
+}
 
 // vpcConfig represents the security groups and subnets attached to a task.
 type vpcConfig struct {
-	Placement      PlacementArgOrString `yaml:"placement"`
-	SecurityGroups []string             `yaml:"security_groups"`
+	Placement      PlacementArgOrString      `yaml:"placement"`
+	SecurityGroups SecurityGroupsIDsOrConfig `yaml:"security_groups"`
 }
 
-func (c *vpcConfig) isEmpty() bool {
-	return c.Placement.IsEmpty() && c.SecurityGroups == nil
+func (v *vpcConfig) isEmpty() bool {
+	return v.Placement.IsEmpty() && v.SecurityGroups.isEmpty()
 }
 
 // PlatformArgsOrString is a custom type which supports unmarshaling yaml which
@@ -463,10 +621,8 @@ type PlatformArgsOrString struct {
 // This method implements the yaml.Unmarshaler (v3) interface.
 func (p *PlatformArgsOrString) UnmarshalYAML(value *yaml.Node) error {
 	if err := value.Decode(&p.PlatformArgs); err != nil {
-		switch err.(type) {
-		case *yaml.TypeError:
-			break
-		default:
+		var yamlTypeErr *yaml.TypeError
+		if !errors.As(err, &yamlTypeErr) {
 			return err
 		}
 	}
