@@ -8,15 +8,17 @@ import (
 	"crypto/sha256"
 	"fmt"
 	"path"
+	"path/filepath"
 )
 
 const (
-	s3ArtifactDirName         = "manual"
-	s3TemplateDirName         = "templates"
-	s3ArtifactAddonsDirName   = "addons"
-	s3ArtifactEnvFilesDirName = "env-files"
-	s3ScriptsDirName          = "scripts"
-	s3CustomResourcesDirName  = "custom-resources"
+	s3ArtifactDirName           = "manual"
+	s3TemplateDirName           = "templates"
+	s3ArtifactAddonsDirName     = "addons"
+	s3ArtifactAddonAssetDirName = "assets"
+	s3ArtifactEnvFilesDirName   = "env-files"
+	s3ScriptsDirName            = "scripts"
+	s3CustomResourcesDirName    = "custom-resources"
 )
 
 // MkdirSHA256 prefixes the key with the SHA256 hash of the contents of "manual/<hash>/key".
@@ -30,6 +32,12 @@ func Addons(key string, content []byte) string {
 	return path.Join(s3ArtifactDirName, s3ArtifactAddonsDirName, key, fmt.Sprintf("%x.yml", sha256.Sum256(content)))
 }
 
+// AddonAsset returns the path to store an addon asset file.
+// Example: manual/addons/frontend/assets/668e2b73ac.
+func AddonAsset(workloadName, hash string) string {
+	return path.Join(s3ArtifactDirName, s3ArtifactAddonsDirName, workloadName, s3ArtifactAddonAssetDirName, hash)
+}
+
 // CFNTemplate returns the path to store cloudformation templates with sha256 of the content.
 // Example: manual/templates/key/sha.yml.
 func CFNTemplate(key string, content []byte) string {
@@ -39,7 +47,8 @@ func CFNTemplate(key string, content []byte) string {
 // EnvFiles returns the path to store an env file artifact with sha256 of the content..
 // Example: manual/env-files/key/sha.env.
 func EnvFiles(key string, content []byte) string {
-	return path.Join(s3ArtifactDirName, s3ArtifactEnvFilesDirName, key, fmt.Sprintf("%x.env", sha256.Sum256(content)))
+	// use filepath.Base to prevent cryptic errors in the ecs agent for paths like "..\magic.env"
+	return path.Join(s3ArtifactDirName, s3ArtifactEnvFilesDirName, filepath.Base(key), fmt.Sprintf("%x.env", sha256.Sum256(content)))
 }
 
 // CustomResource returns the path to store a custom resource with a sha256 of the contents of the file.
