@@ -6,6 +6,7 @@ package template
 import (
 	"bytes"
 	"fmt"
+	"strconv"
 	"text/template"
 
 	"github.com/aws/aws-sdk-go/service/secretsmanager"
@@ -324,15 +325,26 @@ type CapacityProviderStrategy struct {
 	CapacityProvider string
 }
 
+// Cooldown holds configuration needed for autoscaling cooldown fields.
+type Cooldown struct {
+	ScaleInCooldown  *float64
+	ScaleOutCooldown *float64
+}
+
 // AutoscalingOpts holds configuration that's needed for Auto Scaling.
 type AutoscalingOpts struct {
-	MinCapacity  *int
-	MaxCapacity  *int
-	CPU          *float64
-	Memory       *float64
-	Requests     *float64
-	ResponseTime *float64
-	QueueDelay   *AutoscalingQueueDelayOpts
+	MinCapacity        *int
+	MaxCapacity        *int
+	CPU                *float64
+	Memory             *float64
+	Requests           *float64
+	ResponseTime       *float64
+	CPUCooldown        Cooldown
+	MemCooldown        Cooldown
+	ReqCooldown        Cooldown
+	RespTimeCooldown   Cooldown
+	QueueDelayCooldown Cooldown
+	QueueDelay         *AutoscalingQueueDelayOpts
 }
 
 // AliasesForHostedZone maps hosted zone IDs to aliases that belong to it.
@@ -424,8 +436,9 @@ type NetworkOpts struct {
 	SecurityGroups []string
 	AssignPublicIP string
 	// SubnetsType and SubnetIDs are mutually exclusive. They won't be set together.
-	SubnetsType string
-	SubnetIDs   []string
+	SubnetsType              string
+	SubnetIDs                []string
+	DenyDefaultSecurityGroup bool
 }
 
 // RuntimePlatformOpts holds configuration needed for Platform configuration.
@@ -459,14 +472,24 @@ func (p RuntimePlatformOpts) isEmpty() bool {
 	return p.OS == "" && p.Arch == ""
 }
 
+// S3ObjectLocation represents an object stored in an S3 bucket.
+type S3ObjectLocation struct {
+	Bucket string // Name of the bucket.
+	Key    string // Key of the object.
+}
+
 // WorkloadOpts holds optional data that can be provided to enable features in a workload stack template.
 type WorkloadOpts struct {
+	AppName            string
+	EnvName            string
+	WorkloadName       string
+	SerializedManifest string // Raw manifest file used to deploy the workload.
+
 	// Additional options that are common between **all** workload templates.
 	Variables                map[string]string
 	Secrets                  map[string]Secret
 	Aliases                  []string
 	HTTPSListener            bool
-	UseImportedCerts         bool
 	Tags                     map[string]string        // Used by App Runner workloads to tag App Runner service resources
 	NestedStack              *WorkloadNestedStackOpts // Outputs from nested stacks such as the addons stack.
 	AddonsExtraParams        string                   // Additional user defined Parameters for the addons stack.
@@ -489,6 +512,7 @@ type WorkloadOpts struct {
 	HTTPVersion              *string
 	ALBEnabled               bool
 	HostedZoneAliases        AliasesForHostedZone
+	CredentialsParameter     string
 
 	// Additional options for service templates.
 	WorkloadType            string
@@ -499,14 +523,8 @@ type WorkloadOpts struct {
 	NLB                     *NetworkLoadBalancer
 	DeploymentConfiguration DeploymentConfigurationOpts
 
-	// Lambda functions.
-	RulePriorityLambda             string
-	DesiredCountLambda             string
-	EnvControllerLambda            string
-	CredentialsParameter           string
-	BacklogPerTaskCalculatorLambda string
-	NLBCertValidatorFunctionLambda string
-	NLBCustomDomainFunctionLambda  string
+	// Custom Resources backed by Lambda functions.
+	CustomResources map[string]S3ObjectLocation
 
 	// Additional options for job templates.
 	ScheduleExpression string
@@ -519,8 +537,6 @@ type WorkloadOpts struct {
 
 	// Input needed for the custom resource that adds a custom domain to the service.
 	Alias                *string
-	ScriptBucketName     *string
-	CustomDomainLambda   *string
 	AWSSDKLayer          *string
 	AppDNSDelegationRole *string
 	AppDNSName           *string
@@ -595,6 +611,7 @@ func withSvcParsingFuncs() ParseOption {
 			"hasSecrets":           hasSecrets,
 			"fmtSlice":             FmtSliceFunc,
 			"quoteSlice":           QuoteSliceFunc,
+			"quote":                strconv.Quote,
 			"randomUUID":           randomUUIDFunc,
 			"jsonMountPoints":      generateMountPointJSON,
 			"jsonSNSTopics":        generateSNSJSON,
